@@ -336,6 +336,7 @@ class Token:
 
     typeScopeId = None
     typeScope = None
+    type = None
 
     astParentId = None
     astParent = None
@@ -355,27 +356,27 @@ class Token:
         self.previous = None
         self.scopeId = element.get('scope')
         self.scope = None
-        type = element.get('type')
-        if type == 'name':
+        self.type = element.get('type')
+        if self.type == 'name':
             self.isName = True
             if element.get('isUnsigned'):
                 self.isUnsigned = True
             if element.get('isSigned'):
                 self.isSigned = True
-        elif type == 'number':
+        elif self.type == 'number':
             self.isNumber = True
             if element.get('isInt'):
                 self.isInt = True
             elif element.get('isFloat'):
                 self.isFloat = True
-        elif type == 'string':
+        elif self.type == 'string':
             self.isString = True
             self.strlen = int(element.get('strlen'))
-        elif type == 'char':
+        elif self.type == 'char':
             self.isChar = True
-        elif type == 'boolean':
+        elif self.type == 'boolean':
             self.isBoolean = True
-        elif type == 'op':
+        elif self.type == 'op':
             self.isOp = True
             if element.get('isArithmeticalOp'):
                 self.isArithmeticalOp = True
@@ -676,7 +677,7 @@ class Function:
         self.argumentId = {}
 
     def __repr__(self):
-        attrs = ["Id", "tokenId", "tokenDefId", "name", "type", "hasVirtualSpecifier", 
+        attrs = ["Id", "tokenId", "tokenDefId", "name", "type", "hasVirtualSpecifier",
                  "isImplicitlyVirtual", "access", "isInlineKeyword", "isStatic", 
                  "isAttributeNoreturn", "overriddenFunction", "nestedIn", "argumentId"]
         return "{}({})".format(
@@ -900,7 +901,7 @@ class Value:
         self.symbolic = IdMap.get(self._symbolicId)
 
     def __repr__(self):
-        attrs = ["intvalue", "tokvalue", "floatvalue", "movedvalue", "uninit", 
+        attrs = ["intvalue", "tokvalue", "floatvalue", "movedvalue", "uninit",
                  "bufferSize", "containerSize", "condition", "valueKind"]
         return "{}({})".format(
             "Value",
@@ -945,29 +946,60 @@ class Suppression:
       fileName    The name of the file to suppress warnings for, can include wildcards
       lineNumber  The number of the line to suppress warnings from, can be 0 to represent any line
       symbolName  The name of the symbol to match warnings for, can include wildcards
+      lineBegin   The first line to suppress warnings from
+      lineEnd     The last line to suppress warnings from
+      suppressionType   The type of suppression which is applied (unique = None (default), file, block, blockBegin, blockEnd, macro)
     """
 
     errorId = None
     fileName = None
     lineNumber = None
     symbolName = None
+    lineBegin = None
+    lineEnd = None
+    suppressionType = None
 
     def __init__(self, element):
         self.errorId = element.get('errorId')
         self.fileName = element.get('fileName')
         self.lineNumber = element.get('lineNumber')
         self.symbolName = element.get('symbolName')
+        self.lineBegin = element.get('lineBegin')
+        self.lineEnd = element.get('lineEnd')
+        self.suppressionType = element.get('type')
 
     def __repr__(self):
-        attrs = ['errorId' , "fileName", "lineNumber", "symbolName"]
+        attrs = ["errorId", "fileName", "lineNumber", "symbolName", "lineBegin", "lineEnd","suppressionType"]
         return "{}({})".format(
             "Suppression",
             ", ".join(("{}={}".format(a, repr(getattr(self, a))) for a in attrs))
         )
 
     def isMatch(self, file, line, message, errorId):
+        # Line Suppression
         if ((self.fileName is None or fnmatch(file, self.fileName))
-                and (self.lineNumber is None or int(line) == int(self.lineNumber))
+                and (self.suppressionType is None) # Verify use of default suppression type (None = unique)
+                and (self.lineNumber is not None and int(line) == int(self.lineNumber))
+                and (self.symbolName is None or fnmatch(message, '*'+self.symbolName+'*'))
+                and fnmatch(errorId, self.errorId)):
+            return True
+        # File Suppression
+        if ((self.fileName is None or fnmatch(file, self.fileName))
+                and (self.suppressionType is not None and self.suppressionType == "file") # Verify use of file (global) suppression type
+                and (self.symbolName is None or fnmatch(message, '*'+self.symbolName+'*'))
+                and fnmatch(errorId, self.errorId)):
+            return True
+        # Block Suppression Mode
+        if ((self.fileName is None or fnmatch(file, self.fileName))
+                and (self.suppressionType is not None and self.suppressionType == "block") # Type for Block suppression
+                and (self.lineBegin is not None and int(line) > int(self.lineBegin)) # Code Match is between the Block suppression
+                and (self.lineEnd is not None and int(line) < int(self.lineEnd)) # Code Match is between the Block suppression
+                and (self.symbolName is None or fnmatch(message, '*'+self.symbolName+'*'))
+                and fnmatch(errorId, self.errorId)):
+            return True
+        # Other Suppression (Globaly set via suppression file or cli command)
+        if ((self.fileName is None or fnmatch(file, self.fileName))
+                and (self.suppressionType is None)
                 and (self.symbolName is None or fnmatch(message, '*'+self.symbolName+'*'))
                 and fnmatch(errorId, self.errorId)):
             return True
@@ -1249,6 +1281,9 @@ class CppcheckData:
         # Iterating <typedef-info>
         iter_typedef_info = False
 
+        # Iterating <directive>
+        iter_directive = False
+
         # Use iterable objects to traverse XML tree for dump files incrementally.
         # Iterative approach is required to avoid large memory consumption.
         # Calling .clear() is necessary to let the element be garbage collected.
@@ -1280,8 +1315,12 @@ class CppcheckData:
                 cfg.standards.set_posix(node)
 
             # Parse directives list
-            elif node.tag == 'directive' and event == 'start':
-                cfg.directives.append(Directive(node))
+            elif node.tag == 'directive':
+                if event == 'start':
+                    cfg.directives.append(Directive(node))
+                    iter_directive = True
+                elif event == 'end':
+                    iter_directive = False
             # Parse macro usage
             elif node.tag == 'macro' and event == 'start':
                 cfg.macro_usage.append(MacroUsage(node))
@@ -1293,7 +1332,7 @@ class CppcheckData:
             # Parse tokens
             elif node.tag == 'tokenlist' and event == 'start':
                 continue
-            elif node.tag == 'token' and event == 'start':
+            elif node.tag == 'token' and event == 'start' and not iter_directive:
                 cfg.tokenlist.append(Token(node))
 
             # Parse scopes
@@ -1619,11 +1658,19 @@ def is_suppressed(location, message, errorId):
             return True
     return False
 
-def reportError(location, severity, message, addon, errorId, extra=''):
+def log_checker(message, addon):
+    if '--cli' in sys.argv:
+        msg = { 'addon': addon,
+                'severity': 'none',
+                'message': message,
+                'errorId': 'logChecker'}
+        sys.stdout.write(json.dumps(msg) + '\n')
+
+def reportError(location, severity, message, addon, errorId, extra='', columnOverride=None):
     if '--cli' in sys.argv:
         msg = { 'file': location.file,
                 'linenr': location.linenr,
-                'column': location.column,
+                'column': location.column if columnOverride is None else columnOverride,
                 'severity': severity,
                 'message': message,
                 'addon': addon,
