@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,13 +31,11 @@
 #include "utils.h"
 
 #include <algorithm>
-#include <cstddef>
+#include <fstream>
 #include <iterator>
 #include <list>
-#include <ostream>
 #include <set>
 #include <string>
-#include <utility>
 #include <vector>
 
 #include <QByteArray>
@@ -56,8 +54,14 @@
 #include <QCharRef>
 #endif
 
+static QString unquote(QString s) {
+    if (s.startsWith("\""))
+        s = s.mid(1, s.size() - 2);
+    return s;
+}
+
 // NOLINTNEXTLINE(performance-unnecessary-value-param) - used as callback so we need to preserve the signature
-static int executeCommand(std::string exe, std::vector<std::string> args, std::string redirect, std::string &output) // cppcheck-suppress passedByValueCallback
+int CheckThread::executeCommand(std::string exe, std::vector<std::string> args, std::string redirect, std::string &output) // cppcheck-suppress passedByValue
 {
     output.clear();
 
@@ -66,7 +70,24 @@ static int executeCommand(std::string exe, std::vector<std::string> args, std::s
         args2 << QString::fromStdString(arg);
 
     QProcess process;
-    process.start(QString::fromStdString(exe), args2);
+
+    QString e = unquote(QString::fromStdString(exe));
+
+    if (e.toLower().replace("\\", "/").endsWith("/python.exe") && !args.empty()) {
+        const QString path = e.left(e.size()-11);
+        QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
+        env.insert("PYTHONPATH", path + "/Lib/site-packages");
+        env.insert("PYTHONHOME", path);
+        process.setProcessEnvironment(env);
+
+        const QString pythonScript = unquote(args2[0]);
+        if (pythonScript.endsWith(".py")) {
+            const QString path2 = pythonScript.left(QString(pythonScript).replace('\\', '/').lastIndexOf("/"));
+            process.setWorkingDirectory(path2);
+        }
+    }
+
+    process.start(e, args2);
     process.waitForFinished();
 
     if (redirect == "2>&1") {
@@ -111,9 +132,9 @@ void CheckThread::run()
     if (!mFiles.isEmpty() || mAnalyseWholeProgram) {
         mAnalyseWholeProgram = false;
         qDebug() << "Whole program analysis";
-        std::list<std::pair<std::string, std::size_t>> files2;
+        std::list<FileWithDetails> files2;
         std::transform(mFiles.cbegin(), mFiles.cend(), std::back_inserter(files2), [&](const QString& file) {
-            return std::pair<std::string, std::size_t>{file.toStdString(), 0};
+            return FileWithDetails{file.toStdString(), 0};
         });
         mCppcheck.analyseWholeProgram(mCppcheck.settings().buildDir, files2, {});
         mFiles.clear();
@@ -124,7 +145,7 @@ void CheckThread::run()
     QString file = mResult.getNextFile();
     while (!file.isEmpty() && mState == Running) {
         qDebug() << "Checking file" << file;
-        mCppcheck.check(file.toStdString());
+        mCppcheck.check(FileWithDetails(file.toStdString()));
         runAddonsAndTools(nullptr, file);
         emit fileChecked(file);
 
@@ -132,16 +153,17 @@ void CheckThread::run()
             file = mResult.getNextFile();
     }
 
-    FileSettings fileSettings = mResult.getNextFileSettings();
-    while (!fileSettings.filename.empty() && mState == Running) {
-        file = QString::fromStdString(fileSettings.filename);
+    const FileSettings* fileSettings = nullptr;
+    mResult.getNextFileSettings(fileSettings);
+    while (fileSettings && mState == Running) {
+        file = QString::fromStdString(fileSettings->filename());
         qDebug() << "Checking file" << file;
-        mCppcheck.check(fileSettings);
-        runAddonsAndTools(&fileSettings, QString::fromStdString(fileSettings.filename));
+        mCppcheck.check(*fileSettings);
+        runAddonsAndTools(fileSettings, QString::fromStdString(fileSettings->filename()));
         emit fileChecked(file);
 
         if (mState == Running)
-            fileSettings = mResult.getNextFileSettings();
+            mResult.getNextFileSettings(fileSettings);
     }
 
     if (mState == Running)
@@ -214,7 +236,7 @@ void CheckThread::runAddonsAndTools(const FileSettings *fileSettings, const QStr
 
             const std::string &buildDir = mCppcheck.settings().buildDir;
             if (!buildDir.empty()) {
-                analyzerInfoFile = QString::fromStdString(AnalyzerInformation::getAnalyzerInfoFile(buildDir, fileSettings->filename, fileSettings->cfg));
+                analyzerInfoFile = QString::fromStdString(AnalyzerInformation::getAnalyzerInfoFile(buildDir, fileSettings->filename(), fileSettings->cfg));
 
                 QStringList args2(args);
                 args2.insert(0,"-E");
@@ -389,7 +411,7 @@ void CheckThread::parseClangErrors(const QString &tool, const QString &file0, QS
     for (const ErrorItem &e : errorItems) {
         if (e.errorPath.isEmpty())
             continue;
-        Suppressions::ErrorMessage errorMessage;
+        SuppressionList::ErrorMessage errorMessage;
         errorMessage.setFileName(e.errorPath.back().file.toStdString());
         errorMessage.lineNumber = e.errorPath.back().line;
         errorMessage.errorId = e.errorId.toStdString();
@@ -410,9 +432,9 @@ void CheckThread::parseClangErrors(const QString &tool, const QString &file0, QS
     }
 }
 
-bool CheckThread::isSuppressed(const Suppressions::ErrorMessage &errorMessage) const
+bool CheckThread::isSuppressed(const SuppressionList::ErrorMessage &errorMessage) const
 {
-    return std::any_of(mSuppressions.cbegin(), mSuppressions.cend(), [&](const Suppressions::Suppression& s) {
+    return std::any_of(mSuppressions.cbegin(), mSuppressions.cend(), [&](const SuppressionList::Suppression& s) {
         return s.isSuppressed(errorMessage);
     });
 }
