@@ -22,6 +22,7 @@
 #include "applicationlist.h"
 #include "aboutdialog.h"
 #include "analyzerinfo.h"
+#include "checkstatistics.h"
 #include "checkthread.h"
 #include "common.h"
 #include "cppcheck.h"
@@ -118,7 +119,8 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
     mPlatformActions(new QActionGroup(this)),
     mCStandardActions(new QActionGroup(this)),
     mCppStandardActions(new QActionGroup(this)),
-    mSelectLanguageActions(new QActionGroup(this))
+    mSelectLanguageActions(new QActionGroup(this)),
+    mSelectReportActions(new QActionGroup(this))
 {
     {
         Settings tempSettings;
@@ -199,6 +201,15 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
     connect(mUI->mResults, &ResultsView::suppressIds, this, &MainWindow::suppressIds);
     connect(mUI->mMenuView, &QMenu::aboutToShow, this, &MainWindow::aboutToShowViewMenu);
 
+    // Change report type
+    connect(mUI->mActionReportNormal, &QAction::triggered, this, &MainWindow::changeReportType);
+    connect(mUI->mActionReportAutosar, &QAction::triggered, this, &MainWindow::changeReportType);
+    connect(mUI->mActionReportCertC, &QAction::triggered, this, &MainWindow::changeReportType);
+    connect(mUI->mActionReportCertCpp, &QAction::triggered, this, &MainWindow::changeReportType);
+    connect(mUI->mActionReportMisraC, &QAction::triggered, this, &MainWindow::changeReportType);
+    connect(mUI->mActionReportMisraCpp2008, &QAction::triggered, this, &MainWindow::changeReportType);
+    connect(mUI->mActionReportMisraCpp2023, &QAction::triggered, this, &MainWindow::changeReportType);
+
     // File menu
     connect(mUI->mActionNewProjectFile, &QAction::triggered, this, &MainWindow::newProjectFile);
     connect(mUI->mActionOpenProjectFile, &QAction::triggered, this, &MainWindow::openProjectFile);
@@ -260,9 +271,19 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
         connect(action, SIGNAL(triggered()), this, SLOT(selectPlatform()));
     }
 
+    mUI->mActionReportNormal->setActionGroup(mSelectReportActions);
+    mUI->mActionReportAutosar->setActionGroup(mSelectReportActions);
+    mUI->mActionReportCertC->setActionGroup(mSelectReportActions);
+    mUI->mActionReportCertCpp->setActionGroup(mSelectReportActions);
+    mUI->mActionReportMisraC->setActionGroup(mSelectReportActions);
+    mUI->mActionReportMisraCpp2008->setActionGroup(mSelectReportActions);
+    mUI->mActionReportMisraCpp2023->setActionGroup(mSelectReportActions);
+
     mUI->mActionC89->setActionGroup(mCStandardActions);
     mUI->mActionC99->setActionGroup(mCStandardActions);
     mUI->mActionC11->setActionGroup(mCStandardActions);
+    //mUI->mActionC17->setActionGroup(mCStandardActions);
+    //mUI->mActionC23->setActionGroup(mCStandardActions);
 
     mUI->mActionCpp03->setActionGroup(mCppStandardActions);
     mUI->mActionCpp11->setActionGroup(mCppStandardActions);
@@ -270,6 +291,7 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
     mUI->mActionCpp17->setActionGroup(mCppStandardActions);
     mUI->mActionCpp20->setActionGroup(mCppStandardActions);
     //mUI->mActionCpp23->setActionGroup(mCppStandardActions);
+    //mUI->mActionCpp26->setActionGroup(mCppStandardActions);
 
     mUI->mActionEnforceC->setActionGroup(mSelectLanguageActions);
     mUI->mActionEnforceCpp->setActionGroup(mSelectLanguageActions);
@@ -308,6 +330,8 @@ MainWindow::MainWindow(TranslationHandler* th, QSettings* settings) :
     } else {
         delete mUI->mLayoutInformation;
     }
+
+    changeReportType();
 }
 
 MainWindow::~MainWindow()
@@ -358,6 +382,15 @@ void MainWindow::loadSettings()
                mSettings->value(SETTINGS_WINDOW_HEIGHT, 600).toInt());
     }
 
+    const ReportType reportType = (ReportType)mSettings->value(SETTINGS_REPORT_TYPE, (int)ReportType::normal).toInt();
+    mUI->mActionReportNormal->setChecked(reportType <= ReportType::normal);
+    mUI->mActionReportAutosar->setChecked(reportType == ReportType::autosar);
+    mUI->mActionReportCertC->setChecked(reportType == ReportType::certC);
+    mUI->mActionReportCertCpp->setChecked(reportType == ReportType::certCpp);
+    mUI->mActionReportMisraC->setChecked(reportType == ReportType::misraC);
+    mUI->mActionReportMisraCpp2008->setChecked(reportType == ReportType::misraCpp2008);
+    mUI->mActionReportMisraCpp2023->setChecked(reportType == ReportType::misraCpp2023);
+
     const ShowTypes &types = mUI->mResults->getShowTypes();
     mUI->mActionShowStyle->setChecked(types.isShown(ShowTypes::ShowStyle));
     mUI->mActionShowErrors->setChecked(types.isShown(ShowTypes::ShowErrors));
@@ -373,6 +406,8 @@ void MainWindow::loadSettings()
     mUI->mActionC89->setChecked(standards.c == Standards::C89);
     mUI->mActionC99->setChecked(standards.c == Standards::C99);
     mUI->mActionC11->setChecked(standards.c == Standards::C11);
+    //mUI->mActionC17->setChecked(standards.c == Standards::C17);
+    //mUI->mActionC23->setChecked(standards.c == Standards::C23);
     standards.setCPP(mSettings->value(SETTINGS_STD_CPP, QString()).toString().toStdString());
     mUI->mActionCpp03->setChecked(standards.cpp == Standards::CPP03);
     mUI->mActionCpp11->setChecked(standards.cpp == Standards::CPP11);
@@ -380,6 +415,7 @@ void MainWindow::loadSettings()
     mUI->mActionCpp17->setChecked(standards.cpp == Standards::CPP17);
     mUI->mActionCpp20->setChecked(standards.cpp == Standards::CPP20);
     //mUI->mActionCpp23->setChecked(standards.cpp == Standards::CPP23);
+    //mUI->mActionCpp26->setChecked(standards.cpp == Standards::CPP26);
 
     // Main window settings
     const bool showMainToolbar = mSettings->value(SETTINGS_TOOLBARS_MAIN_SHOW, true).toBool();
@@ -437,6 +473,15 @@ void MainWindow::saveSettings() const
     mSettings->setValue(SETTINGS_WINDOW_HEIGHT, size().height());
     mSettings->setValue(SETTINGS_WINDOW_MAXIMIZED, isMaximized());
 
+    const ReportType reportType = mUI->mActionReportAutosar->isChecked() ? ReportType::autosar :
+                                  mUI->mActionReportCertC->isChecked() ? ReportType::certC :
+                                  mUI->mActionReportCertCpp->isChecked() ? ReportType::certCpp :
+                                  mUI->mActionReportMisraC->isChecked() ? ReportType::misraC :
+                                  mUI->mActionReportMisraCpp2008->isChecked() ? ReportType::misraCpp2008 :
+                                  mUI->mActionReportMisraCpp2023->isChecked() ? ReportType::misraCpp2023 :
+                                  ReportType::normal;
+    mSettings->setValue(SETTINGS_REPORT_TYPE, (int)reportType);
+
     // Show * states
     mSettings->setValue(SETTINGS_SHOW_STYLE, mUI->mActionShowStyle->isChecked());
     mSettings->setValue(SETTINGS_SHOW_ERRORS, mUI->mActionShowErrors->isChecked());
@@ -451,6 +496,10 @@ void MainWindow::saveSettings() const
         mSettings->setValue(SETTINGS_STD_C, "C99");
     if (mUI->mActionC11->isChecked())
         mSettings->setValue(SETTINGS_STD_C, "C11");
+    //if (mUI->mActionC17->isChecked())
+    //    mSettings->setValue(SETTINGS_STD_C, "C17");
+    //if (mUI->mActionC23->isChecked())
+    //    mSettings->setValue(SETTINGS_STD_C, "C23");
 
     if (mUI->mActionCpp03->isChecked())
         mSettings->setValue(SETTINGS_STD_CPP, "C++03");
@@ -464,6 +513,8 @@ void MainWindow::saveSettings() const
         mSettings->setValue(SETTINGS_STD_CPP, "C++20");
     //if (mUI.mActionCpp23->isChecked())
     //    mSettings->setValue(SETTINGS_STD_CPP, "C++23");
+    //if (mUI.mActionCpp26->isChecked())
+    //    mSettings->setValue(SETTINGS_STD_CPP, "C++26");
 
     // Main window settings
     mSettings->setValue(SETTINGS_TOOLBARS_MAIN_SHOW, mUI->mToolBarMain->isVisible());
@@ -649,7 +700,7 @@ void MainWindow::analyzeCode(const QString& code, const QString& filename)
     checkLockDownUI();
     clearResults();
     mUI->mResults->checkingStarted(1);
-    cppcheck.check(filename.toStdString(), code.toStdString());
+    cppcheck.check(FileWithDetails(filename.toStdString()), code.toStdString());
     analysisDone();
 
     // Expand results
@@ -830,17 +881,23 @@ Library::Error MainWindow::loadLibrary(Library &library, const QString &filename
     // Try to load the library from the project folder..
     if (mProjectFile) {
         QString path = QFileInfo(mProjectFile->getFilename()).canonicalPath();
-        ret = library.load(nullptr, (path+"/"+filename).toLatin1());
+        QString libpath = path+"/"+filename;
+        qDebug().noquote() << "looking for library '" + libpath + "'";
+        ret = library.load(nullptr, libpath.toLatin1());
         if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
             return ret;
     }
 
     // Try to load the library from the application folder..
     const QString appPath = QFileInfo(QCoreApplication::applicationFilePath()).canonicalPath();
-    ret = library.load(nullptr, (appPath+"/"+filename).toLatin1());
+    QString libpath = appPath+"/"+filename;
+    qDebug().noquote() << "looking for library '" + libpath + "'";
+    ret = library.load(nullptr, libpath.toLatin1());
     if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
         return ret;
-    ret = library.load(nullptr, (appPath+"/cfg/"+filename).toLatin1());
+    libpath = appPath+"/cfg/"+filename;
+    qDebug().noquote() << "looking for library '" + libpath + "'";
+    ret = library.load(nullptr, libpath.toLatin1());
     if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
         return ret;
 
@@ -848,10 +905,14 @@ Library::Error MainWindow::loadLibrary(Library &library, const QString &filename
     // Try to load the library from FILESDIR/cfg..
     const QString filesdir = FILESDIR;
     if (!filesdir.isEmpty()) {
-        ret = library.load(nullptr, (filesdir+"/cfg/"+filename).toLatin1());
+        libpath = filesdir+"/cfg/"+filename;
+        qDebug().noquote() << "looking for library '" + libpath + "'";
+        ret = library.load(nullptr, libpath.toLatin1());
         if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
             return ret;
-        ret = library.load(nullptr, (filesdir+filename).toLatin1());
+        libpath = filesdir+"/"+filename;
+        qDebug().noquote() << "looking for library '" + libpath + "'";
+        ret = library.load(nullptr, libpath.toLatin1());
         if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
             return ret;
     }
@@ -860,14 +921,20 @@ Library::Error MainWindow::loadLibrary(Library &library, const QString &filename
     // Try to load the library from the cfg subfolder..
     const QString datadir = getDataDir();
     if (!datadir.isEmpty()) {
-        ret = library.load(nullptr, (datadir+"/"+filename).toLatin1());
+        libpath = datadir+"/"+filename;
+        qDebug().noquote() << "looking for library '" + libpath + "'";
+        ret = library.load(nullptr, libpath.toLatin1());
         if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
             return ret;
-        ret = library.load(nullptr, (datadir+"/cfg/"+filename).toLatin1());
+        libpath = datadir+"/cfg/"+filename;
+        qDebug().noquote() << "looking for library '" + libpath + "'";
+        ret = library.load(nullptr, libpath.toLatin1());
         if (ret.errorcode != Library::ErrorCode::FILE_NOT_FOUND)
             return ret;
     }
 
+    qDebug().noquote() << "library not found: '" + filename + "'";
+
     return ret;
 }
 
@@ -939,11 +1006,13 @@ QString MainWindow::loadAddon(Settings &settings, const QString &filesDir, const
         const QString misraFile = fromNativePath(mSettings->value(SETTINGS_MISRA_FILE).toString());
         if (!misraFile.isEmpty()) {
             QString arg;
+            picojson::array arr;
             if (misraFile.endsWith(".pdf", Qt::CaseInsensitive))
                 arg = "--misra-pdf=" + misraFile;
             else
                 arg = "--rule-texts=" + misraFile;
-            obj["args"] = picojson::value(arg.toStdString());
+            arr.emplace_back(arg.toStdString());
+            obj["args"] = picojson::value(arr);
         }
     }
 
@@ -1005,6 +1074,8 @@ QPair<bool,Settings> MainWindow::getCppcheckSettings()
         QStringList dirs = mProjectFile->getIncludeDirs();
         addIncludeDirs(dirs, result);
 
+        result.inlineSuppressions = mProjectFile->getInlineSuppression();
+
         const QStringList defines = mProjectFile->getDefines();
         for (const QString& define : defines) {
             if (!result.userDefines.empty())
@@ -1098,6 +1169,8 @@ QPair<bool,Settings> MainWindow::getCppcheckSettings()
             result.setMisraRuleTexts(CheckThread::executeCommand);
         }
     }
+    else
+        result.inlineSuppressions = mSettings->value(SETTINGS_INLINE_SUPPRESSIONS, false).toBool();
 
     // Include directories (and files) are searched in listed order.
     // Global include directories must be added AFTER the per project include
@@ -1122,7 +1195,6 @@ QPair<bool,Settings> MainWindow::getCppcheckSettings()
     result.force = mSettings->value(SETTINGS_CHECK_FORCE, 1).toBool();
     result.xml = false;
     result.jobs = mSettings->value(SETTINGS_CHECK_THREADS, 1).toInt();
-    result.inlineSuppressions = mSettings->value(SETTINGS_INLINE_SUPPRESSIONS, false).toBool();
     result.certainty.setEnabled(Certainty::inconclusive, mSettings->value(SETTINGS_INCONCLUSIVE_ERRORS, false).toBool());
     if (!mProjectFile || result.platform.type == Platform::Type::Unspecified)
         result.platform.set((Platform::Type) mSettings->value(SETTINGS_CHECKED_PLATFORM, 0).toInt());
@@ -1130,9 +1202,7 @@ QPair<bool,Settings> MainWindow::getCppcheckSettings()
     result.standards.setC(mSettings->value(SETTINGS_STD_C, QString()).toString().toStdString());
     result.enforcedLang = (Standards::Language)mSettings->value(SETTINGS_ENFORCED_LANGUAGE, 0).toInt();
 
-    if (result.jobs <= 1) {
-        result.jobs = 1;
-    }
+    result.jobs = std::max(result.jobs, 1u);
 
     Settings::terminate(false);
 
@@ -1236,7 +1306,7 @@ void MainWindow::reAnalyzeModified()
 void MainWindow::reAnalyzeAll()
 {
     if (mProjectFile)
-        analyzeProject(mProjectFile);
+        analyzeProject(mProjectFile, QStringList());
     else
         reAnalyze(true);
 }
@@ -1244,13 +1314,13 @@ void MainWindow::reAnalyzeAll()
 void MainWindow::checkLibrary()
 {
     if (mProjectFile)
-        analyzeProject(mProjectFile, true);
+        analyzeProject(mProjectFile, QStringList(), true);
 }
 
 void MainWindow::checkConfiguration()
 {
     if (mProjectFile)
-        analyzeProject(mProjectFile, false, true);
+        analyzeProject(mProjectFile, QStringList(), false, true);
 }
 
 void MainWindow::reAnalyzeSelected(const QStringList& files)
@@ -1260,6 +1330,16 @@ void MainWindow::reAnalyzeSelected(const QStringList& files)
     if (mThread->isChecking())
         return;
 
+    if (mProjectFile) {
+        // Clear details, statistics and progress
+        mUI->mResults->clear(false);
+        for (int i = 0; i < files.size(); ++i)
+            mUI->mResults->clearRecheckFile(files[i]);
+
+        analyzeProject(mProjectFile, files);
+        return;
+    }
+
     const QPair<bool, Settings> checkSettingsPair = getCppcheckSettings();
     if (!checkSettingsPair.first)
         return;
@@ -1394,7 +1474,7 @@ void MainWindow::enableCheckButtons(bool enable)
     mUI->mActionAnalyzeFiles->setEnabled(enable);
 
     if (mProjectFile) {
-        mUI->mActionReanalyzeModified->setEnabled(false);
+        mUI->mActionReanalyzeModified->setEnabled(enable);
         mUI->mActionReanalyzeAll->setEnabled(enable);
     } else if (!enable || mThread->hasPreviousFiles()) {
         mUI->mActionReanalyzeModified->setEnabled(enable);
@@ -1593,7 +1673,7 @@ void MainWindow::complianceReport()
 
     mUI->mResults->save(tempResults.fileName(), Report::XMLV2, mCppcheckCfgProductName);
 
-    ComplianceReportDialog dlg(mProjectFile, tempResults.fileName());
+    ComplianceReportDialog dlg(mProjectFile, tempResults.fileName(), mUI->mResults->getStatistics()->getCheckersReport());
     dlg.exec();
 }
 
@@ -1725,7 +1805,7 @@ void MainWindow::loadProjectFile(const QString &filePath)
     mProjectFile = new ProjectFile(filePath, this);
     mProjectFile->setActiveProject();
     if (!loadLastResults())
-        analyzeProject(mProjectFile);
+        analyzeProject(mProjectFile, QStringList());
 }
 
 QString MainWindow::getLastResults() const
@@ -1742,6 +1822,7 @@ bool MainWindow::loadLastResults()
         return false;
     if (!QFileInfo::exists(lastResults))
         return false;
+    mUI->mResults->clear(true);
     mUI->mResults->readErrorsXml(lastResults);
     mUI->mResults->setCheckDirectory(mSettings->value(SETTINGS_LAST_CHECK_PATH,QString()).toString());
     mUI->mActionViewStats->setEnabled(true);
@@ -1749,7 +1830,7 @@ bool MainWindow::loadLastResults()
     return true;
 }
 
-void MainWindow::analyzeProject(const ProjectFile *projectFile, const bool checkLibrary, const bool checkConfiguration)
+void MainWindow::analyzeProject(const ProjectFile *projectFile, const QStringList& recheckFiles, const bool checkLibrary, const bool checkConfiguration)
 {
     Settings::terminate(false);
 
@@ -1851,7 +1932,7 @@ void MainWindow::analyzeProject(const ProjectFile *projectFile, const bool check
         return;
     }
 
-    QStringList paths = projectFile->getCheckPaths();
+    QStringList paths = recheckFiles.isEmpty() ? projectFile->getCheckPaths() : recheckFiles;
 
     // If paths not given then check the root path (which may be the project
     // file's location, see above). This is to keep the compatibility with
@@ -1892,7 +1973,7 @@ void MainWindow::newProjectFile()
     ProjectFileDialog dlg(mProjectFile, isCppcheckPremium(), this);
     if (dlg.exec() == QDialog::Accepted) {
         addProjectMRU(filepath);
-        analyzeProject(mProjectFile);
+        analyzeProject(mProjectFile, QStringList());
     } else {
         closeProjectFile();
     }
@@ -1923,7 +2004,7 @@ void MainWindow::editProjectFile()
     ProjectFileDialog dlg(mProjectFile, isCppcheckPremium(), this);
     if (dlg.exec() == QDialog::Accepted) {
         mProjectFile->write();
-        analyzeProject(mProjectFile);
+        analyzeProject(mProjectFile, QStringList());
     }
 }
 
@@ -2099,7 +2180,7 @@ static int getVersion(const QString& nameWithVersion) {
             break;
         if (c == ' ') {
             if (ret > 0 && dot == 1 && nameWithVersion.endsWith(" dev"))
-                return ret * 1000000 + v * 1000 + 500;
+                return (ret * 1000000) + (v * 1000) + 500;
             dot = ret = v = 0;
         }
         else if (c == '.') {
@@ -2166,3 +2247,56 @@ bool MainWindow::isCppcheckPremium() const {
     return mCppcheckCfgProductName.startsWith("Cppcheck Premium ");
 }
 
+void MainWindow::changeReportType() {
+    const ReportType reportType = mUI->mActionReportAutosar->isChecked() ? ReportType::autosar :
+                                  mUI->mActionReportCertC->isChecked() ? ReportType::certC :
+                                  mUI->mActionReportCertCpp->isChecked() ? ReportType::certCpp :
+                                  mUI->mActionReportMisraC->isChecked() ? ReportType::misraC :
+                                  mUI->mActionReportMisraCpp2008->isChecked() ? ReportType::misraCpp2008 :
+                                  mUI->mActionReportMisraCpp2023->isChecked() ? ReportType::misraCpp2023 :
+                                  ReportType::normal;
+
+    mUI->mResults->setReportType(reportType);
+
+    auto setTextAndHint = [](QAction* a, const QString& s) {
+        a->setVisible(!s.isEmpty());
+        a->setText(s);
+        a->setToolTip(s);
+    };
+
+    const QString showMandatory = tr("Show Mandatory");
+    const QString showRequired = tr("Show Required");
+    const QString showAdvisory = tr("Show Advisory");
+    const QString showDocument = tr("Show Document");
+
+    if (mUI->mActionReportAutosar->isChecked()) {
+        setTextAndHint(mUI->mActionShowErrors, "");
+        setTextAndHint(mUI->mActionShowWarnings, showRequired);
+        setTextAndHint(mUI->mActionShowStyle, showAdvisory);
+        setTextAndHint(mUI->mActionShowPortability, "");
+        setTextAndHint(mUI->mActionShowPerformance, "");
+        setTextAndHint(mUI->mActionShowInformation, "");
+    } else if (mUI->mActionReportMisraC->isChecked() || mUI->mActionReportMisraCpp2008->isChecked() || mUI->mActionReportMisraCpp2023->isChecked()) {
+        setTextAndHint(mUI->mActionShowErrors, mUI->mActionReportMisraCpp2008->isChecked() ? "" : showMandatory);
+        setTextAndHint(mUI->mActionShowWarnings, showRequired);
+        setTextAndHint(mUI->mActionShowStyle, showAdvisory);
+        setTextAndHint(mUI->mActionShowPortability, "");
+        setTextAndHint(mUI->mActionShowPerformance, "");
+        setTextAndHint(mUI->mActionShowInformation, mUI->mActionReportMisraCpp2008->isChecked() ? showDocument : QString());
+    } else if (mUI->mActionReportCertC->isChecked() || mUI->mActionReportCertCpp->isChecked()) {
+        setTextAndHint(mUI->mActionShowErrors, tr("Show L1"));
+        setTextAndHint(mUI->mActionShowWarnings, tr("Show L2"));
+        setTextAndHint(mUI->mActionShowStyle, tr("Show L3"));
+        setTextAndHint(mUI->mActionShowPortability, "");
+        setTextAndHint(mUI->mActionShowPerformance, "");
+        setTextAndHint(mUI->mActionShowInformation, "");
+    } else {
+        setTextAndHint(mUI->mActionShowErrors, tr("Show errors"));
+        setTextAndHint(mUI->mActionShowWarnings, tr("Show warnings"));
+        setTextAndHint(mUI->mActionShowStyle, tr("Show style"));
+        setTextAndHint(mUI->mActionShowPortability, tr("Show portability"));
+        setTextAndHint(mUI->mActionShowPerformance, tr("Show performance"));
+        setTextAndHint(mUI->mActionShowInformation, tr("Show information"));
+    }
+}
+
