@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #include "resultsview.h"
 #include "settings.h"
 
+#include <algorithm>
 #include <string>
 #include <unordered_set>
 #include <utility>
@@ -92,10 +93,7 @@ void ThreadHandler::check(const Settings &settings)
     setThreadCount(settings.jobs);
 
     mRunningThreadCount = mThreads.size();
-
-    if (mResults.getFileCount() < mRunningThreadCount) {
-        mRunningThreadCount = mResults.getFileCount();
-    }
+    mRunningThreadCount = std::min(mResults.getFileCount(), mRunningThreadCount);
 
     QStringList addonsAndTools = mAddonsAndTools;
     for (const std::string& addon: settings.addons) {
