@@ -1,4 +1,4 @@
-/*
+/* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
  * Copyright (C) 2007-2024 Cppcheck team.
  *
@@ -22,6 +22,7 @@
 #define astutilsH
 //---------------------------------------------------------------------------
 
+#include <cstdint>
 #include <functional>
 #include <list>
 #include <stack>
@@ -39,7 +40,7 @@
 
 class Settings;
 
-enum class ChildrenToVisit {
+enum class ChildrenToVisit : std::uint8_t {
     none,
     op1,
     op2,
@@ -110,11 +111,11 @@ const Token* findParent(const Token* tok, const TFunc& pred)
     return parent;
 }
 
-const Token* findExpression(const nonneg int exprid,
+const Token* findExpression(nonneg int exprid,
                             const Token* start,
                             const Token* end,
                             const std::function<bool(const Token*)>& pred);
-const Token* findExpression(const Token* start, const nonneg int exprid);
+const Token* findExpression(const Token* start, nonneg int exprid);
 
 template<class T, class OuputIterator, REQUIRES("T must be a Token class", std::is_convertible<T*, const Token*> )>
 void astFlattenCopy(T* tok, const char* op, OuputIterator out, nonneg int depth = 100)
@@ -212,7 +213,7 @@ const Token* getParentLifetime(const Token* tok);
 const Token* getParentLifetime(const Token* tok, const Library& library);
 
 std::vector<ValueType> getParentValueTypes(const Token* tok,
-                                           const Settings* settings = nullptr,
+                                           const Settings& settings,
                                            const Token** parent = nullptr);
 
 bool astIsLHS(const Token* tok);
@@ -251,6 +252,10 @@ bool succeeds(const Token* tok1, const Token* tok2);
 bool exprDependsOnThis(const Token* expr, bool onVar = true, nonneg int depth = 0);
 
 struct ReferenceToken {
+    ReferenceToken(const Token* t, ErrorPath e)
+        : token(t)
+        , errors(std::move(e))
+    {}
     const Token* token;
     ErrorPath errors;
 };
@@ -262,9 +267,9 @@ SmallVector<ReferenceToken> followAllReferences(const Token* tok,
                                                 int depth = 20);
 const Token* followReferences(const Token* tok, ErrorPath* errors = nullptr);
 
-CPPCHECKLIB bool isSameExpression(bool macro, const Token *tok1, const Token *tok2, const Library& library, bool pure, bool followVar, ErrorPath* errors=nullptr);
+CPPCHECKLIB bool isSameExpression(bool macro, const Token *tok1, const Token *tok2, const Settings& settings, bool pure, bool followVar, ErrorPath* errors=nullptr);
 
-bool isEqualKnownValue(const Token * const tok1, const Token * const tok2);
+bool isEqualKnownValue(const Token * tok1, const Token * tok2);
 
 bool isStructuredBindingVariable(const Variable* var);
 
@@ -273,7 +278,7 @@ const Token* isInLoopCondition(const Token* tok);
 /**
  * Is token used as boolean, that is to say cast to a bool, or used as a condition in a if/while/for
  */
-CPPCHECKLIB bool isUsedAsBool(const Token* const tok, const Settings* settings = nullptr);
+CPPCHECKLIB bool isUsedAsBool(const Token* tok, const Settings& settings);
 
 /**
  * Are the tokens' flags equal?
@@ -285,12 +290,12 @@ bool compareTokenFlags(const Token* tok1, const Token* tok2, bool macro);
  * @param isNot  do you want to know if cond1 is !cond2 or if cond1 and cond2 are non-overlapping. true: cond1==!cond2  false: cond1==true => cond2==false
  * @param cond1  condition1
  * @param cond2  condition2
- * @param library files data
+ * @param settings settings
  * @param pure boolean
  */
-bool isOppositeCond(bool isNot, const Token * const cond1, const Token * const cond2, const Library& library, bool pure, bool followVar, ErrorPath* errors=nullptr);
+bool isOppositeCond(bool isNot, const Token * cond1, const Token * cond2, const Settings& settings, bool pure, bool followVar, ErrorPath* errors=nullptr);
 
-bool isOppositeExpression(const Token * const tok1, const Token * const tok2, const Library& library, bool pure, bool followVar, ErrorPath* errors=nullptr);
+bool isOppositeExpression(const Token * tok1, const Token * tok2, const Settings& settings, bool pure, bool followVar, ErrorPath* errors=nullptr);
 
 bool isConstFunctionCall(const Token* ftok, const Library& library);
 
@@ -303,7 +308,7 @@ bool isUniqueExpression(const Token* tok);
 bool isEscapeFunction(const Token* ftok, const Library* library);
 
 /** Is scope a return scope (scope will unconditionally return) */
-CPPCHECKLIB bool isReturnScope(const Token* const endToken,
+CPPCHECKLIB bool isReturnScope(const Token* endToken,
                                const Library& library,
                                const Token** unknownFunc = nullptr,
                                bool functionScope = false);
@@ -328,7 +333,7 @@ std::vector<const Variable*> getArgumentVars(const Token* tok, int argnr);
  * @param settings      program settings
  * @param inconclusive  pointer to output variable which indicates that the answer of the question is inconclusive
  */
-bool isVariableChangedByFunctionCall(const Token *tok, int indirect, nonneg int varid, const Settings *settings, bool *inconclusive);
+bool isVariableChangedByFunctionCall(const Token *tok, int indirect, nonneg int varid, const Settings &settings, bool *inconclusive);
 
 /** Is variable changed by function call?
  * In case the answer of the question is inconclusive, e.g. because the function declaration is not known
@@ -338,38 +343,37 @@ bool isVariableChangedByFunctionCall(const Token *tok, int indirect, nonneg int
  * @param settings      program settings
  * @param inconclusive pointer to output variable which indicates that the answer of the question is inconclusive
  */
-CPPCHECKLIB bool isVariableChangedByFunctionCall(const Token *tok, int indirect, const Settings *settings, bool *inconclusive);
+CPPCHECKLIB bool isVariableChangedByFunctionCall(const Token *tok, int indirect, const Settings &settings, bool *inconclusive);
 
 /** Is variable changed in block of code? */
-CPPCHECKLIB bool isVariableChanged(const Token *start, const Token *end, const nonneg int exprid, bool globalvar, const Settings *settings, int depth = 20);
-bool isVariableChanged(const Token *start, const Token *end, int indirect, const nonneg int exprid, bool globalvar, const Settings *settings, int depth = 20);
+CPPCHECKLIB bool isVariableChanged(const Token *start, const Token *end, nonneg int exprid, bool globalvar, const Settings &settings, int depth = 20);
+bool isVariableChanged(const Token *start, const Token *end, int indirect, nonneg int exprid, bool globalvar, const Settings &settings, int depth = 20);
 
-bool isVariableChanged(const Token *tok, int indirect, const Settings *settings, int depth = 20);
+bool isVariableChanged(const Token *tok, int indirect, const Settings &settings, int depth = 20);
 
-bool isVariableChanged(const Variable * var, const Settings *settings, int depth = 20);
+bool isVariableChanged(const Variable * var, const Settings &settings, int depth = 20);
 
 bool isVariablesChanged(const Token* start,
                         const Token* end,
                         int indirect,
                         const std::vector<const Variable*> &vars,
-                        const Settings* settings);
+                        const Settings& settings);
 
-bool isThisChanged(const Token* tok, int indirect, const Settings* settings);
-const Token* findThisChanged(const Token* start, const Token* end, int indirect, const Settings* settings);
+bool isThisChanged(const Token* tok, int indirect, const Settings& settings);
 
-const Token* findVariableChanged(const Token *start, const Token *end, int indirect, const nonneg int exprid, bool globalvar, const Settings *settings, int depth = 20);
-Token* findVariableChanged(Token *start, const Token *end, int indirect, const nonneg int exprid, bool globalvar, const Settings *settings, int depth = 20);
+const Token* findVariableChanged(const Token *start, const Token *end, int indirect, nonneg int exprid, bool globalvar, const Settings &settings, int depth = 20);
+Token* findVariableChanged(Token *start, const Token *end, int indirect, nonneg int exprid, bool globalvar, const Settings &settings, int depth = 20);
 
 CPPCHECKLIB const Token* findExpressionChanged(const Token* expr,
                                                const Token* start,
                                                const Token* end,
-                                               const Settings* settings,
+                                               const Settings& settings,
                                                int depth = 20);
 
 const Token* findExpressionChangedSkipDeadCode(const Token* expr,
                                                const Token* start,
                                                const Token* end,
-                                               const Settings* settings,
+                                               const Settings& settings,
                                                const std::function<std::vector<MathLib::bigint>(const Token* tok)>& evaluate,
                                                int depth = 20);
 
@@ -377,7 +381,7 @@ bool isExpressionChangedAt(const Token* expr,
                            const Token* tok,
                            int indirect,
                            bool globalvar,
-                           const Settings* settings,
+                           const Settings& settings,
                            int depth = 20);
 
 /// If token is an alias if another variable
@@ -385,8 +389,6 @@ bool isAliasOf(const Token *tok, nonneg int varid, bool* inconclusive = nullptr)
 
 bool isAliasOf(const Token* tok, const Token* expr, int* indirect = nullptr, bool* inconclusive = nullptr);
 
-bool isAliased(const Variable *var);
-
 const Token* getArgumentStart(const Token* ftok);
 
 /** Determines the number of arguments - if token is a function call or macro
@@ -437,7 +439,7 @@ bool isConstVarExpression(const Token* tok, const std::function<bool(const Token
 
 bool isLeafDot(const Token* tok);
 
-enum class ExprUsage { None, NotUsed, PassedByReference, Used, Inconclusive };
+enum class ExprUsage : std::uint8_t { None, NotUsed, PassedByReference, Used, Inconclusive };
 
 ExprUsage getExprUsage(const Token* tok, int indirect, const Settings& settings);
 
