@@ -39,7 +39,6 @@
 #include <algorithm>
 #include <array>
 #include <cstddef>
-#include <iostream>
 #include <list>
 #include <vector>
 
@@ -104,47 +103,6 @@ static bool isVarTokComparison(const Token * tok, const Token ** vartok,
 
 //---------------------------------------------------------------------------
 
-void VarInfo::print()
-{
-    std::cout << "size=" << alloctype.size() << std::endl;
-    for (std::map<int, AllocInfo>::const_iterator it = alloctype.cbegin(); it != alloctype.cend(); ++it) {
-        std::string strusage;
-        const auto use = possibleUsage.find(it->first);
-        if (use != possibleUsage.end())
-            strusage = use->second.first->str();
-
-        std::string status;
-        switch (it->second.status) {
-        case OWNED:
-            status = "owned";
-            break;
-        case DEALLOC:
-            status = "dealloc";
-            break;
-        case ALLOC:
-            status = "alloc";
-            break;
-        case NOALLOC:
-            status = "noalloc";
-            break;
-        case REALLOC:
-            status = "realloc";
-            break;
-        default:
-            status = "?";
-            break;
-        }
-
-        std::cout << "status=" << status << " "
-                  << "alloctype='" << it->second.type << "' "
-                  << "possibleUsage='" << strusage << "' "
-                  << "conditionalAlloc=" << (conditionalAlloc.find(it->first) != conditionalAlloc.end() ? "yes" : "no") << " "
-                  << "referenced=" << (referenced.find(it->first) != referenced.end() ? "yes" : "no") << " "
-                  << "reallocedFrom=" << it->second.reallocedFromType
-                  << std::endl;
-    }
-}
-
 void VarInfo::possibleUsageAll(const std::pair<const Token*, Usage>& functionUsage)
 {
     possibleUsage.clear();
@@ -304,6 +262,30 @@ static const Token * isFunctionCall(const Token * nameToken)
     return nullptr;
 }
 
+static const Token* getOutparamAllocation(const Token* tok, const Settings& settings)
+{
+    if (!tok)
+        return nullptr;
+    int argn{};
+    const Token* ftok = getTokenArgumentFunction(tok, argn);
+    if (!ftok)
+        return nullptr;
+    if (const Library::AllocFunc* allocFunc = settings.library.getAllocFuncInfo(ftok)) {
+        if (allocFunc->arg == argn + 1)
+            return ftok;
+    }
+    return nullptr;
+}
+
+static const Token* getReturnValueFromOutparamAlloc(const Token* alloc, const Settings& settings)
+{
+    if (const Token* ftok = getOutparamAllocation(alloc, settings)) {
+        if (Token::simpleMatch(ftok->astParent()->astParent(), "="))
+            return ftok->next()->astParent()->astOperand1();
+    }
+    return nullptr;
+}
+
 bool CheckLeakAutoVar::checkScope(const Token * const startToken,
                                   VarInfo &varInfo,
                                   std::set<int> notzero,
@@ -401,7 +383,7 @@ bool CheckLeakAutoVar::checkScope(const Token * const startToken,
             if (Token::simpleMatch(tokAssignOp->astOperand1(), "."))
                 continue;
             // taking address of another variable..
-            if (Token::Match(tokAssignOp, "= %var% [+;]")) {
+            if (Token::Match(tokAssignOp, "= %var% +|;|?|%comp%")) {
                 if (varTok->tokAt(2)->varId() != varTok->varId()) {
                     // If variable points at allocated memory => error
                     leakIfAllocated(varTok, varInfo);
@@ -567,6 +549,15 @@ bool CheckLeakAutoVar::checkScope(const Token * const startToken,
                         if (astIsVariableComparison(tok3, "!=", "0", &vartok) &&
                             (notzero.find(vartok->varId()) != notzero.end()))
                             varInfo2.clear();
+
+                        if (std::any_of(varInfo1.alloctype.begin(), varInfo1.alloctype.end(), [&](const std::pair<int, VarInfo::AllocInfo>& info) {
+                            if (info.second.status != VarInfo::ALLOC)
+                                return false;
+                            const Token* ret = getReturnValueFromOutparamAlloc(info.second.allocTok, *mSettings);
+                            return ret && vartok && ret->varId() && ret->varId() == vartok->varId();
+                        })) {
+                            varInfo1.clear();
+                        }
                     } else if (isVarTokComparison(tok3, &vartok, alloc_failed_conds)) {
                         varInfo1.reallocToAlloc(vartok->varId());
                         varInfo1.erase(vartok->varId());
@@ -701,7 +692,7 @@ bool CheckLeakAutoVar::checkScope(const Token * const startToken,
                 continue;
             functionCall(ftok, openingPar, varInfo, allocation, af);
 
-            tok = ftok->next()->link();
+            tok = ftok->linkAt(1);
 
             // Handle scopes that might be noreturn
             if (allocation.status == VarInfo::NOALLOC && Token::simpleMatch(tok, ") ; }")) {
@@ -923,7 +914,7 @@ void CheckLeakAutoVar::changeAllocStatus(VarInfo &varInfo, const VarInfo::AllocI
         if (allocation.status == VarInfo::NOALLOC) {
             // possible usage
             varInfo.possibleUsage[arg->varId()] = { tok, VarInfo::USED };
-            if (var->second.status == VarInfo::DEALLOC && arg->previous()->str() == "&")
+            if (var->second.status == VarInfo::DEALLOC && arg->strAt(-1) == "&")
                 varInfo.erase(arg->varId());
         } else if (var->second.managed()) {
             doubleFreeError(tok, var->second.allocTok, arg->str(), allocation.type);
@@ -1097,21 +1088,6 @@ void CheckLeakAutoVar::leakIfAllocated(const Token *vartok,
     }
 }
 
-static const Token* getOutparamAllocation(const Token* tok, const Settings* settings)
-{
-    if (!tok)
-        return nullptr;
-    int argn{};
-    const Token* ftok = getTokenArgumentFunction(tok, argn);
-    if (!ftok)
-        return nullptr;
-    if (const Library::AllocFunc* allocFunc = settings->library.getAllocFuncInfo(ftok)) {
-        if (allocFunc->arg == argn + 1)
-            return ftok;
-    }
-    return nullptr;
-}
-
 void CheckLeakAutoVar::ret(const Token *tok, VarInfo &varInfo, const bool isEndOfScope)
 {
     const std::map<int, VarInfo::AllocInfo> &alloctype = varInfo.alloctype;
@@ -1134,7 +1110,7 @@ void CheckLeakAutoVar::ret(const Token *tok, VarInfo &varInfo, const bool isEndO
             // don't warn if we leave an inner scope
             if (isEndOfScope && var->scope() && tok != var->scope()->bodyEnd)
                 continue;
-            enum class PtrUsage { NONE, DEREF, PTR } used = PtrUsage::NONE;
+            enum class PtrUsage : std::uint8_t { NONE, DEREF, PTR } used = PtrUsage::NONE;
             for (const Token *tok2 = tok; tok2; tok2 = tok2->next()) {
                 if (tok2->str() == ";")
                     break;
@@ -1168,7 +1144,7 @@ void CheckLeakAutoVar::ret(const Token *tok, VarInfo &varInfo, const bool isEndO
             // don't warn when returning after checking return value of outparam allocation
             const Token* outparamFunc{};
             if ((tok->scope()->type == Scope::ScopeType::eIf || tok->scope()->type== Scope::ScopeType::eElse) &&
-                (outparamFunc = getOutparamAllocation(it->second.allocTok, mSettings))) {
+                (outparamFunc = getOutparamAllocation(it->second.allocTok, *mSettings))) {
                 const Scope* scope = tok->scope();
                 if (scope->type == Scope::ScopeType::eElse) {
                     scope = scope->bodyStart->tokAt(-2)->scope();
@@ -1177,7 +1153,7 @@ void CheckLeakAutoVar::ret(const Token *tok, VarInfo &varInfo, const bool isEndO
                 const Token* const ifStart = ifEnd->link();
                 const Token* const alloc = it->second.allocTok;
                 if (precedes(ifStart, alloc) && succeeds(ifEnd, alloc)) { // allocation and check in if
-                    if (Token::Match(outparamFunc->next()->astParent(), "%comp%"))
+                    if (outparamFunc->next()->astParent() == ifStart || Token::Match(outparamFunc->next()->astParent(), "%comp%"))
                         continue;
                 } else { // allocation result assigned to variable
                     const Token* const retAssign = outparamFunc->next()->astParent();
