@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -68,7 +68,7 @@ void CheckType::checkTooBigBitwiseShift()
 
     for (const Token *tok = mTokenizer->tokens(); tok; tok = tok->next()) {
         // C++ and macro: OUT(x<<y)
-        if (mTokenizer->isCPP() && Token::Match(tok, "[;{}] %name% (") && Token::simpleMatch(tok->linkAt(2), ") ;") && tok->next()->isUpperCaseName() && !tok->next()->function())
+        if (tok->isCpp() && Token::Match(tok, "[;{}] %name% (") && Token::simpleMatch(tok->linkAt(2), ") ;") && tok->next()->isUpperCaseName() && !tok->next()->function())
             tok = tok->linkAt(2);
 
         if (!tok->astOperand1() || !tok->astOperand2())
@@ -99,11 +99,11 @@ void CheckType::checkTooBigBitwiseShift()
             continue;
 
         // Get biggest rhs value. preferably a value which doesn't have 'condition'.
-        const ValueFlow::Value * value = tok->astOperand2()->getValueGE(lhsbits, mSettings);
+        const ValueFlow::Value * value = tok->astOperand2()->getValueGE(lhsbits, *mSettings);
         if (value && mSettings->isEnabled(value, false))
             tooBigBitwiseShiftError(tok, lhsbits, *value);
         else if (lhstype->sign == ValueType::Sign::SIGNED) {
-            value = tok->astOperand2()->getValueGE(lhsbits-1, mSettings);
+            value = tok->astOperand2()->getValueGE(lhsbits-1, *mSettings);
             if (value && mSettings->isEnabled(value, false))
                 tooBigSignedBitwiseShiftError(tok, lhsbits, *value);
         }
@@ -133,7 +133,7 @@ void CheckType::tooBigSignedBitwiseShiftError(const Token *tok, int lhsbits, con
 {
     constexpr char id[] = "shiftTooManyBitsSigned";
 
-    const bool cpp14 = mSettings->standards.cpp >= Standards::CPP14;
+    const bool cpp14 = ((tok && tok->isCpp()) || (mTokenizer && mTokenizer->isCPP())) && (mSettings->standards.cpp >= Standards::CPP14);
 
     std::string behaviour = "undefined";
     if (cpp14)
@@ -198,9 +198,9 @@ void CheckType::checkIntegerOverflow()
         const MathLib::bigint maxvalue = (((MathLib::biguint)1) << (bits - 1)) - 1;
 
         // is there a overflow result value
-        const ValueFlow::Value *value = tok->getValueGE(maxvalue + 1, mSettings);
+        const ValueFlow::Value *value = tok->getValueGE(maxvalue + 1, *mSettings);
         if (!value)
-            value = tok->getValueLE(-maxvalue - 2, mSettings);
+            value = tok->getValueLE(-maxvalue - 2, *mSettings);
         if (!value || !mSettings->isEnabled(value,false))
             continue;
 
@@ -259,7 +259,7 @@ void CheckType::checkSignConversion()
             if (!tok1)
                 continue;
             const ValueFlow::Value* negativeValue =
-                ValueFlow::findValue(tok1->values(), mSettings, [&](const ValueFlow::Value& v) {
+                ValueFlow::findValue(tok1->values(), *mSettings, [&](const ValueFlow::Value& v) {
                 return !v.isImpossible() && v.isIntValue() && (v.intvalue <= -1 || v.wideintvalue <= -1);
             });
             if (!negativeValue)
@@ -299,7 +299,7 @@ void CheckType::signConversionError(const Token *tok, const ValueFlow::Value *ne
 //---------------------------------------------------------------------------
 // Checking for long cast of int result   const long x = var1 * var2;
 //---------------------------------------------------------------------------
-static bool checkTypeCombination(const ValueType& src, const ValueType& tgt, const Settings* settings)
+static bool checkTypeCombination(ValueType src, ValueType tgt, const Settings& settings)
 {
     static const std::pair<ValueType::Type, ValueType::Type> typeCombinations[] = {
         { ValueType::Type::INT, ValueType::Type::LONG },
@@ -310,6 +310,9 @@ static bool checkTypeCombination(const ValueType& src, const ValueType& tgt, con
         { ValueType::Type::DOUBLE, ValueType::Type::LONGDOUBLE },
     };
 
+    src.reference = Reference::None;
+    tgt.reference = Reference::None;
+
     const std::size_t sizeSrc = ValueFlow::getSizeOf(src, settings);
     const std::size_t sizeTgt = ValueFlow::getSizeOf(tgt, settings);
     if (!(sizeSrc > 0 && sizeTgt > 0 && sizeSrc < sizeTgt))
@@ -322,7 +325,7 @@ static bool checkTypeCombination(const ValueType& src, const ValueType& tgt, con
 
 void CheckType::checkLongCast()
 {
-    if (!mSettings->severity.isEnabled(Severity::style))
+    if (!mSettings->severity.isEnabled(Severity::style) && !mSettings->isPremiumEnabled("truncLongCastAssignment"))
         return;
 
     logChecker("CheckType::checkLongCast"); // style
@@ -343,7 +346,7 @@ void CheckType::checkLongCast()
 
         if (!lhstype || !rhstype)
             continue;
-        if (!checkTypeCombination(*rhstype, *lhstype, mSettings))
+        if (!checkTypeCombination(*rhstype, *lhstype, *mSettings))
             continue;
 
         // assign int result to long/longlong const nonpointer?
@@ -370,7 +373,7 @@ void CheckType::checkLongCast()
             if (tok->str() == "return") {
                 if (Token::Match(tok->astOperand1(), "<<|*")) {
                     const ValueType *type = tok->astOperand1()->valueType();
-                    if (type && checkTypeCombination(*type, *retVt, mSettings) &&
+                    if (type && checkTypeCombination(*type, *retVt, *mSettings) &&
                         type->pointer == 0U &&
                         type->originalTypeName.empty())
                         ret = tok;
