@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,26 +34,17 @@
 #include <algorithm>
 #include <cctype>
 #include <cstring>
-#include <fstream> // IWYU pragma: keep
+#include <fstream>
 #include <map>
-#include <sstream> // IWYU pragma: keep
+#include <sstream>
 #include <tuple>
 #include <utility>
 #include <vector>
 
 #include "xml.h"
 
-namespace CTU {
-    class FileInfo;
-}
-
 //---------------------------------------------------------------------------
 
-// Register this check class
-namespace {
-    CheckUnusedFunctions instance;
-}
-
 static const CWE CWE561(561U);   // Dead Code
 
 static std::string stripTemplateParameters(const std::string& funcName) {
@@ -68,15 +59,16 @@ static std::string stripTemplateParameters(const std::string& funcName) {
 // FUNCTION USAGE - Check for unused functions etc
 //---------------------------------------------------------------------------
 
-void CheckUnusedFunctions::clear()
+static bool isRecursiveCall(const Token* ftok)
 {
-    instance.mFunctions.clear();
-    instance.mFunctionCalls.clear();
+    return ftok->function() && ftok->function() == Scope::nestedInFunction(ftok->scope());
 }
 
-void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char FileName[], const Settings *settings)
+void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const Settings &settings)
 {
-    const bool doMarkup = settings->library.markupFile(FileName);
+    const char * const FileName = tokenizer.list.getFiles().front().c_str();
+
+    const bool doMarkup = settings.library.markupFile(FileName);
 
     // Function declarations..
     if (!doMarkup) {
@@ -93,6 +85,15 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
             if (func->isExtern())
                 continue;
 
+            bool foundAllBaseClasses{};
+            if (const Function* ofunc = func->getOverriddenFunction(&foundAllBaseClasses)) {
+                if (!foundAllBaseClasses || ofunc->isPure())
+                    continue;
+            }
+            else if (func->isImplicitlyVirtual()) {
+                continue;
+            }
+
             mFunctionDecl.emplace_back(func);
 
             FunctionUsage &usage = mFunctions[stripTemplateParameters(func->name())];
@@ -100,6 +101,7 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
             if (!usage.lineNumber)
                 usage.lineNumber = func->token->linenr();
 
+            // TODO: why always overwrite this but not the filename and line?
             usage.fileIndex = func->token->fileIndex();
             const std::string& fileName = tokenizer.list.file(func->token);
 
@@ -125,8 +127,8 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
             lambdaEndToken = findLambdaEndToken(tok);
 
         // parsing of library code to find called functions
-        if (settings->library.isexecutableblock(FileName, tok->str())) {
-            const Token * markupVarToken = tok->tokAt(settings->library.blockstartoffset(FileName));
+        if (settings.library.isexecutableblock(FileName, tok->str())) {
+            const Token * markupVarToken = tok->tokAt(settings.library.blockstartoffset(FileName));
             // not found
             if (!markupVarToken)
                 continue;
@@ -134,16 +136,16 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
             bool start = true;
             // find all function calls in library code (starts with '(', not if or while etc)
             while ((scope || start) && markupVarToken) {
-                if (markupVarToken->str() == settings->library.blockstart(FileName)) {
+                if (markupVarToken->str() == settings.library.blockstart(FileName)) {
                     scope++;
                     start = false;
-                } else if (markupVarToken->str() == settings->library.blockend(FileName))
+                } else if (markupVarToken->str() == settings.library.blockend(FileName))
                     scope--;
-                else if (!settings->library.iskeyword(FileName, markupVarToken->str())) {
+                else if (!settings.library.iskeyword(FileName, markupVarToken->str())) {
                     mFunctionCalls.insert(markupVarToken->str());
                     if (mFunctions.find(markupVarToken->str()) != mFunctions.end())
                         mFunctions[markupVarToken->str()].usedOtherFile = true;
-                    else if (markupVarToken->next()->str() == "(") {
+                    else if (markupVarToken->strAt(1) == "(") {
                         FunctionUsage &func = mFunctions[markupVarToken->str()];
                         func.filename = tokenizer.list.getFiles()[markupVarToken->fileIndex()];
                         if (func.filename.empty() || func.filename == "+")
@@ -157,10 +159,10 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
         }
 
         if (!doMarkup // only check source files
-            && settings->library.isexporter(tok->str()) && tok->next() != nullptr) {
+            && settings.library.isexporter(tok->str()) && tok->next() != nullptr) {
             const Token * propToken = tok->next();
             while (propToken && propToken->str() != ")") {
-                if (settings->library.isexportedprefix(tok->str(), propToken->str())) {
+                if (settings.library.isexportedprefix(tok->str(), propToken->str())) {
                     const Token* nextPropToken = propToken->next();
                     const std::string& value = nextPropToken->str();
                     if (mFunctions.find(value) != mFunctions.end()) {
@@ -168,7 +170,7 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
                     }
                     mFunctionCalls.insert(value);
                 }
-                if (settings->library.isexportedsuffix(tok->str(), propToken->str())) {
+                if (settings.library.isexportedsuffix(tok->str(), propToken->str())) {
                     const Token* prevPropToken = propToken->previous();
                     const std::string& value = prevPropToken->str();
                     if (value != ")" && mFunctions.find(value) != mFunctions.end()) {
@@ -180,7 +182,7 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
             }
         }
 
-        if (doMarkup && settings->library.isimporter(FileName, tok->str()) && tok->next()) {
+        if (doMarkup && settings.library.isimporter(FileName, tok->str()) && tok->next()) {
             const Token * propToken = tok->next();
             if (propToken->next()) {
                 propToken = propToken->next();
@@ -196,8 +198,8 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
             }
         }
 
-        if (settings->library.isreflection(tok->str())) {
-            const int argIndex = settings->library.reflectionArgument(tok->str());
+        if (settings.library.isreflection(tok->str())) {
+            const int argIndex = settings.library.reflectionArgument(tok->str());
             if (argIndex >= 0) {
                 const Token * funcToken = tok->next();
                 int index = 0;
@@ -219,6 +221,13 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
             }
         }
 
+        if (tok->hasAttributeCleanup()) {
+            const std::string& funcname = tok->getAttributeCleanup();
+            mFunctions[funcname].usedOtherFile = true;
+            mFunctionCalls.insert(funcname);
+            continue;
+        }
+
         const Token *funcname = nullptr;
 
         if (doMarkup)
@@ -240,11 +249,11 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
             while (Token::Match(funcname, "%name% :: %name%"))
                 funcname = funcname->tokAt(2);
 
-            if (!Token::Match(funcname, "%name% [(),;]:}>]") || funcname->varId())
+            if (!Token::Match(funcname, "%name% [(),;]:}>]"))
                 continue;
         }
 
-        if (!funcname || funcname->isKeyword() || funcname->isStandardType())
+        if (!funcname || funcname->isKeyword() || funcname->isStandardType() || funcname->varId() || funcname->enumerator() || funcname->type())
             continue;
 
         // funcname ( => Assert that the end parentheses isn't followed by {
@@ -257,6 +266,8 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
         }
 
         if (funcname) {
+            if (isRecursiveCall(funcname))
+                continue;
             const auto baseName = stripTemplateParameters(funcname->str());
             FunctionUsage &func = mFunctions[baseName];
             const std::string& called_from_file = tokenizer.list.getFiles()[funcname->fileIndex()];
@@ -319,8 +330,16 @@ static bool isOperatorFunction(const std::string & funcName)
     return std::find(additionalOperators.cbegin(), additionalOperators.cend(), funcName.substr(operatorPrefix.length())) != additionalOperators.cend();
 }
 
-bool CheckUnusedFunctions::check(ErrorLogger * const errorLogger, const Settings& settings) const
+#define logChecker(id) \
+    do { \
+        const ErrorMessage errmsg({}, nullptr, Severity::internal, "logChecker", (id), CWE(0U), Certainty::normal); \
+        errorLogger.reportErr(errmsg); \
+    } while (false)
+
+bool CheckUnusedFunctions::check(const Settings& settings, ErrorLogger& errorLogger) const
 {
+    logChecker("CheckUnusedFunctions::check"); // unusedFunction
+
     using ErrorParams = std::tuple<std::string, unsigned int, unsigned int, std::string>;
     std::vector<ErrorParams> errors; // ensure well-defined order
 
@@ -353,40 +372,18 @@ bool CheckUnusedFunctions::check(ErrorLogger * const errorLogger, const Settings
     return !errors.empty();
 }
 
-void CheckUnusedFunctions::unusedFunctionError(ErrorLogger * const errorLogger,
+void CheckUnusedFunctions::unusedFunctionError(ErrorLogger& errorLogger,
                                                const std::string &filename, unsigned int fileIndex, unsigned int lineNumber,
                                                const std::string &funcname)
 {
     std::list<ErrorMessage::FileLocation> locationList;
     if (!filename.empty()) {
-        locationList.emplace_back(filename, lineNumber);
+        locationList.emplace_back(filename, lineNumber, 0);
         locationList.back().fileIndex = fileIndex;
     }
 
-    const ErrorMessage errmsg(locationList, emptyString, Severity::style, "$symbol:" + funcname + "\nThe function '$symbol' is never used.", "unusedFunction", CWE561, Certainty::normal);
-    if (errorLogger)
-        errorLogger->reportErr(errmsg);
-    else
-        Check::writeToErrorList(errmsg);
-}
-
-Check::FileInfo *CheckUnusedFunctions::getFileInfo(const Tokenizer *tokenizer, const Settings *settings) const
-{
-    if (!settings->checks.isEnabled(Checks::unusedFunction))
-        return nullptr;
-    if (settings->useSingleJob() && settings->buildDir.empty())
-        instance.parseTokens(*tokenizer, tokenizer->list.getFiles().front().c_str(), settings);
-    return nullptr;
-}
-
-bool CheckUnusedFunctions::analyseWholeProgram(const CTU::FileInfo *ctu, const std::list<Check::FileInfo*> &fileInfo, const Settings& settings, ErrorLogger &errorLogger)
-{
-    (void)ctu;
-    (void)fileInfo;
-    CheckUnusedFunctions dummy(nullptr, &settings, &errorLogger);
-    dummy.
-    logChecker("CheckUnusedFunctions::analyseWholeProgram"); // unusedFunctions
-    return check(&errorLogger, settings);
+    const ErrorMessage errmsg(std::move(locationList), emptyString, Severity::style, "$symbol:" + funcname + "\nThe function '$symbol' is never used.", "unusedFunction", CWE561, Certainty::normal);
+    errorLogger.reportErr(errmsg);
 }
 
 CheckUnusedFunctions::FunctionDecl::FunctionDecl(const Function *f)
@@ -416,7 +413,7 @@ namespace {
     };
 }
 
-void CheckUnusedFunctions::analyseWholeProgram(const Settings &settings, ErrorLogger * const errorLogger, const std::string &buildDir)
+void CheckUnusedFunctions::analyseWholeProgram(const Settings &settings, ErrorLogger &errorLogger, const std::string &buildDir)
 {
     std::map<std::string, Location> decls;
     std::set<std::string> calls;
@@ -480,3 +477,23 @@ void CheckUnusedFunctions::analyseWholeProgram(const Settings &settings, ErrorLo
         }
     }
 }
+
+void CheckUnusedFunctions::updateFunctionData(const CheckUnusedFunctions& check)
+{
+    for (const auto& entry : check.mFunctions)
+    {
+        FunctionUsage &usage = mFunctions[entry.first];
+        if (!usage.lineNumber)
+            usage.lineNumber = entry.second.lineNumber;
+        // TODO: why always overwrite this but not the filename and line?
+        usage.fileIndex = entry.second.fileIndex;
+        if (usage.filename.empty())
+            usage.filename = entry.second.filename;
+        // cppcheck-suppress bitwiseOnBoolean - TODO: FP
+        usage.usedOtherFile |= entry.second.usedOtherFile;
+        // cppcheck-suppress bitwiseOnBoolean - TODO: FP
+        usage.usedSameFile |= entry.second.usedSameFile;
+    }
+    mFunctionDecl.insert(mFunctionDecl.cend(), check.mFunctionDecl.cbegin(), check.mFunctionDecl.cend());
+    mFunctionCalls.insert(check.mFunctionCalls.cbegin(), check.mFunctionCalls.cend());
+}
