@@ -1,4 +1,4 @@
-/*
+/* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
  * Copyright (C) 2007-2024 Cppcheck team.
  *
@@ -28,6 +28,7 @@
 #include "mathlib.h"
 #include "vfvalue.h"
 
+#include <cstdint>
 #include <list>
 #include <map>
 #include <string>
@@ -51,7 +52,7 @@ namespace tinyxml2 {
 namespace CTU {
     class CPPCHECKLIB FileInfo : public Check::FileInfo {
     public:
-        enum class InvalidValueType { null, uninit, bufferOverflow };
+        enum class InvalidValueType : std::uint8_t { null, uninit, bufferOverflow };
 
         std::string toString() const override;
 
@@ -133,7 +134,7 @@ namespace CTU {
                                                                   const UnsafeUsage &unsafeUsage,
                                                                   const std::map<std::string, std::list<const CallBase *>> &callsMap,
                                                                   const char info[],
-                                                                  const FunctionCall ** const functionCallPtr,
+                                                                  const FunctionCall ** functionCallPtr,
                                                                   bool warning);
     };
 
