@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include "color.h"
 #include "cppcheck.h"
 #include "path.h"
+#include "settings.h"
 #include "suppressions.h"
 #include "token.h"
 #include "tokenlist.h"
@@ -31,8 +32,9 @@
 #include <cassert>
 #include <cctype>
 #include <cstring>
+#include <fstream>
 #include <iomanip>
-#include <sstream> // IWYU pragma: keep
+#include <sstream>
 #include <string>
 #include <unordered_map>
 #include <utility>
@@ -45,7 +47,6 @@ const std::set<std::string> ErrorLogger::mCriticalErrorIds{
     "internalAstError",
     "instantiationError",
     "internalError",
-    "misra-config",
     "premium-internalError",
     "premium-invalidArgument",
     "premium-invalidLicense",
@@ -136,15 +137,19 @@ ErrorMessage::ErrorMessage(const ErrorPath &errorPath, const TokenList *tokenLis
         if (!tok)
             continue;
 
-        std::string info = e.second;
+        const std::string& path_info = e.second;
 
-        if (startsWith(info,"$symbol:") && info.find('\n') < info.size()) {
-            const std::string::size_type pos = info.find('\n');
-            const std::string &symbolName = info.substr(8, pos - 8);
-            info = replaceStr(info.substr(pos+1), "$symbol", symbolName);
+        std::string info;
+        if (startsWith(path_info,"$symbol:") && path_info.find('\n') < path_info.size()) {
+            const std::string::size_type pos = path_info.find('\n');
+            const std::string symbolName = path_info.substr(8, pos - 8);
+            info = replaceStr(path_info.substr(pos+1), "$symbol", symbolName);
+        }
+        else {
+            info = path_info;
         }
 
-        callStack.emplace_back(tok, info, tokenList);
+        callStack.emplace_back(tok, std::move(info), tokenList);
     }
 
     if (tokenList && !tokenList->getFiles().empty())
@@ -242,14 +247,11 @@ ErrorMessage ErrorMessage::fromInternalError(const InternalError &internalError,
 
     std::list<ErrorMessage::FileLocation> locationList;
     if (tokenList && internalError.token) {
-        ErrorMessage::FileLocation loc(internalError.token, tokenList);
-        locationList.push_back(std::move(loc));
+        locationList.emplace_back(internalError.token, tokenList);
     } else {
-        ErrorMessage::FileLocation loc2(filename, 0, 0);
-        locationList.push_back(std::move(loc2));
+        locationList.emplace_back(filename, 0, 0);
         if (tokenList && (filename != tokenList->getSourceFilePath())) {
-            ErrorMessage::FileLocation loc(tokenList->getSourceFilePath(), 0, 0);
-            locationList.push_back(std::move(loc));
+            locationList.emplace_back(tokenList->getSourceFilePath(), 0, 0);
         }
     }
     ErrorMessage errmsg(std::move(locationList),
@@ -272,11 +274,9 @@ std::string ErrorMessage::serialize() const
     serializeString(oss, severityToString(severity));
     serializeString(oss, std::to_string(cwe.id));
     serializeString(oss, std::to_string(hash));
+    serializeString(oss, fixInvalidChars(remark));
     serializeString(oss, file0);
-    if (certainty == Certainty::inconclusive) {
-        const std::string text("inconclusive");
-        serializeString(oss, text);
-    }
+    serializeString(oss, (certainty == Certainty::inconclusive) ? "1" : "0");
 
     const std::string saneShortMessage = fixInvalidChars(mShortMessage);
     const std::string saneVerboseMessage = fixInvalidChars(mVerboseMessage);
@@ -310,9 +310,9 @@ void ErrorMessage::deserialize(const std::string &data)
     callStack.clear();
 
     std::istringstream iss(data);
-    std::array<std::string, 7> results;
+    std::array<std::string, 9> results;
     std::size_t elem = 0;
-    while (iss.good() && elem < 7) {
+    while (iss.good() && elem < 9) {
         unsigned int len = 0;
         if (!(iss >> len))
             throw InternalError(nullptr, "Internal Error: Deserialization of error message failed - invalid length");
@@ -330,20 +330,15 @@ void ErrorMessage::deserialize(const std::string &data)
 
             if (!iss.good())
                 throw InternalError(nullptr, "Internal Error: Deserialization of error message failed - premature end of data");
-
-            if (temp == "inconclusive") {
-                certainty = Certainty::inconclusive;
-                continue;
-            }
         }
 
-        results[elem++] = temp;
+        results[elem++] = std::move(temp);
     }
 
     if (!iss.good())
         throw InternalError(nullptr, "Internal Error: Deserialization of error message failed - premature end of data");
 
-    if (elem != 7)
+    if (elem != 9)
         throw InternalError(nullptr, "Internal Error: Deserialization of error message failed - insufficient elements");
 
     id = std::move(results[0]);
@@ -360,9 +355,12 @@ void ErrorMessage::deserialize(const std::string &data)
         if (!strToInt(results[3], hash, &err))
             throw InternalError(nullptr, "Internal Error: Deserialization of error message failed - invalid hash - " + err);
     }
-    file0 = std::move(results[4]);
-    mShortMessage = std::move(results[5]);
-    mVerboseMessage = std::move(results[6]);
+    remark = std::move(results[4]);
+    file0 = std::move(results[5]);
+    if (results[6] == "1")
+        certainty = Certainty::inconclusive;
+    mShortMessage = std::move(results[7]);
+    mVerboseMessage = std::move(results[8]);
 
     unsigned int stackSize = 0;
     if (!(iss >> stackSize))
@@ -411,10 +409,11 @@ void ErrorMessage::deserialize(const std::string &data)
 
         // (*loc).line << '\t' << (*loc).column << '\t' << (*loc).getfile(false) << '\t' << loc->getOrigFile(false) << '\t' << loc->getinfo();
 
-        ErrorMessage::FileLocation loc(substrings[3], strToInt<int>(substrings[0]), strToInt<unsigned int>(substrings[1]));
-        loc.setfile(std::move(substrings[2]));
+        std::string info;
         if (substrings.size() == 5)
-            loc.setinfo(substrings[4]);
+            info = std::move(substrings[4]);
+        ErrorMessage::FileLocation loc(substrings[3], std::move(info), strToInt<int>(substrings[0]), strToInt<unsigned int>(substrings[1]));
+        loc.setfile(std::move(substrings[2]));
 
         callStack.push_back(std::move(loc));
 
@@ -425,14 +424,9 @@ void ErrorMessage::deserialize(const std::string &data)
 
 std::string ErrorMessage::getXMLHeader(std::string productName)
 {
-    std::string version = CppCheck::version();
-    if (!productName.empty() && std::isdigit(productName.back())) {
-        const std::string::size_type pos = productName.find_last_not_of(".0123456789");
-        if (pos > 1 && pos != std::string::npos && productName[pos] == ' ') {
-            version = productName.substr(pos+1);
-            productName.erase(pos);
-        }
-    }
+    const auto nameAndVersion = Settings::getNameAndVersion(productName);
+    productName = nameAndVersion.first;
+    const std::string version = nameAndVersion.first.empty() ? CppCheck::version() : nameAndVersion.second;
 
     tinyxml2::XMLPrinter printer;
 
@@ -498,6 +492,9 @@ std::string ErrorMessage::toXML() const
     if (!file0.empty())
         printer.PushAttribute("file0", file0.c_str());
 
+    if (!remark.empty())
+        printer.PushAttribute("remark", fixInvalidChars(remark).c_str());
+
     for (std::list<FileLocation>::const_reverse_iterator it = callStack.crbegin(); it != callStack.crend(); ++it) {
         printer.OpenElement("location", false);
         printer.PushAttribute("file", it->getfile().c_str());
@@ -525,22 +522,6 @@ std::string ErrorMessage::toXML() const
     return printer.CStr();
 }
 
-/**
- * Replace all occurrences of searchFor with replaceWith in the
- * given source.
- * @param source The string to modify
- * @param searchFor What should be searched for
- * @param replaceWith What will replace the found item
- */
-static void findAndReplace(std::string &source, const std::string &searchFor, const std::string &replaceWith)
-{
-    std::string::size_type index = 0;
-    while ((index = source.find(searchFor, index)) != std::string::npos) {
-        source.replace(index, searchFor.length(), replaceWith);
-        index += replaceWith.length();
-    }
-}
-
 // TODO: read info from some shared resource instead?
 static std::string readCode(const std::string &file, int linenr, int column, const char endl[])
 {
@@ -659,6 +640,7 @@ std::string ErrorMessage::toString(bool verbose, const std::string &templateForm
     findAndReplace(result, "{severity}", severityToString(severity));
     findAndReplace(result, "{cwe}", std::to_string(cwe.id));
     findAndReplace(result, "{message}", verbose ? mVerboseMessage : mShortMessage);
+    findAndReplace(result, "{remark}", remark);
     if (!callStack.empty()) {
         if (result.find("{callstack}") != std::string::npos)
             findAndReplace(result, "{callstack}", ErrorLogger::callStackToString(callStack));
@@ -749,8 +731,7 @@ std::string ErrorMessage::FileLocation::getOrigFile(bool convert) const
 
 void ErrorMessage::FileLocation::setfile(std::string file)
 {
-    mFileName = Path::fromNativeSeparators(std::move(file));
-    mFileName = Path::simplifyPath(std::move(mFileName));
+    mFileName = Path::simplifyPath(std::move(file));
 }
 
 std::string ErrorMessage::FileLocation::stringify() const
@@ -758,7 +739,7 @@ std::string ErrorMessage::FileLocation::stringify() const
     std::string str;
     str += '[';
     str += Path::toNativeSeparators(mFileName);
-    if (line != Suppressions::Suppression::NO_LINE) {
+    if (line != SuppressionList::Suppression::NO_LINE) {
         str += ':';
         str += std::to_string(line);
     }
