@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,16 +21,21 @@
 #include "analyzer.h"
 #include "astutils.h"
 #include "config.h"
+#include "errorlogger.h"
 #include "errortypes.h"
 #include "mathlib.h"
 #include "settings.h"
 #include "symboldatabase.h"
 #include "token.h"
+#include "tokenlist.h"
+#include "utils.h"
 #include "valueptr.h"
 #include "vfvalue.h"
 
 #include <algorithm>
+#include <cassert>
 #include <cstdio>
+#include <cstdint>
 #include <functional>
 #include <list>
 #include <string>
@@ -41,18 +46,21 @@
 
 namespace {
     struct ForwardTraversal {
-        enum class Progress { Continue, Break, Skip };
-        enum class Terminate { None, Bail, Inconclusive };
-        ForwardTraversal(const ValuePtr<Analyzer>& analyzer, const Settings& settings)
-            : analyzer(analyzer), settings(settings)
+        enum class Progress : std::uint8_t { Continue, Break, Skip };
+        enum class Terminate : std::uint8_t { None, Bail, Inconclusive };
+        ForwardTraversal(const ValuePtr<Analyzer>& analyzer, const TokenList& tokenList, ErrorLogger& errorLogger, const Settings& settings)
+            : analyzer(analyzer), tokenList(tokenList), errorLogger(errorLogger), settings(settings)
         {}
         ValuePtr<Analyzer> analyzer;
+        const TokenList& tokenList;
+        ErrorLogger& errorLogger;
         const Settings& settings;
         Analyzer::Action actions;
         bool analyzeOnly{};
         bool analyzeTerminate{};
         Analyzer::Terminate terminate = Analyzer::Terminate::None;
         std::vector<Token*> loopEnds;
+        int branchCount = 0;
 
         Progress Break(Analyzer::Terminate t = Analyzer::Terminate::None) {
             if ((!analyzeOnly || analyzeTerminate) && t != Analyzer::Terminate::None)
@@ -90,6 +98,15 @@ namespace {
             return actions.isModified();
         }
 
+        bool stopOnCondition(const Token* condTok) const
+        {
+            if (analyzer->isConditional() && findAstNode(condTok, [](const Token* tok) {
+                return tok->isIncompleteVar();
+            }))
+                return true;
+            return analyzer->stopOnCondition(condTok);
+        }
+
         std::pair<bool, bool> evalCond(const Token* tok, const Token* ctx = nullptr) const {
             if (!tok)
                 return std::make_pair(false, false);
@@ -109,7 +126,7 @@ namespace {
         }
 
         template<class T, class F, REQUIRES("T must be a Token class", std::is_convertible<T*, const Token*> )>
-        Progress traverseTok(T* tok, F f, bool traverseUnknown, T** out = nullptr) {
+        Progress traverseTok(T* tok, const F &f, bool traverseUnknown, T** out = nullptr) {
             if (Token::Match(tok, "asm|goto"))
                 return Break(Analyzer::Terminate::Bail);
             if (Token::Match(tok, "setjmp|longjmp (")) {
@@ -159,7 +176,7 @@ namespace {
         }
 
         template<class T, class F, REQUIRES("T must be a Token class", std::is_convertible<T*, const Token*> )>
-        Progress traverseRecursive(T* tok, F f, bool traverseUnknown, unsigned int recursion=0) {
+        Progress traverseRecursive(T* tok, const F &f, bool traverseUnknown, unsigned int recursion=0) {
             if (!tok)
                 return Progress::Continue;
             if (recursion > 10000)
@@ -187,12 +204,12 @@ namespace {
         template<class T, class F, REQUIRES("T must be a Token class", std::is_convertible<T*, const Token*> )>
         Progress traverseConditional(T* tok, F f, bool traverseUnknown) {
             if (Token::Match(tok, "?|&&|%oror%") && tok->astOperand1() && tok->astOperand2()) {
-                T* condTok = tok->astOperand1();
+                const T* condTok = tok->astOperand1();
                 T* childTok = tok->astOperand2();
                 bool checkThen, checkElse;
                 std::tie(checkThen, checkElse) = evalCond(condTok);
                 if (!checkThen && !checkElse) {
-                    if (!traverseUnknown && analyzer->stopOnCondition(condTok) && stopUpdates()) {
+                    if (!traverseUnknown && stopOnCondition(condTok) && stopUpdates()) {
                         return Progress::Continue;
                     }
                     checkThen = true;
@@ -304,7 +321,7 @@ namespace {
             for (const Token* tok=start; tok != end; tok = tok->previous()) {
                 if (Token::simpleMatch(tok, "}")) {
                     const Token* ftok = nullptr;
-                    const bool r = isReturnScope(tok, &settings.library, &ftok);
+                    const bool r = isReturnScope(tok, settings.library, &ftok);
                     if (r)
                         return true;
                 }
@@ -314,13 +331,13 @@ namespace {
 
         bool isEscapeScope(const Token* endBlock, bool& unknown) const {
             const Token* ftok = nullptr;
-            const bool r = isReturnScope(endBlock, &settings.library, &ftok);
+            const bool r = isReturnScope(endBlock, settings.library, &ftok);
             if (!r && ftok)
                 unknown = true;
             return r;
         }
 
-        enum class Status {
+        enum class Status : std::uint8_t {
             None,
             Inconclusive,
         };
@@ -376,13 +393,13 @@ namespace {
                 std::pair<const Token*, const Token*> exprToks = stepTok->findExpressionStartEndTokens();
                 if (exprToks.first != nullptr && exprToks.second != nullptr)
                     stepChangesCond |=
-                        findExpressionChanged(condTok, exprToks.first, exprToks.second->next(), &settings, true) != nullptr;
+                        findExpressionChanged(condTok, exprToks.first, exprToks.second->next(), settings) != nullptr;
             }
-            const bool bodyChangesCond = findExpressionChanged(condTok, endBlock->link(), endBlock, &settings, true);
+            const bool bodyChangesCond = findExpressionChanged(condTok, endBlock->link(), endBlock, settings);
             // Check for mutation in the condition
             const bool condChanged =
                 nullptr != findAstNode(condTok, [&](const Token* tok) {
-                return isVariableChanged(tok, 0, &settings, true);
+                return isVariableChanged(tok, 0, settings);
             });
             const bool changed = stepChangesCond || bodyChangesCond || condChanged;
             if (!changed)
@@ -443,7 +460,7 @@ namespace {
             if (checkElse && isDoWhile &&
                 (condTok->hasKnownIntValue() ||
                  (!bodyAnalysis.isModified() && !condAnalysis.isModified() && condAnalysis.isRead()))) {
-                if (updateRange(endBlock->link(), endBlock) == Progress::Break)
+                if (updateScope(endBlock) == Progress::Break)
                     return Break();
                 return updateRecursive(condTok);
             }
@@ -460,10 +477,10 @@ namespace {
                     if (updateRecursive(condTok) == Progress::Break)
                         return Break();
             }
-            if (!checkThen && !checkElse && !isDoWhile && analyzer->stopOnCondition(condTok) && stopUpdates())
+            if (!checkThen && !checkElse && !isDoWhile && stopOnCondition(condTok) && stopUpdates())
                 return Break(Analyzer::Terminate::Conditional);
             // condition is false, we don't enter the loop
-            if (checkElse)
+            if (checkElse && !isDoWhile)
                 return Progress::Continue;
             if (checkThen || isDoWhile) {
                 // Since we are re-entering the loop then assume the condition is true to update the state
@@ -519,8 +536,17 @@ namespace {
             return updateLoop(endToken, endBlock, condTok, initTok, stepTok, true);
         }
 
-        Progress updateScope(Token* endBlock) {
-            return updateRange(endBlock->link(), endBlock);
+        Progress updateScope(Token* endBlock, int depth = 20)
+        {
+            if (!endBlock)
+                return Break();
+            assert(endBlock->link());
+            Token* ctx = endBlock->link()->previous();
+            if (Token::simpleMatch(ctx, ")"))
+                ctx = ctx->link()->previous();
+            if (ctx)
+                analyzer->updateState(ctx);
+            return updateRange(endBlock->link(), endBlock, depth);
         }
 
         Progress updateRange(Token* start, const Token* end, int depth = 20) {
@@ -589,10 +615,10 @@ namespace {
                     const Scope* scope = tok->scope();
                     if (!scope)
                         return Break();
-                    if (Token::Match(tok->link()->previous(), ")|else {")) {
-                        const Token* tok2 = tok->link()->previous();
-                        const bool inElse = Token::simpleMatch(tok2, "else {");
-                        const bool inLoop = inElse ? false : Token::Match(tok2->link()->previous(), "while|for (");
+                    if (contains({Scope::eDo, Scope::eFor, Scope::eWhile, Scope::eIf, Scope::eElse, Scope::eSwitch}, scope->type)) {
+                        const bool inElse = scope->type == Scope::eElse;
+                        const bool inDoWhile = scope->type == Scope::eDo;
+                        const bool inLoop = contains({Scope::eDo, Scope::eFor, Scope::eWhile}, scope->type);
                         Token* condTok = getCondTokFromEnd(tok);
                         if (!condTok)
                             return Break();
@@ -621,24 +647,24 @@ namespace {
                             }
                         }
                         analyzer->assume(condTok, !inElse, Analyzer::Assume::Quiet);
-                        if (Token::simpleMatch(tok, "} else {"))
+                        assert(!inDoWhile || Token::simpleMatch(tok, "} while ("));
+                        if (Token::simpleMatch(tok, "} else {") || inDoWhile)
                             tok = tok->linkAt(2);
-                    } else if (scope->type == Scope::eTry) {
+                    } else if (contains({Scope::eTry, Scope::eCatch}, scope->type)) {
                         if (!analyzer->lowerToPossible())
                             return Break(Analyzer::Terminate::Bail);
                     } else if (scope->type == Scope::eLambda) {
                         return Break();
-                    } else if (scope->type == Scope::eDo && Token::simpleMatch(tok, "} while (")) {
-                        if (updateLoopExit(end, tok, tok->tokAt(2)->astOperand2()) == Progress::Break)
-                            return Break();
-                        tok = tok->linkAt(2);
-                    } else if (Token::simpleMatch(tok->next(), "else {")) {
-                        tok = tok->linkAt(2);
                     }
                 } else if (tok->isControlFlowKeyword() && Token::Match(tok, "if|while|for (") &&
-                           Token::simpleMatch(tok->next()->link(), ") {")) {
-                    Token* endCond = tok->next()->link();
-                    Token* endBlock = endCond->next()->link();
+                           Token::simpleMatch(tok->linkAt(1), ") {")) {
+                    if ((settings.vfOptions.maxForwardBranches > 0) && (++branchCount > settings.vfOptions.maxForwardBranches)) {
+                        // TODO: should be logged on function-level instead of file-level
+                        reportError(Severity::information, "normalCheckLevelMaxBranches", "Limiting analysis of branches. Use --check-level=exhaustive to analyze all branches.");
+                        return Break(Analyzer::Terminate::Bail);
+                    }
+                    Token* endCond = tok->linkAt(1);
+                    Token* endBlock = endCond->linkAt(1);
                     Token* condTok = getCondTok(tok);
                     Token* initTok = getInitTok(tok);
                     if (initTok && updateRecursive(initTok) == Progress::Break)
@@ -673,7 +699,7 @@ namespace {
                         Branch elseBranch{endBlock->tokAt(2) ? endBlock->linkAt(2) : nullptr};
                         // Check if condition is true or false
                         std::tie(thenBranch.check, elseBranch.check) = evalCond(condTok);
-                        if (!thenBranch.check && !elseBranch.check && analyzer->stopOnCondition(condTok) && stopUpdates())
+                        if (!thenBranch.check && !elseBranch.check && stopOnCondition(condTok) && stopUpdates())
                             return Break(Analyzer::Terminate::Conditional);
                         const bool hasElse = Token::simpleMatch(endBlock, "} else {");
                         bool bail = false;
@@ -682,7 +708,7 @@ namespace {
                         thenBranch.escape = isEscapeScope(endBlock, thenBranch.escapeUnknown);
                         if (thenBranch.check) {
                             thenBranch.active = true;
-                            if (updateRange(endCond->next(), endBlock, depth - 1) == Progress::Break)
+                            if (updateScope(endBlock, depth - 1) == Progress::Break)
                                 return Break();
                         } else if (!elseBranch.check) {
                             thenBranch.active = true;
@@ -694,7 +720,7 @@ namespace {
                             elseBranch.escape = isEscapeScope(endBlock->linkAt(2), elseBranch.escapeUnknown);
                             if (elseBranch.check) {
                                 elseBranch.active = true;
-                                const Progress result = updateRange(endBlock->tokAt(2), endBlock->linkAt(2), depth - 1);
+                                const Progress result = updateScope(endBlock->linkAt(2), depth - 1);
                                 if (result == Progress::Break)
                                     return Break();
                             } else if (!thenBranch.check) {
@@ -744,9 +770,9 @@ namespace {
                         }
                     }
                 } else if (Token::simpleMatch(tok, "try {")) {
-                    Token* endBlock = tok->next()->link();
+                    Token* endBlock = tok->linkAt(1);
                     ForwardTraversal tryTraversal = fork();
-                    tryTraversal.updateRange(tok->next(), endBlock, depth - 1);
+                    tryTraversal.updateScope(endBlock, depth - 1);
                     bool bail = tryTraversal.actions.isModified();
                     if (bail) {
                         actions = tryTraversal.actions;
@@ -760,14 +786,14 @@ namespace {
                             return Break();
                         endBlock = endCatch->linkAt(1);
                         ForwardTraversal ft = fork();
-                        ft.updateRange(endBlock->link(), endBlock, depth - 1);
+                        ft.updateScope(endBlock, depth - 1);
                         bail |= ft.terminate != Analyzer::Terminate::None || ft.actions.isModified();
                     }
                     if (bail)
                         return Break();
                     tok = endBlock;
                 } else if (Token::simpleMatch(tok, "do {")) {
-                    Token* endBlock = tok->next()->link();
+                    Token* endBlock = tok->linkAt(1);
                     Token* condTok = Token::simpleMatch(endBlock, "} while (") ? endBlock->tokAt(2)->astOperand2() : nullptr;
                     if (updateLoop(end, endBlock, condTok) == Progress::Break)
                         return Break();
@@ -786,6 +812,7 @@ namespace {
                 } else if (Token::simpleMatch(tok, "switch (")) {
                     if (updateRecursive(tok->next()->astOperand2()) == Progress::Break)
                         return Break();
+                    actions |= Analyzer::Action::Write; // bailout for switch scope
                     return Break();
                 } else if (Token* callTok = callExpr(tok)) {
                     // TODO: Dont traverse tokens a second time
@@ -817,6 +844,12 @@ namespace {
             return Progress::Continue;
         }
 
+        void reportError(Severity severity, const std::string& id, const std::string& msg) {
+            ErrorMessage::FileLocation loc(tokenList.getSourceFilePath(), 0, 0);
+            const ErrorMessage errmsg({std::move(loc)}, tokenList.getSourceFilePath(), severity, msg, id, Certainty::normal);
+            errorLogger.reportErr(errmsg);
+        }
+
         static bool isFunctionCall(const Token* tok)
         {
             if (!Token::simpleMatch(tok, "("))
@@ -875,22 +908,24 @@ namespace {
     };
 }
 
-Analyzer::Result valueFlowGenericForward(Token* start, const Token* end, const ValuePtr<Analyzer>& a, const Settings& settings)
+Analyzer::Result valueFlowGenericForward(Token* start, const Token* end, const ValuePtr<Analyzer>& a, const TokenList& tokenList, ErrorLogger& errorLogger, const Settings& settings)
 {
     if (a->invalid())
         return Analyzer::Result{Analyzer::Action::None, Analyzer::Terminate::Bail};
-    ForwardTraversal ft{a, settings};
+    ForwardTraversal ft{a, tokenList, errorLogger, settings};
+    if (start)
+        ft.analyzer->updateState(start);
     ft.updateRange(start, end);
     return Analyzer::Result{ ft.actions, ft.terminate };
 }
 
-Analyzer::Result valueFlowGenericForward(Token* start, const ValuePtr<Analyzer>& a, const Settings& settings)
+Analyzer::Result valueFlowGenericForward(Token* start, const ValuePtr<Analyzer>& a, const TokenList& tokenList, ErrorLogger& errorLogger, const Settings& settings)
 {
     if (Settings::terminated())
         throw TerminateException();
     if (a->invalid())
         return Analyzer::Result{Analyzer::Action::None, Analyzer::Terminate::Bail};
-    ForwardTraversal ft{a, settings};
-    ft.updateRecursive(start);
+    ForwardTraversal ft{a, tokenList, errorLogger, settings};
+    (void)ft.updateRecursive(start);
     return Analyzer::Result{ ft.actions, ft.terminate };
 }
