@@ -47,17 +47,17 @@ void ImportProject::ignorePaths(const std::vector<std::string> &ipaths)
     for (std::list<FileSettings>::iterator it = fileSettings.begin(); it != fileSettings.end();) {
         bool ignore = false;
         for (std::string i : ipaths) {
-            if (it->filename.size() > i.size() && it->filename.compare(0,i.size(),i)==0) {
+            if (it->filename().size() > i.size() && it->filename().compare(0,i.size(),i)==0) {
                 ignore = true;
                 break;
             }
-            if (isValidGlobPattern(i) && matchglob(i, it->filename)) {
+            if (isValidGlobPattern(i) && matchglob(i, it->filename())) {
                 ignore = true;
                 break;
             }
             if (!Path::isAbsolute(i)) {
                 i = mPath + i;
-                if (it->filename.size() > i.size() && it->filename.compare(0,i.size(),i)==0) {
+                if (it->filename().size() > i.size() && it->filename().compare(0,i.size(),i)==0) {
                     ignore = true;
                     break;
                 }
@@ -138,7 +138,7 @@ static bool simplifyPathWithVariables(std::string &s, std::map<std::string, std:
     }
     if (s.find("$(") != std::string::npos)
         return false;
-    s = Path::simplifyPath(Path::fromNativeSeparators(std::move(s)));
+    s = Path::simplifyPath(std::move(s));
     return true;
 }
 
@@ -164,7 +164,7 @@ void ImportProject::fsSetIncludePaths(FileSettings& fs, const std::string &basep
         }
 
         if (endsWith(s,'/')) // this is a temporary hack, simplifyPath can crash if path ends with '/'
-            s.erase(s.size() - 1U); // TODO: Use std::string::pop_back() as soon as travis supports it
+            s.pop_back();
 
         if (s.find("$(") == std::string::npos) {
             s = Path::simplifyPath(basepath + s);
@@ -174,7 +174,7 @@ void ImportProject::fsSetIncludePaths(FileSettings& fs, const std::string &basep
         }
         if (s.empty())
             continue;
-        fs.includePaths.push_back(s + '/');
+        fs.includePaths.push_back(s.back() == '/' ? s : (s + '/'));
     }
 }
 
@@ -203,7 +203,8 @@ ImportProject::Type ImportProject::import(const std::string &filename, Settings
         }
     } else if (endsWith(filename, ".vcxproj")) {
         std::map<std::string, std::string, cppcheck::stricmp> variables;
-        if (importVcxproj(filename, variables, emptyString, fileFilters)) {
+        std::vector<SharedItemsProject> sharedItemsProjects;
+        if (importVcxproj(filename, variables, emptyString, fileFilters, sharedItemsProjects)) {
             setRelativePaths(filename);
             return ImportProject::Type::VS_VCXPROJ;
         }
@@ -399,22 +400,23 @@ bool ImportProject::importCompileCommands(std::istream &istr)
         if (!Path::acceptFile(file))
             continue;
 
-        FileSettings fs;
+        std::string path;
         if (Path::isAbsolute(file))
-            fs.filename = Path::simplifyPath(file);
+            path = Path::simplifyPath(file);
 #ifdef _WIN32
         else if (file[0] == '/' && directory.size() > 2 && std::isalpha(directory[0]) && directory[1] == ':')
             // directory: C:\foo\bar
             // file: /xy/z.c
             // => c:/xy/z.c
-            fs.filename = Path::simplifyPath(directory.substr(0,2) + file);
+            path = Path::simplifyPath(directory.substr(0,2) + file);
 #endif
         else
-            fs.filename = Path::simplifyPath(directory + file);
-        if (!sourceFileExists(fs.filename)) {
-            printError("'" + fs.filename + "' from compilation database does not exist");
+            path = Path::simplifyPath(directory + file);
+        if (!sourceFileExists(path)) {
+            printError("'" + path + "' from compilation database does not exist");
             return false;
         }
+        FileSettings fs{std::move(path)};
         fsParseCommand(fs, command); // read settings; -D, -I, -U, -std, -m*, -f*
         std::map<std::string, std::string, cppcheck::stricmp> variables;
         fsSetIncludePaths(fs, directory, fs.includePaths, variables);
@@ -445,7 +447,7 @@ bool ImportProject::importSln(std::istream &istr, const std::string &path, const
     variables["SolutionDir"] = path;
 
     bool found = false;
-
+    std::vector<SharedItemsProject> sharedItemsProjects;
     while (std::getline(istr,line)) {
         if (!startsWith(line,"Project("))
             continue;
@@ -460,7 +462,7 @@ bool ImportProject::importSln(std::istream &istr, const std::string &path, const
         if (!Path::isAbsolute(vcxproj))
             vcxproj = path + vcxproj;
         vcxproj = Path::fromNativeSeparators(std::move(vcxproj));
-        if (!importVcxproj(vcxproj, variables, emptyString, fileFilters)) {
+        if (!importVcxproj(vcxproj, variables, emptyString, fileFilters, sharedItemsProjects)) {
             printError("failed to load '" + vcxproj + "' from Visual Studio solution");
             return false;
         }
@@ -499,7 +501,7 @@ namespace {
         }
         std::string name;
         std::string configuration;
-        enum { Win32, x64, Unknown } platform = Unknown;
+        enum : std::uint8_t { Win32, x64, Unknown } platform = Unknown;
         std::string platformStr;
     };
 
@@ -697,7 +699,7 @@ static void loadVisualStudioProperties(const std::string &props, std::map<std::s
     }
 }
 
-bool ImportProject::importVcxproj(const std::string &filename, std::map<std::string, std::string, cppcheck::stricmp> &variables, const std::string &additionalIncludeDirectories, const std::vector<std::string> &fileFilters)
+bool ImportProject::importVcxproj(const std::string &filename, std::map<std::string, std::string, cppcheck::stricmp> &variables, const std::string &additionalIncludeDirectories, const std::vector<std::string> &fileFilters, std::vector<SharedItemsProject> &cache)
 {
     variables["ProjectDir"] = Path::simplifyPath(Path::getPathFromFilename(filename));
 
@@ -705,6 +707,7 @@ bool ImportProject::importVcxproj(const std::string &filename, std::map<std::str
     std::list<std::string> compileList;
     std::list<ItemDefinitionGroup> itemDefinitionGroupList;
     std::string includePath;
+    std::vector<SharedItemsProject> sharedItemsProjects;
 
     bool useOfMfc = false;
 
@@ -736,8 +739,10 @@ bool ImportProject::importVcxproj(const std::string &filename, std::map<std::str
                 for (const tinyxml2::XMLElement *e = node->FirstChildElement(); e; e = e->NextSiblingElement()) {
                     if (std::strcmp(e->Name(), "ClCompile") == 0) {
                         const char *include = e->Attribute("Include");
-                        if (include && Path::acceptFile(include))
-                            compileList.emplace_back(include);
+                        if (include && Path::acceptFile(include)) {
+                            std::string toInclude = Path::simplifyPath(Path::isAbsolute(include) ? include : Path::getPathFromFilename(filename) + include);
+                            compileList.emplace_back(toInclude);
+                        }
                     }
                 }
             }
@@ -755,14 +760,54 @@ bool ImportProject::importVcxproj(const std::string &filename, std::map<std::str
                             loadVisualStudioProperties(projectAttribute, variables, includePath, additionalIncludeDirectories, itemDefinitionGroupList);
                     }
                 }
+            } else if (labelAttribute && std::strcmp(labelAttribute, "Shared") == 0) {
+                for (const tinyxml2::XMLElement *e = node->FirstChildElement(); e; e = e->NextSiblingElement()) {
+                    if (std::strcmp(e->Name(), "Import") == 0) {
+                        const char *projectAttribute = e->Attribute("Project");
+                        if (projectAttribute) {
+                            // Path to shared items project is relative to current project directory,
+                            // unless the string starts with $(SolutionDir)
+                            std::string pathToSharedItemsFile;
+                            if (std::string(projectAttribute).rfind("$(SolutionDir)", 0) == 0) {
+                                pathToSharedItemsFile = projectAttribute;
+                            } else {
+                                pathToSharedItemsFile = variables["ProjectDir"] + projectAttribute;
+                            }
+                            if (!simplifyPathWithVariables(pathToSharedItemsFile, variables)) {
+                                printError("Could not simplify path to referenced shared items project");
+                                return false;
+                            }
+
+                            SharedItemsProject toAdd = importVcxitems(pathToSharedItemsFile, fileFilters, cache);
+                            if (!toAdd.successful) {
+                                printError("Could not load shared items project \"" + pathToSharedItemsFile + "\" from original path \"" + std::string(projectAttribute) + "\".");
+                                return false;
+                            }
+                            sharedItemsProjects.emplace_back(toAdd);
+                        }
+                    }
+                }
             }
         }
     }
     // # TODO: support signedness of char via /J (and potential XML option for it)?
     // we can only set it globally but in this context it needs to be treated per file
 
-    for (const std::string &c : compileList) {
-        const std::string cfilename = Path::simplifyPath(Path::isAbsolute(c) ? c : Path::getPathFromFilename(filename) + c);
+    // Include shared items project files
+    std::vector<std::string> sharedItemsIncludePaths;
+    for (const auto& sharedProject : sharedItemsProjects) {
+        for (const auto &file : sharedProject.sourceFiles) {
+            std::string pathToFile = Path::simplifyPath(Path::getPathFromFilename(sharedProject.pathToProjectFile) + file);
+            compileList.emplace_back(std::move(pathToFile));
+        }
+        for (const auto &p : sharedProject.includePaths) {
+            std::string path = Path::simplifyPath(Path::getPathFromFilename(sharedProject.pathToProjectFile) + p);
+            sharedItemsIncludePaths.emplace_back(std::move(path));
+        }
+    }
+
+    // Project files
+    for (const std::string &cfilename : compileList) {
         if (!fileFilters.empty() && !matchglobs(fileFilters, cfilename))
             continue;
 
@@ -776,8 +821,7 @@ bool ImportProject::importVcxproj(const std::string &filename, std::map<std::str
                     continue;
             }
 
-            FileSettings fs;
-            fs.filename = cfilename;
+            FileSettings fs{cfilename};
             fs.cfg = p.name;
             // TODO: detect actual MSC version
             fs.msc = true;
@@ -809,6 +853,9 @@ bool ImportProject::importVcxproj(const std::string &filename, std::map<std::str
             }
             fsSetDefines(fs, fs.defines);
             fsSetIncludePaths(fs, Path::getPathFromFilename(filename), toStringList(includePath + ';' + additionalIncludePaths), variables);
+            for (const auto &path : sharedItemsIncludePaths) {
+                fs.includePaths.emplace_back(path);
+            }
             fileSettings.push_back(std::move(fs));
         }
     }
@@ -816,6 +863,68 @@ bool ImportProject::importVcxproj(const std::string &filename, std::map<std::str
     return true;
 }
 
+ImportProject::SharedItemsProject ImportProject::importVcxitems(const std::string& filename, const std::vector<std::string>& fileFilters, std::vector<SharedItemsProject> &cache)
+{
+    auto isInCacheCheck = [filename](const ImportProject::SharedItemsProject& e) -> bool {
+        return filename == e.pathToProjectFile;
+    };
+    const auto iterator = std::find_if(cache.begin(), cache.end(), isInCacheCheck);
+    if (iterator != std::end(cache)) {
+        return *iterator;
+    }
+
+    SharedItemsProject result;
+    result.pathToProjectFile = filename;
+
+    tinyxml2::XMLDocument doc;
+    const tinyxml2::XMLError error = doc.LoadFile(filename.c_str());
+    if (error != tinyxml2::XML_SUCCESS) {
+        printError(std::string("Visual Studio project file is not a valid XML - ") + tinyxml2::XMLDocument::ErrorIDToName(error));
+        return result;
+    }
+    const tinyxml2::XMLElement * const rootnode = doc.FirstChildElement();
+    if (rootnode == nullptr) {
+        printError("Visual Studio project file has no XML root node");
+        return result;
+    }
+    for (const tinyxml2::XMLElement *node = rootnode->FirstChildElement(); node; node = node->NextSiblingElement()) {
+        if (std::strcmp(node->Name(), "ItemGroup") == 0) {
+            for (const tinyxml2::XMLElement *e = node->FirstChildElement(); e; e = e->NextSiblingElement()) {
+                if (std::strcmp(e->Name(), "ClCompile") == 0) {
+                    const char* include = e->Attribute("Include");
+                    if (include && Path::acceptFile(include)) {
+                        std::string file(include);
+                        findAndReplace(file, "$(MSBuildThisFileDirectory)", "./");
+
+                        // Don't include file if it matches the filter
+                        if (!fileFilters.empty() && !matchglobs(fileFilters, file))
+                            continue;
+
+                        result.sourceFiles.emplace_back(file);
+                    } else {
+                        printError("Could not find shared items source file");
+                        return result;
+                    }
+                }
+            }
+        } else if (std::strcmp(node->Name(), "ItemDefinitionGroup") == 0) {
+            ItemDefinitionGroup temp(node, "");
+            for (const auto& includePath : toStringList(temp.additionalIncludePaths)) {
+                if (includePath == "%(AdditionalIncludeDirectories)")
+                    continue;
+
+                std::string toAdd(includePath);
+                findAndReplace(toAdd, "$(MSBuildThisFileDirectory)", "./");
+                result.includePaths.emplace_back(toAdd);
+            }
+        }
+    }
+
+    result.successful = true;
+    cache.emplace_back(result);
+    return result;
+}
+
 bool ImportProject::importBcb6Prj(const std::string &projectFilename)
 {
     tinyxml2::XMLDocument doc;
@@ -1072,10 +1181,9 @@ bool ImportProject::importBcb6Prj(const std::string &projectFilename)
         //
         // We can also force C++ compilation for all files using the -P command line switch.
         const bool cppMode = forceCppMode || Path::getFilenameExtensionInLowerCase(c) == ".cpp";
-        FileSettings fs;
+        FileSettings fs{Path::simplifyPath(Path::isAbsolute(c) ? c : projectDir + c)};
         fsSetIncludePaths(fs, projectDir, toStringList(includePath), variables);
         fsSetDefines(fs, cppMode ? cppDefines : defines);
-        fs.filename = Path::simplifyPath(Path::isAbsolute(c) ? c : projectDir + c);
         fileSettings.push_back(std::move(fs));
     }
 
@@ -1222,6 +1330,8 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
             checkLevelExhaustive = true;
         else if (strcmp(node->Name(), CppcheckXml::CheckUnusedTemplatesElementName) == 0)
             temp.checkUnusedTemplates = (strcmp(readSafe(node->GetText(), ""), "true") == 0);
+        else if (strcmp(node->Name(), CppcheckXml::InlineSuppression) == 0)
+            temp.inlineSuppressions = (strcmp(readSafe(node->GetText(), ""), "true") == 0);
         else if (strcmp(node->Name(), CppcheckXml::MaxCtuDepthElementName) == 0)
             temp.maxCtuDepth = strToInt<int>(readSafe(node->GetText(), "2")); // TODO: bail out when missing?
         else if (strcmp(node->Name(), CppcheckXml::MaxTemplateRecursionElementName) == 0)
@@ -1285,6 +1395,7 @@ bool ImportProject::importCppcheckGuiProject(std::istream &istr, Settings *setti
     settings->checkUnusedTemplates = temp.checkUnusedTemplates;
     settings->maxCtuDepth = temp.maxCtuDepth;
     settings->maxTemplateRecursion = temp.maxTemplateRecursion;
+    settings->inlineSuppressions |= temp.inlineSuppressions;
     settings->safeChecks = temp.safeChecks;
 
     if (checkLevelExhaustive)
@@ -1311,17 +1422,18 @@ void ImportProject::selectOneVsConfig(Platform::Type platform)
             remove = true;
         else if ((platform == Platform::Type::Win32A || platform == Platform::Type::Win32W) && fs.platformType == Platform::Type::Win64)
             remove = true;
-        else if (filenames.find(fs.filename) != filenames.end())
+        else if (filenames.find(fs.filename()) != filenames.end())
             remove = true;
         if (remove) {
             it = fileSettings.erase(it);
         } else {
-            filenames.insert(fs.filename);
+            filenames.insert(fs.filename());
             ++it;
         }
     }
 }
 
+// cppcheck-suppress unusedFunction - used by GUI only
 void ImportProject::selectVsConfigurations(Platform::Type platform, const std::vector<std::string> &configurations)
 {
     for (std::list<FileSettings>::iterator it = fileSettings.begin(); it != fileSettings.end();) {
@@ -1346,6 +1458,7 @@ void ImportProject::selectVsConfigurations(Platform::Type platform, const std::v
     }
 }
 
+// cppcheck-suppress unusedFunction - used by GUI only
 std::list<std::string> ImportProject::getVSConfigs()
 {
     return std::list<std::string>(mAllVSConfigs.cbegin(), mAllVSConfigs.cend());
@@ -1357,7 +1470,7 @@ void ImportProject::setRelativePaths(const std::string &filename)
         return;
     const std::vector<std::string> basePaths{Path::fromNativeSeparators(Path::getCurrentPath())};
     for (auto &fs: fileSettings) {
-        fs.filename = Path::getRelativePath(fs.filename, basePaths);
+        fs.file = FileWithDetails{Path::getRelativePath(fs.filename(), basePaths)};
         for (auto &includePath: fs.includePaths)
             includePath = Path::getRelativePath(includePath, basePaths);
     }
