@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -293,8 +293,10 @@ std::vector<ValueFlow::Value> infer(const ValuePtr<InferModel>& model,
         return !model->match(value);
     };
     lhsValues.remove_if(notMatch);
+    if (lhsValues.empty())
+        return result;
     rhsValues.remove_if(notMatch);
-    if (lhsValues.empty() || rhsValues.empty())
+    if (rhsValues.empty())
         return result;
 
     Interval lhs = Interval::fromValues(lhsValues);
