@@ -1,6 +1,6 @@
-/*
+/* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,18 +25,18 @@
 #include "mathlib.h"
 #include "standards.h"
 
+#include <array>
+#include <cstdint>
 #include <map>
 #include <memory>
 #include <set>
 #include <string>
 #include <unordered_map>
-#include <unordered_set>
 #include <utility>
 #include <vector>
 
 class Token;
-class Settings;
-enum class Severity;
+enum class Severity : std::uint8_t;
 
 namespace tinyxml2 {
     class XMLDocument;
@@ -50,16 +50,20 @@ namespace tinyxml2 {
  * @brief Library definitions handling
  */
 class CPPCHECKLIB Library {
-    // TODO: get rid of this
-    friend class TestSymbolDatabase; // For testing only
-    friend class TestSingleExecutorBase; // For testing only
-    friend class TestThreadExecutorBase; // For testing only
-    friend class TestProcessExecutorBase; // For testing only
+    friend struct LibraryHelper; // for testing
 
 public:
-    Library() = default;
+    Library();
+    ~Library();
 
-    enum class ErrorCode { OK, FILE_NOT_FOUND, BAD_XML, UNKNOWN_ELEMENT, MISSING_ATTRIBUTE, BAD_ATTRIBUTE_VALUE, UNSUPPORTED_FORMAT, DUPLICATE_PLATFORM_TYPE, PLATFORM_TYPE_REDEFINED };
+    Library(const Library& other);
+    Library& operator=(const Library& other) &;
+
+    enum class ErrorCode : std::uint8_t {
+        OK,
+        FILE_NOT_FOUND, BAD_XML, UNKNOWN_ELEMENT, MISSING_ATTRIBUTE, BAD_ATTRIBUTE_VALUE,
+        UNSUPPORTED_FORMAT, DUPLICATE_PLATFORM_TYPE, PLATFORM_TYPE_REDEFINED, DUPLICATE_DEFINE
+    };
 
     class Error {
     public:
@@ -71,13 +75,12 @@ public:
         std::string reason;
     };
 
-    Error load(const char exename[], const char path[]);
-    Error load(const tinyxml2::XMLDocument &doc);
+    Error load(const char exename[], const char path[], bool debug = false);
 
     struct AllocFunc {
         int groupId;
         int arg;
-        enum class BufferSize {none,malloc,calloc,strdup};
+        enum class BufferSize : std::uint8_t {none,malloc,calloc,strdup};
         BufferSize bufferSize;
         int bufferSizeArg1;
         int bufferSizeArg2;
@@ -105,30 +108,19 @@ public:
 
     // TODO: get rid of this
     /** get allocation info for function by name (deprecated, use other alloc) */
-    const AllocFunc* getAllocFuncInfo(const char name[]) const {
-        return getAllocDealloc(mAlloc, name);
-    }
+    const AllocFunc* getAllocFuncInfo(const char name[]) const;
 
     // TODO: get rid of this
     /** get deallocation info for function by name (deprecated, use other alloc) */
-    const AllocFunc* getDeallocFuncInfo(const char name[]) const {
-        return getAllocDealloc(mDealloc, name);
-    }
+    const AllocFunc* getDeallocFuncInfo(const char name[]) const;
 
     // TODO: get rid of this
     /** get allocation id for function by name (deprecated, use other alloc) */
-    // cppcheck-suppress unusedFunction
-    int allocId(const char name[]) const {
-        const AllocFunc* af = getAllocDealloc(mAlloc, name);
-        return af ? af->groupId : 0;
-    }
+    int allocId(const char name[]) const;
 
     // TODO: get rid of this
     /** get deallocation id for function by name (deprecated, use other alloc) */
-    int deallocId(const char name[]) const {
-        const AllocFunc* af = getAllocDealloc(mDealloc, name);
-        return af ? af->groupId : 0;
-    }
+    int deallocId(const char name[]) const;
 
     static bool isCompliantValidationExpression(const char* p);
 
@@ -158,6 +150,7 @@ public:
         int ptr2Arg;
         int sizeArg;
         int strlenArg;
+        int countArg;
     };
     const NonOverlappingData* getNonOverlappingData(const Token *ftok) const;
 
@@ -166,7 +159,7 @@ public:
         Standards standards;
         Severity severity;
     };
-    std::map<std::string, WarnInfo> functionwarn;
+    const std::map<std::string, WarnInfo>& functionwarn() const;
 
     const WarnInfo* getWarnInfo(const Token* ftok) const;
 
@@ -174,7 +167,7 @@ public:
     bool isNotLibraryFunction(const Token *ftok) const;
     bool matchArguments(const Token *ftok, const std::string &functionName) const;
 
-    enum class UseRetValType { NONE, DEFAULT, ERROR_CODE };
+    enum class UseRetValType : std::uint8_t { NONE, DEFAULT, ERROR_CODE };
     UseRetValType getUseRetValType(const Token* ftok) const;
 
     const std::string& returnValue(const Token *ftok) const;
@@ -191,7 +184,7 @@ public:
     public:
         Container() = default;
 
-        enum class Action {
+        enum class Action : std::uint8_t {
             RESIZE,
             CLEAR,
             PUSH,
@@ -200,12 +193,13 @@ public:
             FIND_CONST,
             INSERT,
             ERASE,
+            APPEND,
             CHANGE_CONTENT,
             CHANGE,
             CHANGE_INTERNAL,
             NO_ACTION
         };
-        enum class Yield {
+        enum class Yield : std::uint8_t {
             AT_INDEX,
             ITEM,
             BUFFER,
@@ -262,7 +256,7 @@ public:
         static Yield yieldFrom(const std::string& yieldName);
         static Action actionFrom(const std::string& actionName);
     };
-    std::unordered_map<std::string, Container> containers;
+    const std::unordered_map<std::string, Container>& containers() const;
     const Container* detectContainer(const Token* typeStart) const;
     const Container* detectIterator(const Token* typeStart) const;
     const Container* detectContainerOrIterator(const Token* typeStart, bool* isIterator = nullptr, bool withoutStd = false) const;
@@ -286,7 +280,7 @@ public:
         IteratorInfo iteratorInfo;
 
         struct MinSize {
-            enum class Type { NONE, STRLEN, ARGVALUE, SIZEOF, MUL, VALUE };
+            enum class Type : std::uint8_t { NONE, STRLEN, ARGVALUE, SIZEOF, MUL, VALUE };
             MinSize(Type t, int a) : type(t), arg(a) {}
             Type type;
             int arg;
@@ -296,13 +290,14 @@ public:
         };
         std::vector<MinSize> minsizes;
 
-        enum class Direction {
+        enum class Direction : std::uint8_t {
             DIR_IN,     ///< Input to called function. Data is treated as read-only.
             DIR_OUT,    ///< Output to caller. Data is passed by reference or address and is potentially written.
             DIR_INOUT,  ///< Input to called function, and output to caller. Data is passed by reference or address and is potentially modified.
             DIR_UNKNOWN ///< direction not known / specified
         };
-        Direction direction = Direction::DIR_UNKNOWN;
+        // argument directions up to ** indirect level (only one can be configured explicitly at the moment)
+        std::array<Direction, 3> direction = { { Direction::DIR_UNKNOWN, Direction::DIR_UNKNOWN, Direction::DIR_UNKNOWN } };
     };
 
     struct Function {
@@ -322,7 +317,7 @@ public:
     };
 
     const Function *getFunction(const Token *ftok) const;
-    std::unordered_map<std::string, Function> functions;
+    const std::unordered_map<std::string, Function>& functions() const;
     bool isUse(const std::string& functionName) const;
     bool isLeakIgnore(const std::string& functionName) const;
     bool isFunctionConst(const std::string& functionName, bool pure) const;
@@ -346,7 +341,7 @@ public:
         return arg && arg->strz;
     }
 
-    bool isIntArgValid(const Token *ftok, int argnr, const MathLib::bigint argvalue) const;
+    bool isIntArgValid(const Token *ftok, int argnr, MathLib::bigint argvalue) const;
     bool isFloatArgValid(const Token *ftok, int argnr, double argvalue) const;
 
     const std::string& validarg(const Token *ftok, int argnr) const {
@@ -366,15 +361,13 @@ public:
         return arg ? &arg->minsizes : nullptr;
     }
 
-    ArgumentChecks::Direction getArgDirection(const Token* ftok, int argnr) const;
+    ArgumentChecks::Direction getArgDirection(const Token* ftok, int argnr, int indirect = 0) const;
 
     bool markupFile(const std::string &path) const;
 
     bool processMarkupAfterCode(const std::string &path) const;
 
-    const std::set<std::string> &markupExtensions() const {
-        return mMarkupExtensions;
-    }
+    const std::set<std::string> &markupExtensions() const;
 
     bool reportErrors(const std::string &path) const;
 
@@ -389,60 +382,43 @@ public:
 
     bool iskeyword(const std::string &file, const std::string &keyword) const;
 
-    bool isexporter(const std::string &prefix) const {
-        return mExporters.find(prefix) != mExporters.end();
-    }
+    bool isexporter(const std::string &prefix) const;
 
-    bool isexportedprefix(const std::string &prefix, const std::string &token) const {
-        const std::map<std::string, ExportedFunctions>::const_iterator it = mExporters.find(prefix);
-        return (it != mExporters.end() && it->second.isPrefix(token));
-    }
+    bool isexportedprefix(const std::string &prefix, const std::string &token) const;
 
-    bool isexportedsuffix(const std::string &prefix, const std::string &token) const {
-        const std::map<std::string, ExportedFunctions>::const_iterator it = mExporters.find(prefix);
-        return (it != mExporters.end() && it->second.isSuffix(token));
-    }
+    bool isexportedsuffix(const std::string &prefix, const std::string &token) const;
 
     bool isimporter(const std::string& file, const std::string &importer) const;
 
     const Token* getContainerFromYield(const Token* tok, Container::Yield yield) const;
     const Token* getContainerFromAction(const Token* tok, Container::Action action) const;
 
-    bool isreflection(const std::string &token) const {
-        return mReflection.find(token) != mReflection.end();
-    }
+    static bool isContainerYield(const Token* cond, Library::Container::Yield y, const std::string& fallback = emptyString);
+    static Library::Container::Yield getContainerYield(const Token* cond);
 
-    int reflectionArgument(const std::string &token) const {
-        const std::map<std::string, int>::const_iterator it = mReflection.find(token);
-        if (it != mReflection.end())
-            return it->second;
-        return -1;
-    }
+    bool isreflection(const std::string &token) const;
 
-    bool isentrypoint(const std::string &func) const {
-        return func == "main" || mEntrypoints.find(func) != mEntrypoints.end();
-    }
+    int reflectionArgument(const std::string &token) const;
 
-    std::vector<std::string> defines; // to provide some library defines
+    bool isentrypoint(const std::string &func) const;
+
+    const std::set<std::string>& defines() const; // to provide some library defines
 
     struct SmartPointer {
         std::string name;
         bool unique = false;
     };
 
-    std::unordered_map<std::string, SmartPointer> smartPointers;
+    const std::unordered_map<std::string, SmartPointer>& smartPointers() const;
     bool isSmartPointer(const Token *tok) const;
     const SmartPointer* detectSmartPointer(const Token* tok, bool withoutStd = false) const;
 
     struct PodType {
         unsigned int size;
         char sign;
-        enum class Type { NO, BOOL, CHAR, SHORT, INT, LONG, LONGLONG } stdtype;
+        enum class Type : std::uint8_t { NO, BOOL, CHAR, SHORT, INT, LONG, LONGLONG } stdtype;
     };
-    const PodType *podtype(const std::string &name) const {
-        const std::unordered_map<std::string, PodType>::const_iterator it = mPodTypes.find(name);
-        return (it != mPodTypes.end()) ? &(it->second) : nullptr;
-    }
+    const PodType *podtype(const std::string &name) const;
 
     struct PlatformType {
         bool operator == (const PlatformType & type) const {
@@ -466,125 +442,31 @@ public:
         bool mConstPtr{};
     };
 
-    struct Platform {
-        const PlatformType *platform_type(const std::string &name) const {
-            const std::map<std::string, PlatformType>::const_iterator it = mPlatformTypes.find(name);
-            return (it != mPlatformTypes.end()) ? &(it->second) : nullptr;
-        }
-        std::map<std::string, PlatformType> mPlatformTypes;
-    };
-
-    const PlatformType *platform_type(const std::string &name, const std::string & platform) const {
-        const std::map<std::string, Platform>::const_iterator it = mPlatforms.find(platform);
-        if (it != mPlatforms.end()) {
-            const PlatformType * const type = it->second.platform_type(name);
-            if (type)
-                return type;
-        }
-
-        const std::map<std::string, PlatformType>::const_iterator it2 = mPlatformTypes.find(name);
-        return (it2 != mPlatformTypes.end()) ? &(it2->second) : nullptr;
-    }
+    const PlatformType *platform_type(const std::string &name, const std::string & platform) const;
 
     /**
      * Get function name for function call
      */
     std::string getFunctionName(const Token *ftok) const;
 
-    static bool isContainerYield(const Token * const cond, Library::Container::Yield y, const std::string& fallback=emptyString);
-
     /** Suppress/check a type */
-    enum class TypeCheck { def,
-                           check,
-                           suppress,
-                           checkFiniteLifetime, // (unusedvar) object has side effects, but immediate destruction is wrong
+    enum class TypeCheck : std::uint8_t {
+        def,
+        check,
+        suppress,
+        checkFiniteLifetime, // (unusedvar) object has side effects, but immediate destruction is wrong
     };
     TypeCheck getTypeCheck(std::string check, std::string typeName) const;
     bool hasAnyTypeCheck(const std::string& typeName) const;
 
 private:
+    Error load(const tinyxml2::XMLDocument &doc);
+
     // load a <function> xml node
-    Error loadFunction(const tinyxml2::XMLElement * const node, const std::string &name, std::set<std::string> &unknown_elements);
+    Error loadFunction(const tinyxml2::XMLElement * node, const std::string &name, std::set<std::string> &unknown_elements);
 
-    class ExportedFunctions {
-    public:
-        void addPrefix(std::string prefix) {
-            mPrefixes.insert(std::move(prefix));
-        }
-        void addSuffix(std::string suffix) {
-            mSuffixes.insert(std::move(suffix));
-        }
-        bool isPrefix(const std::string& prefix) const {
-            return (mPrefixes.find(prefix) != mPrefixes.end());
-        }
-        bool isSuffix(const std::string& suffix) const {
-            return (mSuffixes.find(suffix) != mSuffixes.end());
-        }
-
-    private:
-        std::set<std::string> mPrefixes;
-        std::set<std::string> mSuffixes;
-    };
-    class CodeBlock {
-    public:
-        CodeBlock() = default;
-
-        void setStart(const char* s) {
-            mStart = s;
-        }
-        void setEnd(const char* e) {
-            mEnd = e;
-        }
-        void setOffset(const int o) {
-            mOffset = o;
-        }
-        void addBlock(const char* blockName) {
-            mBlocks.insert(blockName);
-        }
-        const std::string& start() const {
-            return mStart;
-        }
-        const std::string& end() const {
-            return mEnd;
-        }
-        int offset() const {
-            return mOffset;
-        }
-        bool isBlock(const std::string& blockName) const {
-            return mBlocks.find(blockName) != mBlocks.end();
-        }
-
-    private:
-        std::string mStart;
-        std::string mEnd;
-        int mOffset{};
-        std::set<std::string> mBlocks;
-    };
-    enum class FalseTrueMaybe { False, True, Maybe };
-    int mAllocId{};
-    std::set<std::string> mFiles;
-    std::map<std::string, AllocFunc> mAlloc; // allocation functions
-    std::map<std::string, AllocFunc> mDealloc; // deallocation functions
-    std::map<std::string, AllocFunc> mRealloc; // reallocation functions
-    std::unordered_map<std::string, FalseTrueMaybe> mNoReturn; // is function noreturn?
-    std::map<std::string, std::string> mReturnValue;
-    std::map<std::string, std::string> mReturnValueType;
-    std::map<std::string, int> mReturnValueContainer;
-    std::map<std::string, std::vector<MathLib::bigint>> mUnknownReturnValues;
-    std::map<std::string, bool> mReportErrors;
-    std::map<std::string, bool> mProcessAfterCode;
-    std::set<std::string> mMarkupExtensions; // file extensions of markup files
-    std::map<std::string, std::set<std::string>> mKeywords;  // keywords for code in the library
-    std::unordered_map<std::string, CodeBlock> mExecutableBlocks; // keywords for blocks of executable code
-    std::map<std::string, ExportedFunctions> mExporters; // keywords that export variables/functions to libraries (meta-code/macros)
-    std::map<std::string, std::set<std::string>> mImporters;  // keywords that import variables/functions
-    std::map<std::string, int> mReflection; // invocation of reflection
-    std::unordered_map<std::string, PodType> mPodTypes; // pod types
-    std::map<std::string, PlatformType> mPlatformTypes; // platform independent typedefs
-    std::map<std::string, Platform> mPlatforms; // platform dependent typedefs
-    std::map<std::pair<std::string,std::string>, TypeCheck> mTypeChecks;
-    std::unordered_map<std::string, NonOverlappingData> mNonOverlappingData;
-    std::unordered_set<std::string> mEntrypoints;
+    struct LibraryData;
+    std::unique_ptr<LibraryData> mData;
 
     const ArgumentChecks * getarg(const Token *ftok, int argnr) const;
 
@@ -595,16 +477,12 @@ private:
         return (it == data.end()) ? nullptr : &it->second;
     }
 
-    enum DetectContainer { ContainerOnly, IteratorOnly, Both };
+    enum DetectContainer : std::uint8_t { ContainerOnly, IteratorOnly, Both };
     const Library::Container* detectContainerInternal(const Token* typeStart, DetectContainer detect, bool* isIterator = nullptr, bool withoutStd = false) const;
 };
 
 CPPCHECKLIB const Library::Container * getLibraryContainer(const Token * tok);
 
-std::shared_ptr<Token> createTokenFromExpression(const std::string& returnValue,
-                                                 const Settings* settings,
-                                                 std::unordered_map<nonneg int, const Token*>* lookupVarId = nullptr);
-
 /// @}
 //---------------------------------------------------------------------------
 #endif // libraryH
