@@ -1,6 +1,6 @@
-/*
+/* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,8 @@
 #include "standards.h"
 
 #include <climits>
+#include <cstddef>
+#include <cstdint>
 #include <stdexcept>
 #include <string>
 #include <vector>
@@ -88,21 +90,21 @@ public:
     nonneg int long_long_bit;  /// bits in long long
 
     /** size of standard types */
-    nonneg int sizeof_bool;
-    nonneg int sizeof_short;
-    nonneg int sizeof_int;
-    nonneg int sizeof_long;
-    nonneg int sizeof_long_long;
-    nonneg int sizeof_float;
-    nonneg int sizeof_double;
-    nonneg int sizeof_long_double;
-    nonneg int sizeof_wchar_t;
-    nonneg int sizeof_size_t;
-    nonneg int sizeof_pointer;
+    std::size_t sizeof_bool;
+    std::size_t sizeof_short;
+    std::size_t sizeof_int;
+    std::size_t sizeof_long;
+    std::size_t sizeof_long_long;
+    std::size_t sizeof_float;
+    std::size_t sizeof_double;
+    std::size_t sizeof_long_double;
+    std::size_t sizeof_wchar_t;
+    std::size_t sizeof_size_t;
+    std::size_t sizeof_pointer;
 
     char defaultSign;  // unsigned:'u', signed:'s', unknown:'\0'
 
-    enum Type {
+    enum Type : std::uint8_t {
         Unspecified, // No platform specified
         Native, // whatever system this code was compiled on
         Win32A,
@@ -120,16 +122,16 @@ public:
     bool set(Type t);
 
     /** set the platform type */
-    bool set(const std::string& platformstr, std::string& errstr, const std::vector<std::string>& paths = {}, bool verbose = false);
+    bool set(const std::string& platformstr, std::string& errstr, const std::vector<std::string>& paths = {}, bool debug = false);
 
     /**
      * load platform file
      * @param exename application path
      * @param filename platform filename
-     * @param verbose log verbose information about the lookup
+     * @param debug log verbose information about the lookup
      * @return returns true if file was loaded successfully
      */
-    bool loadFromFile(const char exename[], const std::string &filename, bool verbose = false);
+    bool loadFromFile(const char exename[], const std::string &filename, bool debug = false);
 
     /** load platform from xml document, primarily for testing */
     bool loadFromXmlDocument(const tinyxml2::XMLDocument *doc);
