@@ -43,24 +43,22 @@ static bool sameline(const simplecpp::Token *tok1, const simplecpp::Token *tok2)
     return tok1 && tok2 && tok1->location.sameline(tok2->location);
 }
 
-/**
- * Remove heading and trailing whitespaces from the input parameter.
- * If string is all spaces/tabs, return empty string.
- * @param s The string to trim.
- */
-static std::string trim(const std::string& s)
-{
-    const std::string::size_type beg = s.find_first_not_of(" \t");
-    if (beg == std::string::npos)
-        return "";
-    const std::string::size_type end = s.find_last_not_of(" \t");
-    return s.substr(beg, end - beg + 1);
-}
+Directive::Directive(const simplecpp::Location & _loc, std::string _str) :
+    file(_loc.file()),
+    linenr(_loc.line),
+    str(std::move(_str))
+{}
 
-Directive::Directive(std::string _file, const int _linenr, const std::string &_str) :
+Directive::Directive(std::string _file, const int _linenr, std::string _str) :
     file(std::move(_file)),
     linenr(_linenr),
-    str(trim(_str))
+    str(std::move(_str))
+{}
+
+Directive::DirectiveToken::DirectiveToken(const simplecpp::Token & _tok) :
+    line(_tok.location.line),
+    column(_tok.location.col),
+    tokStr(_tok.str())
 {}
 
 char Preprocessor::macroChar = char(1);
@@ -177,7 +175,7 @@ static std::string getRelativeFilename(const simplecpp::Token* tok, const Settin
             }
         }
     }
-    return Path::simplifyPath(relativeFilename);
+    return Path::simplifyPath(std::move(relativeFilename));
 }
 
 static void addInlineSuppressions(const simplecpp::TokenList &tokens, const Settings &settings, SuppressionList &suppressions, std::list<BadInlineSuppression> &bad)
@@ -342,7 +340,7 @@ std::list<Directive> Preprocessor::createDirectives(const simplecpp::TokenList &
                 continue;
             if (tok->next && tok->next->str() == "endfile")
                 continue;
-            Directive directive(tok->location.file(), tok->location.line, emptyString);
+            Directive directive(tok->location, emptyString);
             for (const simplecpp::Token *tok2 = tok; tok2 && tok2->location.line == directive.linenr; tok2 = tok2->next) {
                 if (tok2->comment)
                     continue;
@@ -352,6 +350,8 @@ std::list<Directive> Preprocessor::createDirectives(const simplecpp::TokenList &
                     directive.str += "include";
                 else
                     directive.str += tok2->str();
+
+                directive.strTokens.emplace_back(*tok2);
             }
             directives.push_back(std::move(directive));
         }
@@ -693,7 +693,7 @@ static simplecpp::DUI createDUI(const Settings &mSettings, const std::string &cf
     if (!cfg.empty())
         splitcfg(cfg, dui.defines, emptyString);
 
-    for (const std::string &def : mSettings.library.defines) {
+    for (const std::string &def : mSettings.library.defines()) {
         const std::string::size_type pos = def.find_first_of(" (");
         if (pos == std::string::npos) {
             dui.defines.push_back(def);
@@ -713,7 +713,7 @@ static simplecpp::DUI createDUI(const Settings &mSettings, const std::string &cf
     dui.includes = mSettings.userIncludes;  // --include
     // TODO: use mSettings.standards.stdValue instead
     // TODO: error out on unknown language?
-    const Standards::Language lang = Path::identify(filename);
+    const Standards::Language lang = Path::identify(filename, mSettings.cppHeaderProbe);
     if (lang == Standards::Language::CPP) {
         dui.std = mSettings.standards.getCPP();
         splitcfg(mSettings.platform.getLimitsDefines(Standards::getCPP(dui.std)), dui.defines, "");
@@ -966,13 +966,19 @@ std::size_t Preprocessor::calculateHash(const simplecpp::TokenList &tokens1, con
 {
     std::string hashData = toolinfo;
     for (const simplecpp::Token *tok = tokens1.cfront(); tok; tok = tok->next) {
-        if (!tok->comment)
+        if (!tok->comment) {
             hashData += tok->str();
+            hashData += static_cast<char>(tok->location.line);
+            hashData += static_cast<char>(tok->location.col);
+        }
     }
     for (std::map<std::string, simplecpp::TokenList *>::const_iterator it = mTokenLists.cbegin(); it != mTokenLists.cend(); ++it) {
         for (const simplecpp::Token *tok = it->second->cfront(); tok; tok = tok->next) {
-            if (!tok->comment)
+            if (!tok->comment) {
                 hashData += tok->str();
+                hashData += static_cast<char>(tok->location.line);
+                hashData += static_cast<char>(tok->location.col);
+            }
         }
     }
     return (std::hash<std::string>{})(hashData);
