@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,8 +24,10 @@
 #include "library.h"
 #include "mathlib.h"
 #include "settings.h"
+#include "standards.h"
 #include "symboldatabase.h"
 #include "token.h"
+#include "tokenlist.h"
 #include "utils.h"
 #include "valueflow.h"
 #include "valueptr.h"
@@ -34,14 +36,15 @@
 #include <cassert>
 #include <cmath>
 #include <functional>
+#include <iterator>
 #include <list>
 #include <memory>
+#include <stack>
+#include <sstream>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include <iostream>
-
 ExprIdToken::ExprIdToken(const Token* tok) : tok(tok), exprid(tok ? tok->exprId() : 0) {}
 
 nonneg int ExprIdToken::getExpressionId() const {
@@ -54,7 +57,9 @@ std::size_t ExprIdToken::Hash::operator()(ExprIdToken etok) const
 }
 
 void ProgramMemory::setValue(const Token* expr, const ValueFlow::Value& value) {
-    mValues[expr] = value;
+    copyOnWrite();
+
+    (*mValues)[expr] = value;
     ValueFlow::Value subvalue = value;
     const Token* subexpr = solveExprValue(
         expr,
@@ -68,12 +73,12 @@ void ProgramMemory::setValue(const Token* expr, const ValueFlow::Value& value) {
     },
         subvalue);
     if (subexpr)
-        mValues[subexpr] = subvalue;
+        (*mValues)[subexpr] = std::move(subvalue);
 }
 const ValueFlow::Value* ProgramMemory::getValue(nonneg int exprid, bool impossible) const
 {
-    const ProgramMemory::Map::const_iterator it = mValues.find(exprid);
-    const bool found = it != mValues.cend() && (impossible || !it->second.isImpossible());
+    const ProgramMemory::Map::const_iterator it = mValues->find(exprid);
+    const bool found = it != mValues->cend() && (impossible || !it->second.isImpossible());
     if (found)
         return &it->second;
     return nullptr;
@@ -98,11 +103,11 @@ void ProgramMemory::setIntValue(const Token* expr, MathLib::bigint value, bool i
     setValue(expr, v);
 }
 
-bool ProgramMemory::getTokValue(nonneg int exprid, const Token** result) const
+bool ProgramMemory::getTokValue(nonneg int exprid, const Token*& result) const
 {
     const ValueFlow::Value* value = getValue(exprid);
     if (value && value->isTokValue()) {
-        *result = value->tokvalue;
+        result = value->tokvalue;
         return true;
     }
     return false;
@@ -144,62 +149,85 @@ void ProgramMemory::setContainerSizeValue(const Token* expr, MathLib::bigint val
 }
 
 void ProgramMemory::setUnknown(const Token* expr) {
-    mValues[expr].valueType = ValueFlow::Value::ValueType::UNINIT;
+    copyOnWrite();
+
+    (*mValues)[expr].valueType = ValueFlow::Value::ValueType::UNINIT;
 }
 
 bool ProgramMemory::hasValue(nonneg int exprid)
 {
-    return mValues.find(exprid) != mValues.end();
+    return mValues->find(exprid) != mValues->end();
 }
 
 const ValueFlow::Value& ProgramMemory::at(nonneg int exprid) const {
-    return mValues.at(exprid);
+    return mValues->at(exprid);
 }
 ValueFlow::Value& ProgramMemory::at(nonneg int exprid) {
-    return mValues.at(exprid);
+    copyOnWrite();
+
+    return mValues->at(exprid);
 }
 
 void ProgramMemory::erase_if(const std::function<bool(const ExprIdToken&)>& pred)
 {
-    for (auto it = mValues.begin(); it != mValues.end();) {
+    if (mValues->empty())
+        return;
+
+    // TODO: how to delay until we actuallly modify?
+    copyOnWrite();
+
+    for (auto it = mValues->begin(); it != mValues->end();) {
         if (pred(it->first))
-            it = mValues.erase(it);
+            it = mValues->erase(it);
         else
             ++it;
     }
 }
 
-void ProgramMemory::swap(ProgramMemory &pm)
+void ProgramMemory::swap(ProgramMemory &pm) NOEXCEPT
 {
     mValues.swap(pm.mValues);
 }
 
 void ProgramMemory::clear()
 {
-    mValues.clear();
+    if (mValues->empty())
+        return;
+
+    copyOnWrite();
+
+    mValues->clear();
 }
 
 bool ProgramMemory::empty() const
 {
-    return mValues.empty();
+    return mValues->empty();
 }
 
-void ProgramMemory::replace(const ProgramMemory &pm)
+// NOLINTNEXTLINE(performance-unnecessary-value-param) - technically correct but we are moving the given values
+void ProgramMemory::replace(ProgramMemory pm)
 {
-    for (auto&& p : pm.mValues) {
-        mValues[p.first] = p.second;
+    if (pm.empty())
+        return;
+
+    copyOnWrite();
+
+    for (auto&& p : (*pm.mValues)) {
+        (*mValues)[p.first] = std::move(p.second);
     }
 }
 
-void ProgramMemory::insert(const ProgramMemory &pm)
+void ProgramMemory::copyOnWrite()
 {
-    for (auto&& p : pm)
-        mValues.insert(p);
+    if (mValues.use_count() == 1)
+        return;
+
+    mValues = std::make_shared<Map>(*mValues);
 }
 
-static ValueFlow::Value execute(const Token* expr, ProgramMemory& pm, const Settings* settings);
+static ValueFlow::Value execute(const Token* expr, ProgramMemory& pm, const Settings& settings);
 
-static bool evaluateCondition(MathLib::bigint r, const Token* condition, ProgramMemory& pm, const Settings* settings)
+static bool evaluateCondition(MathLib::bigint r, const Token* condition, ProgramMemory& pm, const Settings& settings)
 {
     if (!condition)
         return false;
@@ -209,12 +237,12 @@ static bool evaluateCondition(MathLib::bigint r, const Token* condition, Program
     return !error && result == r;
 }
 
-bool conditionIsFalse(const Token* condition, ProgramMemory pm, const Settings* settings)
+bool conditionIsFalse(const Token* condition, ProgramMemory pm, const Settings& settings)
 {
     return evaluateCondition(0, condition, pm, settings);
 }
 
-bool conditionIsTrue(const Token* condition, ProgramMemory pm, const Settings* settings)
+bool conditionIsTrue(const Token* condition, ProgramMemory pm, const Settings& settings)
 {
     return evaluateCondition(1, condition, pm, settings);
 }
@@ -272,9 +300,11 @@ static bool isBasicForLoop(const Token* tok)
     return true;
 }
 
-void programMemoryParseCondition(ProgramMemory& pm, const Token* tok, const Token* endTok, const Settings* settings, bool then)
+static void programMemoryParseCondition(ProgramMemory& pm, const Token* tok, const Token* endTok, const Settings& settings, bool then)
 {
     auto eval = [&](const Token* t) -> std::vector<MathLib::bigint> {
+        if (!t)
+            return std::vector<MathLib::bigint>{};
         if (t->hasKnownIntValue())
             return {t->values().front().intvalue};
         MathLib::bigint result = 0;
@@ -294,12 +324,12 @@ void programMemoryParseCondition(ProgramMemory& pm, const Token* tok, const Toke
             return;
         if (!truevalue.isIntValue())
             return;
-        if (endTok && findExpressionChanged(vartok, tok->next(), endTok, settings, true))
+        if (endTok && findExpressionChanged(vartok, tok->next(), endTok, settings))
             return;
         const bool impossible = (tok->str() == "==" && !then) || (tok->str() == "!=" && then);
         const ValueFlow::Value& v = then ? truevalue : falsevalue;
         pm.setValue(vartok, impossible ? asImpossible(v) : v);
-        const Token* containerTok = settings->library.getContainerFromYield(vartok, Library::Container::Yield::SIZE);
+        const Token* containerTok = settings.library.getContainerFromYield(vartok, Library::Container::Yield::SIZE);
         if (containerTok)
             pm.setContainerSizeValue(containerTok, v.intvalue, !impossible);
     } else if (Token::simpleMatch(tok, "!")) {
@@ -321,18 +351,17 @@ void programMemoryParseCondition(ProgramMemory& pm, const Token* tok, const Toke
             else
                 pm.setIntValue(tok, 0, then);
         }
-    } else if (tok->exprId() > 0) {
-        if (endTok && findExpressionChanged(tok, tok->next(), endTok, settings, true))
+    } else if (tok && tok->exprId() > 0) {
+        if (endTok && findExpressionChanged(tok, tok->next(), endTok, settings))
             return;
         pm.setIntValue(tok, 0, then);
-        assert(settings);
-        const Token* containerTok = settings->library.getContainerFromYield(tok, Library::Container::Yield::EMPTY);
+        const Token* containerTok = settings.library.getContainerFromYield(tok, Library::Container::Yield::EMPTY);
         if (containerTok)
             pm.setContainerSizeValue(containerTok, 0, then);
     }
 }
 
-static void fillProgramMemoryFromConditions(ProgramMemory& pm, const Scope* scope, const Token* endTok, const Settings* settings)
+static void fillProgramMemoryFromConditions(ProgramMemory& pm, const Scope* scope, const Token* endTok, const Settings& settings)
 {
     if (!scope)
         return;
@@ -352,12 +381,12 @@ static void fillProgramMemoryFromConditions(ProgramMemory& pm, const Scope* scop
     }
 }
 
-static void fillProgramMemoryFromConditions(ProgramMemory& pm, const Token* tok, const Settings* settings)
+static void fillProgramMemoryFromConditions(ProgramMemory& pm, const Token* tok, const Settings& settings)
 {
     fillProgramMemoryFromConditions(pm, tok->scope(), tok, settings);
 }
 
-static void fillProgramMemoryFromAssignments(ProgramMemory& pm, const Token* tok, const Settings* settings, const ProgramMemory& state, const ProgramMemory::Map& vars)
+static void fillProgramMemoryFromAssignments(ProgramMemory& pm, const Token* tok, const Settings& settings, const ProgramMemory& state, const ProgramMemory::Map& vars)
 {
     int indentlevel = 0;
     for (const Token *tok2 = tok; tok2; tok2 = tok2->previous()) {
@@ -380,7 +409,7 @@ static void fillProgramMemoryFromAssignments(ProgramMemory& pm, const Token* tok
                 }
             }
         } else if (tok2->exprId() > 0 && Token::Match(tok2, ".|(|[|*|%var%") && !pm.hasValue(tok2->exprId()) &&
-                   isVariableChanged(tok2, 0, settings, true)) {
+                   isVariableChanged(tok2, 0, settings)) {
             pm.setUnknown(tok2);
         }
 
@@ -417,44 +446,39 @@ static void fillProgramMemoryFromAssignments(ProgramMemory& pm, const Token* tok
     }
 }
 
-static void removeModifiedVars(ProgramMemory& pm, const Token* tok, const Token* origin)
+static void removeModifiedVars(ProgramMemory& pm, const Token* tok, const Token* origin, const Settings& settings)
 {
     pm.erase_if([&](const ExprIdToken& e) {
-        return isVariableChanged(origin, tok, e.getExpressionId(), false, nullptr, true);
+        return isVariableChanged(origin, tok, e.getExpressionId(), false, settings);
     });
 }
 
 static ProgramMemory getInitialProgramState(const Token* tok,
                                             const Token* origin,
-                                            const Settings* settings,
+                                            const Settings& settings,
                                             const ProgramMemory::Map& vars = ProgramMemory::Map {})
 {
     ProgramMemory pm;
     if (origin) {
-        fillProgramMemoryFromConditions(pm, origin, nullptr);
+        fillProgramMemoryFromConditions(pm, origin, settings);
         const ProgramMemory state = pm;
         fillProgramMemoryFromAssignments(pm, tok, settings, state, vars);
-        removeModifiedVars(pm, tok, origin);
+        removeModifiedVars(pm, tok, origin, settings);
     }
     return pm;
 }
 
-ProgramMemoryState::ProgramMemoryState(const Settings* s) : settings(s) {}
-
-void ProgramMemoryState::insert(const ProgramMemory &pm, const Token* origin)
+ProgramMemoryState::ProgramMemoryState(const Settings* s) : settings(s)
 {
-    if (origin)
-        for (auto&& p : pm)
-            origins.insert(std::make_pair(p.first.getExpressionId(), origin));
-    state.insert(pm);
+    assert(settings != nullptr);
 }
 
-void ProgramMemoryState::replace(const ProgramMemory &pm, const Token* origin)
+void ProgramMemoryState::replace(ProgramMemory pm, const Token* origin)
 {
     if (origin)
-        for (auto&& p : pm)
+        for (const auto& p : pm)
             origins[p.first.getExpressionId()] = origin;
-    state.replace(pm);
+    state.replace(std::move(pm));
 }
 
 static void addVars(ProgramMemory& pm, const ProgramMemory::Map& vars)
@@ -469,11 +493,11 @@ void ProgramMemoryState::addState(const Token* tok, const ProgramMemory::Map& va
 {
     ProgramMemory pm = state;
     addVars(pm, vars);
-    fillProgramMemoryFromConditions(pm, tok, settings);
+    fillProgramMemoryFromConditions(pm, tok, *settings);
     ProgramMemory local = pm;
-    fillProgramMemoryFromAssignments(pm, tok, settings, local, vars);
+    fillProgramMemoryFromAssignments(pm, tok, *settings, local, vars);
     addVars(pm, vars);
-    replace(pm, tok);
+    replace(std::move(pm), tok);
 }
 
 void ProgramMemoryState::assume(const Token* tok, bool b, bool isEmpty)
@@ -482,7 +506,7 @@ void ProgramMemoryState::assume(const Token* tok, bool b, bool isEmpty)
     if (isEmpty)
         pm.setContainerSizeValue(tok, 0, b);
     else
-        programMemoryParseCondition(pm, tok, nullptr, settings, b);
+        programMemoryParseCondition(pm, tok, nullptr, *settings, b);
     const Token* origin = tok;
     const Token* top = tok->astTop();
     if (top && Token::Match(top->previous(), "for|while|if (") && !Token::simpleMatch(tok->astParent(), "?")) {
@@ -491,23 +515,25 @@ void ProgramMemoryState::assume(const Token* tok, bool b, bool isEmpty)
             origin = origin->link();
         }
     }
-    replace(pm, origin);
+    replace(std::move(pm), origin);
 }
 
 void ProgramMemoryState::removeModifiedVars(const Token* tok)
 {
-    ProgramMemory pm = state;
+    const ProgramMemory& pm = state;
     auto eval = [&](const Token* cond) -> std::vector<MathLib::bigint> {
-        if (conditionIsTrue(cond, pm, settings))
+        ProgramMemory pm2 = pm;
+        auto result = execute(cond, pm2, *settings);
+        if (isTrue(result))
             return {1};
-        if (conditionIsFalse(cond, pm, settings))
+        if (isFalse(result))
             return {0};
         return {};
     };
     state.erase_if([&](const ExprIdToken& e) {
         const Token* start = origins[e.getExpressionId()];
         const Token* expr = e.tok;
-        if (!expr || findExpressionChangedSkipDeadCode(expr, start, tok, settings, true, eval)) {
+        if (!expr || findExpressionChangedSkipDeadCode(expr, start, tok, *settings, eval)) {
             origins.erase(e.getExpressionId());
             return true;
         }
@@ -533,7 +559,7 @@ ProgramMemory ProgramMemoryState::get(const Token* tok, const Token* ctx, const
     return local.state;
 }
 
-ProgramMemory getProgramMemory(const Token* tok, const Token* expr, const ValueFlow::Value& value, const Settings* settings)
+ProgramMemory getProgramMemory(const Token* tok, const Token* expr, const ValueFlow::Value& value, const Settings& settings)
 {
     ProgramMemory programMemory;
     programMemory.replace(getInitialProgramState(tok, value.tokvalue, settings));
@@ -576,7 +602,6 @@ static bool isIntegralValue(const ValueFlow::Value& value)
 static ValueFlow::Value evaluate(const std::string& op, const ValueFlow::Value& lhs, const ValueFlow::Value& rhs)
 {
     ValueFlow::Value result;
-    combineValueProperties(lhs, rhs, result);
     if (lhs.isImpossible() && rhs.isImpossible())
         return ValueFlow::Value::unknown();
     if (lhs.isImpossible() || rhs.isImpossible()) {
@@ -1221,14 +1246,14 @@ static std::vector<const Token*> setDifference(const std::vector<const Token*>&
 static bool evalSameCondition(const ProgramMemory& state,
                               const Token* storedValue,
                               const Token* cond,
-                              const Settings* settings)
+                              const Settings& settings)
 {
     assert(!conditionIsTrue(cond, state, settings));
     ProgramMemory pm = state;
     programMemoryParseCondition(pm, storedValue, nullptr, settings, true);
     if (pm == state)
         return false;
-    return conditionIsTrue(cond, pm, settings);
+    return conditionIsTrue(cond, std::move(pm), settings);
 }
 
 static void pruneConditions(std::vector<const Token*>& conds,
@@ -1256,7 +1281,10 @@ namespace {
         int fdepth = 4;
         int depth = 10;
 
-        explicit Executor(ProgramMemory* pm = nullptr, const Settings* settings = nullptr) : pm(pm), settings(settings) {}
+        Executor(ProgramMemory* pm, const Settings* settings) : pm(pm), settings(settings)
+        {
+            assert(settings != nullptr);
+        }
 
         static ValueFlow::Value unknown() {
             return ValueFlow::Value::unknown();
@@ -1271,9 +1299,10 @@ namespace {
                 ValueFlow::Value r = state.execute(tok);
                 if (r.isUninitValue())
                     continue;
-                result.insert(std::make_pair(tok->exprId(), r));
+                const bool brk = b && isTrueOrFalse(r, *b);
+                result.emplace(tok->exprId(), std::move(r));
                 // Short-circuit evaluation
-                if (b && isTrueOrFalse(r, *b))
+                if (brk)
                     break;
             }
             return result;
@@ -1297,7 +1326,7 @@ namespace {
         ValueFlow::Value executeMultiCondition(bool b, const Token* expr)
         {
             if (pm->hasValue(expr->exprId())) {
-                const ValueFlow::Value& v = pm->at(expr->exprId());
+                const ValueFlow::Value& v = utils::as_const(*pm).at(expr->exprId());
                 if (v.isIntValue())
                     return v;
             }
@@ -1366,7 +1395,7 @@ namespace {
                         continue;
                     for (const Token* cond1 : diffConditions1) {
                         auto it = std::find_if(diffConditions2.begin(), diffConditions2.end(), [&](const Token* cond2) {
-                            return evalSameCondition(*pm, cond2, cond1, settings);
+                            return evalSameCondition(*pm, cond2, cond1, *settings);
                         });
                         if (it == diffConditions2.end())
                             break;
@@ -1467,7 +1496,7 @@ namespace {
                 return lhs;
             } else if (expr->str() == "[" && expr->astOperand1() && expr->astOperand2()) {
                 const Token* tokvalue = nullptr;
-                if (!pm->getTokValue(expr->astOperand1()->exprId(), &tokvalue)) {
+                if (!pm->getTokValue(expr->astOperand1()->exprId(), tokvalue)) {
                     auto tokvalue_it = std::find_if(expr->astOperand1()->values().cbegin(),
                                                     expr->astOperand1()->values().cend(),
                                                     std::mem_fn(&ValueFlow::Value::isTokValue));
@@ -1495,17 +1524,17 @@ namespace {
                 if (!lhs.isUninitValue() && !rhs.isUninitValue())
                     r = evaluate(expr->str(), lhs, rhs);
                 if (expr->isComparisonOp() && (r.isUninitValue() || r.isImpossible())) {
-                    if (rhs.isIntValue()) {
+                    if (rhs.isIntValue() && !expr->astOperand1()->values().empty()) {
                         std::vector<ValueFlow::Value> result =
-                            infer(ValueFlow::makeIntegralInferModel(), expr->str(), expr->astOperand1()->values(), {rhs});
+                            infer(ValueFlow::makeIntegralInferModel(), expr->str(), expr->astOperand1()->values(), {std::move(rhs)});
                         if (!result.empty() && result.front().isKnown())
-                            return result.front();
+                            return std::move(result.front());
                     }
-                    if (lhs.isIntValue()) {
+                    if (lhs.isIntValue() && !expr->astOperand2()->values().empty()) {
                         std::vector<ValueFlow::Value> result =
-                            infer(ValueFlow::makeIntegralInferModel(), expr->str(), {lhs}, expr->astOperand2()->values());
+                            infer(ValueFlow::makeIntegralInferModel(), expr->str(), {std::move(lhs)}, expr->astOperand2()->values());
                         if (!result.empty() && result.front().isKnown())
-                            return result.front();
+                            return std::move(result.front());
                     }
                     return unknown();
                 }
@@ -1542,13 +1571,16 @@ namespace {
 
                 return unknown();
             } else if (expr->str() == "(" && expr->isCast()) {
-                if (Token::simpleMatch(expr->previous(), ">") && expr->previous()->link())
-                    return execute(expr->astOperand2());
+                if (expr->astOperand2()) {
+                    if (expr->astOperand1()->str() != "dynamic_cast")
+                        return execute(expr->astOperand2());
+                    return unknown();
+                }
                 return execute(expr->astOperand1());
             }
             if (expr->exprId() > 0 && pm->hasValue(expr->exprId())) {
-                ValueFlow::Value result = pm->at(expr->exprId());
-                if (result.isImpossible() && result.isIntValue() && result.intvalue == 0 && isUsedAsBool(expr)) {
+                ValueFlow::Value result = utils::as_const(*pm).at(expr->exprId());
+                if (result.isImpossible() && result.isIntValue() && result.intvalue == 0 && isUsedAsBool(expr, *settings)) {
                     result.intvalue = !result.intvalue;
                     result.setKnown();
                 }
@@ -1580,7 +1612,7 @@ namespace {
                             ex.fdepth--;
                             auto r = ex.execute(f->functionScope);
                             if (!r.empty())
-                                result = r.front();
+                                result = std::move(r.front());
                             // TODO: Track values changed by reference
                         }
                     } else {
@@ -1596,7 +1628,7 @@ namespace {
                                     arg_map[argn] = v;
                                 argn++;
                             }
-                            return evaluateLibraryFunction(arg_map, returnValue, settings);
+                            return evaluateLibraryFunction(arg_map, returnValue, *settings, ftok->isCpp());
                         }
                     }
                 }
@@ -1604,11 +1636,12 @@ namespace {
                 visitAstNodes(expr->astOperand2(), [&](const Token* child) {
                     if (child->exprId() > 0 && pm->hasValue(child->exprId())) {
                         ValueFlow::Value& v = pm->at(child->exprId());
+                        assert(settings != nullptr);
                         if (v.valueType == ValueFlow::Value::ValueType::CONTAINER_SIZE) {
-                            if (ValueFlow::isContainerSizeChanged(child, v.indirect, settings))
+                            if (ValueFlow::isContainerSizeChanged(child, v.indirect, *settings))
                                 v = unknown();
                         } else if (v.valueType != ValueFlow::Value::ValueType::UNINIT) {
-                            if (isVariableChanged(child, v.indirect, settings, true))
+                            if (isVariableChanged(child, v.indirect, *settings))
                                 v = unknown();
                         }
                     }
@@ -1640,6 +1673,14 @@ namespace {
             return *it;
         }
 
+        static bool updateValue(ValueFlow::Value& v, ValueFlow::Value x)
+        {
+            const bool returnValue = !x.isUninitValue() && !x.isImpossible();
+            if (v.isUninitValue() || returnValue)
+                v = std::move(x);
+            return returnValue;
+        }
+
         ValueFlow::Value execute(const Token* expr)
         {
             depth--;
@@ -1648,13 +1689,31 @@ namespace {
                 }};
             if (depth < 0)
                 return unknown();
-            ValueFlow::Value v = executeImpl(expr);
-            if (!v.isUninitValue())
+            ValueFlow::Value v = unknown();
+            if (updateValue(v, executeImpl(expr)))
                 return v;
             if (!expr)
                 return v;
-            if (expr->exprId() > 0 && pm->hasValue(expr->exprId()))
-                return pm->at(expr->exprId());
+            if (expr->exprId() > 0 && pm->hasValue(expr->exprId())) {
+                if (updateValue(v, utils::as_const(*pm).at(expr->exprId())))
+                    return v;
+            }
+            // Find symbolic values
+            for (const ValueFlow::Value& value : expr->values()) {
+                if (!value.isSymbolicValue())
+                    continue;
+                if (!value.isKnown())
+                    continue;
+                if (value.tokvalue->exprId() > 0 && !pm->hasValue(value.tokvalue->exprId()))
+                    continue;
+                ValueFlow::Value v2 = utils::as_const(*pm).at(value.tokvalue->exprId());
+                if (!v2.isIntValue() && value.intvalue != 0)
+                    continue;
+                v2.intvalue += value.intvalue;
+                return v2;
+            }
+            if (v.isImpossible() && v.isIntValue())
+                return v;
             if (const ValueFlow::Value* value = getImpossibleValue(expr))
                 return *value;
             return v;
@@ -1714,32 +1773,81 @@ namespace {
     };
 }     // namespace
 
-static ValueFlow::Value execute(const Token* expr, ProgramMemory& pm, const Settings* settings)
+static ValueFlow::Value execute(const Token* expr, ProgramMemory& pm, const Settings& settings)
 {
-    Executor ex{&pm, settings};
+    Executor ex{&pm, &settings};
     return ex.execute(expr);
 }
 
-std::vector<ValueFlow::Value> execute(const Scope* scope, ProgramMemory& pm, const Settings* settings)
+std::vector<ValueFlow::Value> execute(const Scope* scope, ProgramMemory& pm, const Settings& settings)
 {
-    Executor ex{&pm, settings};
+    Executor ex{&pm, &settings};
     return ex.execute(scope);
 }
 
+static std::shared_ptr<Token> createTokenFromExpression(const std::string& returnValue,
+                                                        const Settings& settings,
+                                                        bool cpp,
+                                                        std::unordered_map<nonneg int, const Token*>& lookupVarId)
+{
+    std::shared_ptr<TokenList> tokenList = std::make_shared<TokenList>(&settings);
+    {
+        const std::string code = "return " + returnValue + ";";
+        std::istringstream istr(code);
+        if (!tokenList->createTokens(istr, cpp ? Standards::Language::CPP : Standards::Language::C))
+            return nullptr;
+    }
+
+    // TODO: put in a helper?
+    // combine operators, set links, etc..
+    std::stack<Token*> lpar;
+    for (Token* tok2 = tokenList->front(); tok2; tok2 = tok2->next()) {
+        if (Token::Match(tok2, "[!<>=] =")) {
+            tok2->str(tok2->str() + "=");
+            tok2->deleteNext();
+        } else if (tok2->str() == "(")
+            lpar.push(tok2);
+        else if (tok2->str() == ")") {
+            if (lpar.empty())
+                return nullptr;
+            Token::createMutualLinks(lpar.top(), tok2);
+            lpar.pop();
+        }
+    }
+    if (!lpar.empty())
+        return nullptr;
+
+    // set varids
+    for (Token* tok2 = tokenList->front(); tok2; tok2 = tok2->next()) {
+        if (!startsWith(tok2->str(), "arg"))
+            continue;
+        nonneg int const id = strToInt<nonneg int>(tok2->str().c_str() + 3);
+        tok2->varId(id);
+        lookupVarId[id] = tok2;
+    }
+
+    // Evaluate expression
+    tokenList->createAst();
+    Token* expr = tokenList->front()->astOperand1();
+    ValueFlow::valueFlowConstantFoldAST(expr, settings);
+    return {tokenList, expr};
+}
+
 ValueFlow::Value evaluateLibraryFunction(const std::unordered_map<nonneg int, ValueFlow::Value>& args,
                                          const std::string& returnValue,
-                                         const Settings* settings)
+                                         const Settings& settings,
+                                         bool cpp)
 {
     thread_local static std::unordered_map<std::string,
-                                           std::function<ValueFlow::Value(const std::unordered_map<nonneg int, ValueFlow::Value>& arg)>>
+                                           std::function<ValueFlow::Value(const std::unordered_map<nonneg int, ValueFlow::Value>&, const Settings&)>>
     functions = {};
     if (functions.count(returnValue) == 0) {
 
         std::unordered_map<nonneg int, const Token*> lookupVarId;
-        std::shared_ptr<Token> expr = createTokenFromExpression(returnValue, settings, &lookupVarId);
+        std::shared_ptr<Token> expr = createTokenFromExpression(returnValue, settings, cpp, lookupVarId);
 
         functions[returnValue] =
-            [lookupVarId, expr, settings](const std::unordered_map<nonneg int, ValueFlow::Value>& xargs) {
+            [lookupVarId, expr](const std::unordered_map<nonneg int, ValueFlow::Value>& xargs, const Settings& settings) {
             if (!expr)
                 return ValueFlow::Value::unknown();
             ProgramMemory pm{};
@@ -1751,14 +1859,14 @@ ValueFlow::Value evaluateLibraryFunction(const std::unordered_map<nonneg int, Va
             return execute(expr.get(), pm, settings);
         };
     }
-    return functions.at(returnValue)(args);
+    return functions.at(returnValue)(args, settings);
 }
 
 void execute(const Token* expr,
              ProgramMemory& programMemory,
              MathLib::bigint* result,
              bool* error,
-             const Settings* settings)
+             const Settings& settings)
 {
     ValueFlow::Value v = execute(expr, programMemory, settings);
     if (!v.isIntValue() || v.isImpossible()) {
