@@ -1,4 +1,4 @@
-/*
+/* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
  * Copyright (C) 2007-2024 Cppcheck team.
  *
@@ -41,8 +41,9 @@
 #include <tuple>
 #include <vector>
 #include <unordered_set>
+#include <utility>
 
-enum class SHOWTIME_MODES;
+enum class SHOWTIME_MODES : std::uint8_t;
 namespace ValueFlow {
     class Value;
 }
@@ -101,7 +102,7 @@ private:
 public:
     Settings();
 
-    static std::string loadCppcheckCfg(Settings& settings, Suppressions& suppressions);
+    static std::string loadCppcheckCfg(Settings& settings, Suppressions& suppressions, bool debug = false);
 
     static std::pair<std::string, std::string> getNameAndVersion(const std::string& productName);
 
@@ -167,9 +168,27 @@ public:
     /** cppcheck.cfg: About text */
     std::string cppcheckCfgAbout;
 
+    /** @brief check Emacs marker to detect extension-less and *.h files as C++ */
+    bool cppHeaderProbe{};
+
     /** @brief Are we running from DACA script? */
     bool daca{};
 
+    /** @brief Internal: Is --debug-lookup or --debug-lookup=all given? */
+    bool debuglookup{};
+
+    /** @brief Internal: Is --debug-lookup=addon given? */
+    bool debuglookupAddon{};
+
+    /** @brief Internal: Is --debug-lookup=config given? */
+    bool debuglookupConfig{};
+
+    /** @brief Internal: Is --debug-lookup=library given? */
+    bool debuglookupLibrary{};
+
+    /** @brief Internal: Is --debug-lookup=platform given? */
+    bool debuglookupPlatform{};
+
     /** @brief Is --debug-normal given? */
     bool debugnormal{};
 
@@ -194,7 +213,7 @@ public:
     bool exceptionHandling{};
 #endif
 
-    enum class ExecutorType
+    enum class ExecutorType : std::uint8_t
     {
 #ifdef HAS_THREADING_MODEL_THREAD
         Thread,
@@ -254,15 +273,6 @@ public:
 
     Platform platform;
 
-    /** @brief Experimental: --performance-valueflow-max-time=T */
-    int performanceValueFlowMaxTime = -1;
-
-    /** @brief --performance-valueflow-max-if-count=C */
-    int performanceValueFlowMaxIfCount = -1;
-
-    /** @brief max number of sets of arguments to pass to subfuncions in valueflow */
-    int performanceValueFlowMaxSubFunctionArgs = 256;
-
     /** @brief pid of cppcheck. Intention is that this is set in the main process. */
     int pid;
 
@@ -391,8 +401,42 @@ public:
     /** @brief forced includes given by the user */
     std::list<std::string> userIncludes;
 
-    /** @brief the maximum iterations of valueflow (--valueflow-max-iterations=T) */
-    std::size_t valueFlowMaxIterations = 4;
+    // TODO: adjust all options so 0 means "disabled" and -1 "means "unlimited"
+    struct ValueFlowOptions
+    {
+        /** @brief the maximum iterations to execute */
+        std::size_t maxIterations = 4;
+
+        /** @brief maximum numer if-branches */
+        int maxIfCount = -1;
+
+        /** @brief maximum number of sets of arguments to pass to subfuncions */
+        int maxSubFunctionArgs = 256;
+
+        /** @brief Experimental: maximum execution time */
+        int maxTime = -1;
+
+        /** @brief Control if condition expression analysis is performed */
+        bool doConditionExpressionAnalysis = true;
+
+        /** @brief Maximum performed for-loop count */
+        int maxForLoopCount = 10000;
+
+        /** @brief Maximum performed forward branches */
+        int maxForwardBranches = -1;
+
+        /** @brief Maximum performed alignof recursion */
+        int maxAlignOfRecursion = 100;
+
+        /** @brief Maximum performed sizeof recursion */
+        int maxSizeOfRecursion = 100;
+
+        /** @brief Maximum expression varid depth */
+        int maxExprVarIdDepth = 4;
+    };
+
+    /** @brief The ValueFlow options */
+    ValueFlowOptions vfOptions;
 
     /** @brief Is --verbose given? */
     bool verbose{};
@@ -458,7 +502,7 @@ public:
         return jobs == 1;
     }
 
-    enum class CheckLevel {
+    enum class CheckLevel : std::uint8_t {
         normal,
         exhaustive
     };
