@@ -1,4 +1,4 @@
-/*
+/* -*- C++ -*-
  * Cppcheck - A tool for static C/C++ code analysis
  * Copyright (C) 2007-2024 Cppcheck team.
  *
@@ -31,6 +31,7 @@
 
 #include <algorithm>
 #include <cctype>
+#include <cstdint>
 #include <iosfwd>
 #include <list>
 #include <map>
@@ -51,7 +52,7 @@ class ValueType;
 /**
  * @brief Access control enumerations.
  */
-enum class AccessControl { Public, Protected, Private, Global, Namespace, Argument, Local, Throw };
+enum class AccessControl : std::uint8_t { Public, Protected, Private, Global, Namespace, Argument, Local, Throw };
 
 /**
  * @brief Array dimension information.
@@ -68,7 +69,7 @@ public:
     const Token* classDef;     ///< Points to "class" token
     const Scope* classScope;
     const Scope* enclosingScope;
-    enum class NeedInitialization {
+    enum class NeedInitialization : std::uint8_t {
         Unknown, True, False
     } needInitialization = NeedInitialization::Unknown;
 
@@ -106,7 +107,7 @@ public:
         else if (classDef_ && classDef_->str() == "using") {
             typeStart = classDef->tokAt(3);
             typeEnd = typeStart;
-            while (typeEnd->next() && typeEnd->next()->str() != ";") {
+            while (typeEnd->next() && typeEnd->strAt(1) != ";") {
                 if (Token::simpleMatch(typeEnd, "decltype ("))
                     typeEnd = typeEnd->linkAt(1);
                 else
@@ -213,7 +214,7 @@ class CPPCHECKLIB Variable {
 public:
     Variable(const Token *name_, const Token *start_, const Token *end_,
              nonneg int index_, AccessControl access_, const Type *type_,
-             const Scope *scope_, const Settings* settings)
+             const Scope *scope_, const Settings& settings)
         : mNameToken(name_),
         mTypeStartToken(start_),
         mTypeEndToken(end_),
@@ -235,7 +236,7 @@ public:
 
     ~Variable();
 
-    Variable &operator=(const Variable &var);
+    Variable &operator=(const Variable &var) &;
 
     /**
      * Get name token.
@@ -692,7 +693,7 @@ private:
     std::vector<Dimension> mDimensions;
 
     /** @brief fill in information, depending on Tokens given at instantiation */
-    void evaluate(const Settings* settings);
+    void evaluate(const Settings& settings);
 };
 
 class CPPCHECKLIB Function {
@@ -747,7 +748,7 @@ class CPPCHECKLIB Function {
     }
 
 public:
-    enum Type { eConstructor, eCopyConstructor, eMoveConstructor, eOperatorEqual, eDestructor, eFunction, eLambda };
+    enum Type : std::uint8_t { eConstructor, eCopyConstructor, eMoveConstructor, eOperatorEqual, eDestructor, eFunction, eLambda };
 
     Function(const Token *tok, const Scope *scope, const Token *tokDef, const Token *tokArgDef);
     Function(const Token *tokenDef, const std::string &clangType);
@@ -1021,7 +1022,7 @@ public:
         const Scope *scope;
     };
 
-    enum ScopeType { eGlobal, eClass, eStruct, eUnion, eNamespace, eFunction, eIf, eElse, eFor, eWhile, eDo, eSwitch, eUnconditional, eTry, eCatch, eLambda, eEnum };
+    enum ScopeType : std::uint8_t { eGlobal, eClass, eStruct, eUnion, eNamespace, eFunction, eIf, eElse, eFor, eWhile, eDo, eSwitch, eUnconditional, eTry, eCatch, eLambda, eEnum };
 
     Scope(const SymbolDatabase *check_, const Token *classDef_, const Scope *nestedIn_);
     Scope(const SymbolDatabase *check_, const Token *classDef_, const Scope *nestedIn_, ScopeType type_, const Token *start_);
@@ -1119,7 +1120,7 @@ public:
     }
 
     // Is there lambda/inline function(s) in this scope?
-    bool hasInlineOrLambdaFunction() const;
+    bool hasInlineOrLambdaFunction(const Token** tokStart = nullptr) const;
 
     /**
      * @brief find a function
@@ -1143,7 +1144,7 @@ public:
 
     void addVariable(const Token *token_, const Token *start_,
                      const Token *end_, AccessControl access_, const Type *type_,
-                     const Scope *scope_, const Settings* settings);
+                     const Scope *scope_, const Settings& settings);
 
     /** @brief initialize varlist */
     void getVariableList(const Settings& settings);
@@ -1190,7 +1191,7 @@ private:
      * @param typetok populated with pointer to the type token, if found
      * @return true if tok points to a variable declaration, false otherwise
      */
-    bool isVariableDeclaration(const Token* const tok, const Token*& vartok, const Token*& typetok) const;
+    bool isVariableDeclaration(const Token* tok, const Token*& vartok, const Token*& typetok) const;
 
     void findFunctionInBase(const std::string & name, nonneg int args, std::vector<const Function *> & matches) const;
 
@@ -1198,7 +1199,7 @@ private:
     void getVariableList(const Settings& settings, const Token *start, const Token *end);
 };
 
-enum class Reference {
+enum class Reference : std::uint8_t {
     None,
     LValue,
     RValue
@@ -1207,8 +1208,8 @@ enum class Reference {
 /** Value type */
 class CPPCHECKLIB ValueType {
 public:
-    enum Sign { UNKNOWN_SIGN, SIGNED, UNSIGNED } sign = UNKNOWN_SIGN;
-    enum Type {
+    enum Sign : std::uint8_t { UNKNOWN_SIGN, SIGNED, UNSIGNED } sign = UNKNOWN_SIGN;
+    enum Type : std::uint8_t {
         UNKNOWN_TYPE,
         POD,
         NONSTD,
@@ -1278,7 +1279,7 @@ public:
 
     static Type typeFromString(const std::string &typestr, bool longType);
 
-    enum class MatchResult { UNKNOWN, SAME, FALLBACK1, FALLBACK2, NOMATCH };
+    enum class MatchResult : std::uint8_t { UNKNOWN, SAME, FALLBACK1, FALLBACK2, NOMATCH };
     static MatchResult matchParameter(const ValueType *call, const ValueType *func);
     static MatchResult matchParameter(const ValueType *call, const Variable *callVar, const Variable *funcVar);
 
@@ -1355,12 +1356,12 @@ public:
     const Type* findType(const Token* startTok, const Scope* startScope, bool lookOutside = false) const;
     Type* findType(const Token* startTok, Scope* startScope, bool lookOutside = false)
     {
-        return const_cast<Type*>(this->findType(startTok, const_cast<const Scope*>(startScope), lookOutside));
+        return const_cast<Type*>(this->findType(startTok, static_cast<const Scope*>(startScope), lookOutside));
     }
 
     const Scope *findScope(const Token *tok, const Scope *startScope) const;
     Scope *findScope(const Token *tok, Scope *startScope) {
-        return const_cast<Scope *>(this->findScope(tok, const_cast<const Scope *>(startScope)));
+        return const_cast<Scope *>(this->findScope(tok, static_cast<const Scope *>(startScope)));
     }
 
     // cppcheck-suppress unusedFunction
