@@ -32,6 +32,7 @@
 #include "token.h"
 
 #include <cctype>
+#include <cstdint>
 #include <exception>
 #include <functional>
 #include <utility>
@@ -95,7 +96,8 @@ void TokenList::determineCppC()
 {
     // only try to determine if it wasn't enforced
     if (mLang == Standards::Language::None) {
-        mLang = Path::identify(getSourceFilePath());
+        ASSERT_LANG(!getSourceFilePath().empty());
+        mLang = Path::identify(getSourceFilePath(), mSettings ? mSettings->cppHeaderProbe : false);
         // TODO: cannot enable assert as this might occur for unknown extensions
         //ASSERT_LANG(mLang != Standards::Language::None);
         if (mLang == Standards::Language::None) {
@@ -302,6 +304,7 @@ Token *TokenList::copyTokens(Token *dest, const Token *first, const Token *last,
 
 void TokenList::insertTokens(Token *dest, const Token *src, nonneg int n)
 {
+    // TODO: put the linking in a helper?
     std::stack<Token *> link;
 
     while (n > 0) {
@@ -372,11 +375,11 @@ bool TokenList::createTokensInternal(std::istream &code, const std::string& file
 // NOLINTNEXTLINE(cppcoreguidelines-rvalue-reference-param-not-moved)
 void TokenList::createTokens(simplecpp::TokenList&& tokenList)
 {
-    if (tokenList.cfront()) {
+    // tokenList.cfront() might be NULL if the file contained nothing to tokenize so we need to check the files instead
+    if (!tokenList.getFiles().empty()) {
         // this is a copy
-        // TODO: the same as TokenList.files - move that instead
         // TODO: this points to mFiles when called from createTokens(std::istream &, const std::string&)
-        mOrigFiles = mFiles = tokenList.cfront()->location.files;
+        mOrigFiles = mFiles = tokenList.getFiles();
     }
     else
         mFiles.clear();
@@ -472,7 +475,7 @@ static Token* skipDecl(Token* tok, std::vector<Token*>* inner = nullptr)
                 vartok = vartok->link();
             else
                 return tok;
-        } else if (Token::Match(vartok, "%var% [:=(]")) {
+        } else if (Token::Match(vartok, "%var% [:=({]")) {
             return vartok;
         } else if (Token::Match(vartok, "decltype|typeof (") && !isDecltypeFuncParam(tok->linkAt(1))) {
             if (inner)
@@ -495,11 +498,11 @@ static bool iscast(const Token *tok, bool cpp)
     if (Token::Match(tok->link(), ") %assign%|,|..."))
         return false;
 
-    if (tok->previous() && tok->previous()->isName() && tok->previous()->str() != "return" &&
+    if (tok->previous() && tok->previous()->isName() && tok->strAt(-1) != "return" &&
         (!cpp || !Token::Match(tok->previous(), "delete|throw")))
         return false;
 
-    if (Token::simpleMatch(tok->previous(), ">") && tok->previous()->link())
+    if (Token::simpleMatch(tok->previous(), ">") && tok->linkAt(-1))
         return false;
 
     if (Token::Match(tok, "( (| typeof (") && Token::Match(tok->link(), ") %num%"))
@@ -546,7 +549,7 @@ static bool iscast(const Token *tok, bool cpp)
         if (!Token::Match(tok2, "%name%|*|::"))
             return false;
 
-        if (tok2->isStandardType() && (tok2->next()->str() != "(" || Token::Match(tok2->next(), "( * *| )")))
+        if (tok2->isStandardType() && (tok2->strAt(1) != "(" || Token::Match(tok2->next(), "( * *| )")))
             type = true;
     }
 
@@ -578,7 +581,7 @@ static Token * findCppTypeInitPar(Token *tok)
 }
 
 // X{} X<Y>{} etc
-static bool iscpp11init_impl(const Token * const tok);
+static bool iscpp11init_impl(const Token * tok);
 static bool iscpp11init(const Token * const tok)
 {
     if (tok->isCpp11init() == TokenImpl::Cpp11init::UNKNOWN)
@@ -802,7 +805,7 @@ static void compileTerm(Token *&tok, AST_state& state)
                     repeat = true;
                 }
                 if (Token::simpleMatch(tok->next(), "<") && Token::Match(tok->linkAt(1), "> %name%")) {
-                    tok = tok->next()->link()->next();
+                    tok = tok->linkAt(1)->next();
                     repeat = true;
                 }
             }
@@ -828,7 +831,7 @@ static void compileTerm(Token *&tok, AST_state& state)
         } else if ((state.cpp && iscpp11init(tok)) || Token::simpleMatch(tok->previous(), "] {")) {
             Token *const end = tok->link();
             if (state.op.empty() || Token::Match(tok->previous(), "[{,]") || Token::Match(tok->tokAt(-2), "%name% (")) {
-                if (Token::Match(tok, "{ . %name% =|{")) {
+                if (Token::Match(tok->tokAt(-1), "!!, { . %name% =|{")) {
                     const int inArrayAssignment = state.inArrayAssignment;
                     state.inArrayAssignment = 1;
                     compileBinOp(tok, state, compileExpression);
@@ -911,7 +914,7 @@ static bool isPrefixUnary(const Token* tok, bool cpp)
             && (tok->previous()->tokType() != Token::eIncDecOp || tok->tokType() == Token::eIncDecOp)))
         return true;
 
-    if (tok->previous()->str() == "}") {
+    if (tok->strAt(-1) == "}") {
         const Token* parent = tok->linkAt(-1)->tokAt(-1);
         return !Token::Match(parent, "%type%") || parent->isKeyword();
     }
@@ -958,7 +961,10 @@ static void compilePrecedence2(Token *&tok, AST_state& state)
                 tok = tok->tokAt(3);
                 break;
             }
-            compileBinOp(tok, state, compileScope);
+            if (!Token::Match(tok->tokAt(-1), "[{,]"))
+                compileBinOp(tok, state, compileScope);
+            else
+                compileUnaryOp(tok, state, compileScope);
         } else if (tok->str() == "[") {
             if (state.cpp && isPrefixUnary(tok, /*cpp*/ true) && Token::Match(tok->link(), "] (|{|<")) { // Lambda
                 // What we do here:
@@ -978,9 +984,9 @@ static void compilePrecedence2(Token *&tok, AST_state& state)
                 }
 
                 const bool hasTemplateArg = Token::simpleMatch(squareBracket->link(), "] <") &&
-                                            Token::simpleMatch(squareBracket->link()->next()->link(), "> (");
+                                            Token::simpleMatch(squareBracket->link()->linkAt(1), "> (");
                 if (Token::simpleMatch(squareBracket->link(), "] (") || hasTemplateArg) {
-                    Token* const roundBracket = hasTemplateArg ? squareBracket->link()->next()->link()->next() : squareBracket->link()->next();
+                    Token* const roundBracket = hasTemplateArg ? squareBracket->link()->linkAt(1)->next() : squareBracket->link()->next();
                     Token* curlyBracket = roundBracket->link()->next();
                     while (Token::Match(curlyBracket, "mutable|const|constexpr|consteval"))
                         curlyBracket = curlyBracket->next();
@@ -1024,11 +1030,17 @@ static void compilePrecedence2(Token *&tok, AST_state& state)
             else
                 compileUnaryOp(tok, state, compileExpression);
             tok = tok2->link()->next();
-        } else if (Token::simpleMatch(tok, "( {") && Token::simpleMatch(tok->linkAt(1)->previous(), "; } )") && !Token::Match(tok->previous(), "%name% (")) {
+        } else if (Token::simpleMatch(tok->previous(), "requires {")) {
+            state.op.push(tok);
+            tok = tok->link()->next();
+            continue;
+        } else if (Token::simpleMatch(tok, "( {") && Token::simpleMatch(tok->linkAt(1)->previous(), "; } )") &&
+                   !Token::Match(tok->previous(), "%name% (")) {
             state.op.push(tok->next());
             tok = tok->link()->next();
             continue;
-        } else if (tok->str() == "(" && (!iscast(tok, state.cpp) || Token::Match(tok->previous(), "if|while|for|switch|catch"))) {
+        } else if (tok->str() == "(" &&
+                   (!iscast(tok, state.cpp) || Token::Match(tok->previous(), "if|while|for|switch|catch"))) {
             Token* tok2 = tok;
             tok = tok->next();
             const bool opPrevTopSquare = !state.op.empty() && state.op.top() && state.op.top()->str() == "[";
@@ -1050,7 +1062,8 @@ static void compilePrecedence2(Token *&tok, AST_state& state)
             tok = tok->link()->next();
             if (Token::simpleMatch(tok, "::"))
                 compileBinOp(tok, state, compileTerm);
-        } else if (iscast(tok, state.cpp) && Token::simpleMatch(tok->link(), ") {") && Token::simpleMatch(tok->link()->linkAt(1), "} [")) {
+        } else if (iscast(tok, state.cpp) && Token::simpleMatch(tok->link(), ") {") &&
+                   Token::simpleMatch(tok->link()->linkAt(1), "} [")) {
             Token *cast = tok;
             tok = tok->link()->next();
             Token *tok1 = tok;
@@ -1072,7 +1085,8 @@ static void compilePrecedence2(Token *&tok, AST_state& state)
                 tok = end->next();
             else
                 throw InternalError(tok, "Syntax error. Unexpected tokens in initializer.", InternalError::AST);
-        } else break;
+        } else
+            break;
     }
 }
 
@@ -1587,7 +1601,7 @@ static Token * createAstAtToken(Token *tok)
             compileExpression(tok3, state1);
         }
         Token *init1 = nullptr;
-        Token * const endPar = tok->next()->link();
+        Token * const endPar = tok->linkAt(1);
         if (tok2 == tok->tokAt(2) && Token::Match(tok2, "%op%|(")) {
             init1 = tok2;
             AST_state state1(cpp);
@@ -1608,7 +1622,7 @@ static Token * createAstAtToken(Token *tok)
             while (tok2 && tok2 != endPar && tok2->str() != ";") {
                 if (tok2->str() == "<" && tok2->link()) {
                     tok2 = tok2->link();
-                } else if (Token::Match(tok2, "%name% )| %op%|(|[|.|:|::") || Token::Match(tok2->previous(), "[(;{}] %cop%|(")) {
+                } else if (Token::Match(tok2, "%name% )| %op%|(|[|{|.|:|::") || Token::Match(tok2->previous(), "[(;{}] %cop%|(")) {
                     init1 = tok2;
                     AST_state state1(cpp);
                     compileExpression(tok2, state1);
@@ -1872,7 +1886,7 @@ void TokenList::validateAst(bool print) const
         if (Token::Match(tok->previous(), "if|while|for|switch|assert|ASSERT (")) {
             if (!tok->astOperand1() || !tok->astOperand2())
                 throw InternalError(tok,
-                                    "Syntax Error: AST broken, '" + tok->previous()->str() +
+                                    "Syntax Error: AST broken, '" + tok->strAt(-1) +
                                     "' doesn't have two operands.",
                                     InternalError::AST);
         }
@@ -1885,7 +1899,7 @@ void TokenList::validateAst(bool print) const
             }
             if (!tok->next()->astOperand1() || !tok->next()->astOperand2()) {
                 const std::string& op =
-                    tok->next()->originalName().empty() ? tok->next()->str() : tok->next()->originalName();
+                    tok->next()->originalName().empty() ? tok->strAt(1) : tok->next()->originalName();
                 throw InternalError(
                           tok, "Syntax Error: AST broken, '" + op + "' doesn't have two operands.", InternalError::AST);
             }
@@ -1926,7 +1940,7 @@ void TokenList::simplifyPlatformTypes()
 
     const bool isCPP11 = isCPP() && (mSettings->standards.cpp >= Standards::CPP11);
 
-    enum { isLongLong, isLong, isInt } type;
+    enum : std::uint8_t { isLongLong, isLong, isInt } type;
 
     /** @todo This assumes a flat address space. Not true for segmented address space (FAR *). */
 
@@ -1959,7 +1973,7 @@ void TokenList::simplifyPlatformTypes()
         if (tok->str() == "::") {
             tok->deleteThis();
         } else if (tok->str() == "std") {
-            if (tok->next()->str() != "::")
+            if (tok->strAt(1) != "::")
                 continue;
             inStd = true;
             tok->deleteNext();
@@ -2180,10 +2194,13 @@ bool TokenList::isCPP() const
     return mLang == Standards::Language::CPP;
 }
 
-void TokenList::setLang(Standards::Language lang)
+void TokenList::setLang(Standards::Language lang, bool force)
 {
     ASSERT_LANG(lang != Standards::Language::None);
-    ASSERT_LANG(mLang == Standards::Language::None);
+    if (!force)
+    {
+        ASSERT_LANG(mLang == Standards::Language::None);
+    }
 
     mLang = lang;
 }
