@@ -28,6 +28,9 @@
 #ifdef __gnu_linux__
 #include <error.h>
 #endif
+#ifdef _GNU_SOURCE
+#include <unistd.h>
+#endif
 #include <getopt.h>
 #include <netdb.h>
 
@@ -41,6 +44,34 @@ void unreachableCode_error(void) // #11197
 }
 #endif
 
+#ifdef _GNU_SOURCE
+void leakReturnValNotUsed_get_current_dir_name(void)
+{
+    // cppcheck-suppress leakReturnValNotUsed
+    get_current_dir_name();
+}
+
+void memleak_get_current_dir_name0(void)
+{
+    const char *const name = get_current_dir_name();
+    if (name)
+    {
+        // cppcheck-suppress memleak
+        return;
+    }
+}
+
+void memleak_get_current_dir_name1(void)
+{
+    const char *const name = get_current_dir_name();
+    if (name)
+    {
+        free(name);
+        return;
+    }
+}
+#endif
+
 int nullPointer_gethostbyname2_r(const char* name, int af, struct hostent* ret, const char* buf, size_t buflen, struct hostent** result, const int* h_errnop)
 {
     // cppcheck-suppress nullPointer
