@@ -1046,6 +1046,26 @@ void memleak_getaddrinfo() // #6994
     // cppcheck-suppress memleak
 }
 
+void memleak_getaddrinfo_if() // #12506
+{
+    struct addrinfo hints = {};
+    struct addrinfo* addrs;
+    int err = getaddrinfo("example.com", "https", &hints, &addrs);
+    if (err != 0) {}
+    else {
+        freeaddrinfo(addrs);
+    }
+}
+
+void memleak_getaddrinfo_if2() // #12996
+{
+    struct addrinfo *addrs = NULL;
+    if (getaddrinfo("a", "b", NULL, &addrs)) {
+        return;
+    }
+    freeaddrinfo(addrs);
+}
+
 void memleak_mmap(int fd)
 {
     // cppcheck-suppress [unusedAllocatedMemory, unreadVariable, constVariablePointer]
