@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,8 @@
 
 #include "cppcheck.h"
 #include "errortypes.h"
+#include "helpers.h"
+#include "library.h"
 #include "options.h"
 #include "redirect.h"
 
@@ -39,12 +41,12 @@
  **/
 namespace {
     struct CompareFixtures {
-        bool operator()(const TestFixture* lhs, const TestFixture* rhs) const {
+        bool operator()(const TestInstance* lhs, const TestInstance* rhs) const {
             return lhs->classname < rhs->classname;
         }
     };
 }
-using TestSet = std::set<TestFixture*, CompareFixtures>;
+using TestSet = std::set<TestInstance*, CompareFixtures>;
 namespace {
     class TestRegistry {
         TestSet _tests;
@@ -55,7 +57,7 @@ namespace {
             return testreg;
         }
 
-        void addTest(TestFixture *t) {
+        void addTest(TestInstance *t) {
             _tests.insert(t);
         }
 
@@ -65,7 +67,11 @@ namespace {
     };
 }
 
-
+TestInstance::TestInstance(const char * _name)
+    : classname(_name)
+{
+    TestRegistry::theInstance().addTest(this);
+}
 
 
 /**
@@ -81,9 +87,7 @@ std::size_t TestFixture::succeeded_todos_counter = 0;
 
 TestFixture::TestFixture(const char * const _name)
     : classname(_name)
-{
-    TestRegistry::theInstance().addTest(this);
-}
+{}
 
 
 bool TestFixture::prepareTest(const char testname[])
@@ -106,7 +110,7 @@ bool TestFixture::prepareTest(const char testname[])
         } else {
             std::cout << classname << "::" << mTestname << std::endl;
         }
-        return true;
+        return !dry_run;
     }
     return false;
 }
@@ -115,14 +119,11 @@ void TestFixture::teardownTest()
 {
     teardownTestInternal();
 
-    // TODO: enable
-    /*
-        {
-        const std::string s = errout.str();
+    {
+        const std::string s = errout_str();
         if (!s.empty())
             throw std::runtime_error("unconsumed ErrorLogger err: " + s);
-        }
-     */
+    }
     {
         const std::string s = output_str();
         if (!s.empty())
@@ -215,9 +216,9 @@ std::string TestFixture::deleteLineNumber(const std::string &message)
     return result;
 }
 
-void TestFixture::assertEqualsWithoutLineNumbers(const char * const filename, const unsigned int linenr, const std::string &expected, const std::string &actual, const std::string &msg) const
+bool TestFixture::assertEqualsWithoutLineNumbers(const char * const filename, const unsigned int linenr, const std::string &expected, const std::string &actual, const std::string &msg) const
 {
-    assertEquals(filename, linenr, deleteLineNumber(expected), deleteLineNumber(actual), msg);
+    return assertEquals(filename, linenr, deleteLineNumber(expected), deleteLineNumber(actual), msg);
 }
 
 bool TestFixture::assertEquals(const char * const filename, const unsigned int linenr, const char expected[], const std::string& actual, const std::string &msg) const
@@ -236,20 +237,21 @@ bool TestFixture::assertEquals(const char * const filename, const unsigned int l
 bool TestFixture::assertEquals(const char * const filename, const unsigned int linenr, const long long expected, const long long actual, const std::string &msg) const
 {
     if (expected != actual) {
-        assertEquals(filename, linenr, std::to_string(expected), std::to_string(actual), msg);
+        return assertEquals(filename, linenr, std::to_string(expected), std::to_string(actual), msg);
     }
-    return expected == actual;
+    return true;
 }
 
-void TestFixture::assertEqualsDouble(const char * const filename, const unsigned int linenr, const double expected, const double actual, const double tolerance, const std::string &msg) const
+bool TestFixture::assertEqualsDouble(const char * const filename, const unsigned int linenr, const double expected, const double actual, const double tolerance, const std::string &msg) const
 {
     if (expected < (actual - tolerance) || expected > (actual + tolerance)) {
         std::ostringstream ostr1;
         ostr1 << expected;
         std::ostringstream ostr2;
         ostr2 << actual;
-        assertEquals(filename, linenr, ostr1.str(), ostr2.str(), msg);
+        return assertEquals(filename, linenr, ostr1.str(), ostr2.str(), msg);
     }
+    return false;
 }
 
 void TestFixture::todoAssertEquals(const char * const filename, const unsigned int linenr,
@@ -263,7 +265,7 @@ void TestFixture::todoAssertEquals(const char * const filename, const unsigned i
 
         ++succeeded_todos_counter;
     } else {
-        assertEquals(filename, linenr, current, actual);
+        (void)assertEquals(filename, linenr, current, actual);
         ++todos_counter;
     }
 }
@@ -301,8 +303,25 @@ void TestFixture::assertThrowFail(const char * const filename, const unsigned in
 void TestFixture::assertNoThrowFail(const char * const filename, const unsigned int linenr) const
 {
     ++fails_counter;
+
+    std::string ex_msg;
+
+    try {
+        // cppcheck-suppress rethrowNoCurrentException
+        throw;
+    }
+    catch (const InternalError& e) {
+        ex_msg = e.errorMessage;
+    }
+    catch (const std::exception& e) {
+        ex_msg = e.what();
+    }
+    catch (...) {
+        ex_msg = "unknown exception";
+    }
+
     errmsg << getLocationStr(filename, linenr) << ": Assertion failed. "
-           << "Unexpected exception was thrown"  << std::endl << "_____" << std::endl;
+           << "Unexpected exception was thrown: " << ex_msg << std::endl << "_____" << std::endl;
 
 }
 
@@ -323,7 +342,9 @@ void TestFixture::printHelp()
         "\n"
         "Options:\n"
         "    -q                   Do not print the test cases that have run.\n"
-        "    -h, --help           Print this help.\n";
+        "    -h, --help           Print this help.\n"
+        "    -n                   Print no summaries.\n"
+        "    -d                   Do not execute the tests.\n";
 }
 
 void TestFixture::run(const std::string &str)
@@ -355,6 +376,7 @@ void TestFixture::run(const std::string &str)
 void TestFixture::processOptions(const options& args)
 {
     quiet_tests = args.quiet();
+    dry_run = args.dry_run();
     exename = args.exe();
 }
 
@@ -370,23 +392,28 @@ std::size_t TestFixture::runTests(const options& args)
             classname.erase(classname.find("::"));
         }
 
-        for (TestFixture * test : TestRegistry::theInstance().tests()) {
+        for (TestInstance * test : TestRegistry::theInstance().tests()) {
             if (classname.empty() || test->classname == classname) {
-                test->processOptions(args);
-                test->run(testname);
+                TestFixture* fixture = test->create();
+                fixture->processOptions(args);
+                fixture->run(testname);
             }
         }
     }
 
-    std::cout << "\n\nTesting Complete\nNumber of tests: " << countTests << std::endl;
-    std::cout << "Number of todos: " << todos_counter;
-    if (succeeded_todos_counter > 0)
-        std::cout << " (" << succeeded_todos_counter << " succeeded)";
-    std::cout << std::endl;
+    if (args.summary() && !args.dry_run()) {
+        std::cout << "\n\nTesting Complete\nNumber of tests: " << countTests << std::endl;
+        std::cout << "Number of todos: " << todos_counter;
+        if (succeeded_todos_counter > 0)
+            std::cout << " (" << succeeded_todos_counter << " succeeded)";
+        std::cout << std::endl;
+    }
     // calling flush here, to do all output before the error messages (in case the output is buffered)
     std::cout.flush();
 
-    std::cerr << "Tests failed: " << fails_counter << std::endl << std::endl;
+    if (args.summary() && !args.dry_run()) {
+        std::cerr << "Tests failed: " << fails_counter << std::endl << std::endl;
+    }
     std::cerr << errmsg.str();
 
     std::cerr.flush();
@@ -402,10 +429,10 @@ void TestFixture::reportErr(const ErrorMessage &msg)
 {
     if (msg.severity == Severity::internal)
         return;
+    if (msg.severity == Severity::information && msg.id == "normalCheckLevelMaxBranches")
+        return;
     const std::string errormessage(msg.toString(mVerbose, mTemplateFormat, mTemplateLocation));
-    // TODO: remove the unique error handling?
-    if (errout.str().find(errormessage) == std::string::npos)
-        errout << errormessage << std::endl;
+    mErrout << errormessage << std::endl;
 }
 
 void TestFixture::setTemplateFormat(const std::string &templateFormat)
@@ -414,7 +441,7 @@ void TestFixture::setTemplateFormat(const std::string &templateFormat)
         mTemplateFormat = "{file}:{line}:{severity}:{message}";
         mTemplateLocation = "{file}:{line}:note:{info}";
     }
-    else if (templateFormat == "simple") {
+    else if (templateFormat == "simple") { // TODO: use the existing one in CmdLineParser
         mTemplateFormat = "{file}:{line}:{column}: {severity}:{inconclusive:inconclusive:} {message} [{id}]";
         mTemplateLocation = "";
     }
@@ -424,11 +451,17 @@ void TestFixture::setTemplateFormat(const std::string &templateFormat)
     }
 }
 
+TestFixture::SettingsBuilder& TestFixture::SettingsBuilder::checkLevel(Settings::CheckLevel level) {
+    settings.setCheckLevel(level);
+    return *this;
+}
+
 TestFixture::SettingsBuilder& TestFixture::SettingsBuilder::library(const char lib[]) {
     if (REDUNDANT_CHECK && std::find(settings.libraries.cbegin(), settings.libraries.cend(), lib) != settings.libraries.cend())
         throw std::runtime_error("redundant setting: libraries (" + std::string(lib) + ")");
     // TODO: exename is not yet set
-    LOAD_LIB_2_EXE(settings.library, lib, fixture.exename.c_str());
+    if (settings.library.load(fixture.exename.c_str(), lib).errorcode != Library::ErrorCode::OK)
+        throw std::runtime_error("library '" + std::string(lib) + "' not found");
     // strip extension
     std::string lib_s(lib);
     const std::string ext(".cfg");
@@ -456,9 +489,11 @@ TestFixture::SettingsBuilder& TestFixture::SettingsBuilder::platform(Platform::T
 TestFixture::SettingsBuilder& TestFixture::SettingsBuilder::libraryxml(const char xmldata[], std::size_t len)
 {
     tinyxml2::XMLDocument doc;
-    if (tinyxml2::XML_SUCCESS != doc.Parse(xmldata, len))
-        throw std::runtime_error("loading XML data failed");
-    if (settings.library.load(doc).errorcode != Library::ErrorCode::OK)
-        throw std::runtime_error("loading library XML failed");
+    const tinyxml2::XMLError xml_error = doc.Parse(xmldata, len);
+    if (tinyxml2::XML_SUCCESS != xml_error)
+        throw std::runtime_error(std::string("loading library XML data failed - ") + tinyxml2::XMLDocument::ErrorIDToName(xml_error));
+    const Library::ErrorCode lib_error = LibraryHelper::loadxmldoc(settings.library, doc).errorcode;
+    if (lib_error != Library::ErrorCode::OK)
+        throw std::runtime_error("loading library XML failed - " + std::to_string(static_cast<int>(lib_error)));
     return *this;
 }
