@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,11 +22,14 @@
 #include "filesettings.h"
 #include "fixture.h"
 #include "helpers.h"
+#include "settings.h"
+
+#include "simplecpp.h"
 
 #include <algorithm>
 #include <list>
 #include <string>
-
+#include <vector>
 
 class TestCppcheck : public TestFixture {
 public:
@@ -54,6 +57,9 @@ private:
         TEST_CASE(checkWithFS);
         TEST_CASE(suppress_error_library);
         TEST_CASE(unique_errors);
+        TEST_CASE(isPremiumCodingStandardId);
+        TEST_CASE(getDumpFileContentsRawTokens);
+        TEST_CASE(getDumpFileContentsLibrary);
     }
 
     void getErrorMessages() const {
@@ -97,7 +103,7 @@ private:
 
         ErrorLogger2 errorLogger;
         CppCheck cppcheck(errorLogger, false, {});
-        ASSERT_EQUALS(1, cppcheck.check(file.path()));
+        ASSERT_EQUALS(1, cppcheck.check(FileWithDetails(file.path())));
         // TODO: how to properly disable these warnings?
         errorLogger.ids.erase(std::remove_if(errorLogger.ids.begin(), errorLogger.ids.end(), [](const std::string& id) {
             return id == "logChecker";
@@ -117,8 +123,7 @@ private:
 
         ErrorLogger2 errorLogger;
         CppCheck cppcheck(errorLogger, false, {});
-        FileSettings fs;
-        fs.filename = file.path();
+        FileSettings fs{file.path()};
         ASSERT_EQUALS(1, cppcheck.check(fs));
         // TODO: how to properly disable these warnings?
         errorLogger.ids.erase(std::remove_if(errorLogger.ids.begin(), errorLogger.ids.end(), [](const std::string& id) {
@@ -142,7 +147,7 @@ private:
         const char xmldata[] = R"(<def format="2"><markup ext=".cpp" reporterrors="false"/></def>)";
         const Settings s = settingsBuilder().libraryxml(xmldata, sizeof(xmldata)).build();
         cppcheck.settings() = s;
-        ASSERT_EQUALS(0, cppcheck.check(file.path()));
+        ASSERT_EQUALS(0, cppcheck.check(FileWithDetails(file.path())));
         // TODO: how to properly disable these warnings?
         errorLogger.ids.erase(std::remove_if(errorLogger.ids.begin(), errorLogger.ids.end(), [](const std::string& id) {
             return id == "logChecker";
@@ -165,11 +170,11 @@ private:
 
         ErrorLogger2 errorLogger;
         CppCheck cppcheck(errorLogger, false, {});
-        ASSERT_EQUALS(1, cppcheck.check(test_file_a.path()));
-        ASSERT_EQUALS(1, cppcheck.check(test_file_b.path()));
+        ASSERT_EQUALS(1, cppcheck.check(FileWithDetails(test_file_a.path())));
+        ASSERT_EQUALS(1, cppcheck.check(FileWithDetails(test_file_b.path())));
         // TODO: how to properly disable these warnings?
-        errorLogger.errmsgs.erase(std::remove_if(errorLogger.errmsgs.begin(), errorLogger.errmsgs.end(), [](const ErrorMessage& errmsg) {
-            return errmsg.id == "logChecker";
+        errorLogger.errmsgs.erase(std::remove_if(errorLogger.errmsgs.begin(), errorLogger.errmsgs.end(), [](const ErrorMessage& msg) {
+            return msg.id == "logChecker";
         }), errorLogger.errmsgs.end());
         // the internal errorlist is cleared after each check() call
         ASSERT_EQUALS(2, errorLogger.errmsgs.size());
@@ -179,6 +184,57 @@ private:
         ASSERT_EQUALS("nullPointer", it->id);
     }
 
+    void isPremiumCodingStandardId() const {
+        ErrorLogger2 errorLogger;
+        CppCheck cppcheck(errorLogger, false, {});
+
+        cppcheck.settings().premiumArgs = "";
+        ASSERT_EQUALS(false, cppcheck.isPremiumCodingStandardId("misra-c2012-0.0"));
+        ASSERT_EQUALS(false, cppcheck.isPremiumCodingStandardId("misra-c2023-0.0"));
+        ASSERT_EQUALS(false, cppcheck.isPremiumCodingStandardId("premium-misra-c2012-0.0"));
+        ASSERT_EQUALS(false, cppcheck.isPremiumCodingStandardId("premium-misra-c2023-0.0"));
+        ASSERT_EQUALS(false, cppcheck.isPremiumCodingStandardId("premium-misra-c++2008-0.0.0"));
+        ASSERT_EQUALS(false, cppcheck.isPremiumCodingStandardId("premium-misra-c++2023-0.0.0"));
+        ASSERT_EQUALS(false, cppcheck.isPremiumCodingStandardId("premium-cert-int50-cpp"));
+        ASSERT_EQUALS(false, cppcheck.isPremiumCodingStandardId("premium-autosar-0-0-0"));
+
+        cppcheck.settings().premiumArgs = "--misra-c-2012 --cert-c++-2016 --autosar";
+        ASSERT_EQUALS(true, cppcheck.isPremiumCodingStandardId("misra-c2012-0.0"));
+        ASSERT_EQUALS(true, cppcheck.isPremiumCodingStandardId("misra-c2023-0.0"));
+        ASSERT_EQUALS(true, cppcheck.isPremiumCodingStandardId("premium-misra-c2012-0.0"));
+        ASSERT_EQUALS(true, cppcheck.isPremiumCodingStandardId("premium-misra-c2023-0.0"));
+        ASSERT_EQUALS(true, cppcheck.isPremiumCodingStandardId("premium-misra-c++2008-0.0.0"));
+        ASSERT_EQUALS(true, cppcheck.isPremiumCodingStandardId("premium-misra-c++2023-0.0.0"));
+        ASSERT_EQUALS(true, cppcheck.isPremiumCodingStandardId("premium-cert-int50-cpp"));
+        ASSERT_EQUALS(true, cppcheck.isPremiumCodingStandardId("premium-autosar-0-0-0"));
+    }
+
+    void getDumpFileContentsRawTokens() const {
+        ErrorLogger2 errorLogger;
+        CppCheck cppcheck(errorLogger, false, {});
+        cppcheck.settings() = settingsBuilder().build();
+        cppcheck.settings().relativePaths = true;
+        cppcheck.settings().basePaths.emplace_back("/some/path");
+        std::vector<std::string> files{"/some/path/test.cpp"};
+        simplecpp::TokenList tokens1(files);
+        const std::string expected = "  <rawtokens>\n"
+                                     "    <file index=\"0\" name=\"test.cpp\"/>\n"
+                                     "  </rawtokens>\n";
+        ASSERT_EQUALS(expected, cppcheck.getDumpFileContentsRawTokens(files, tokens1));
+    }
+
+    void getDumpFileContentsLibrary() const {
+        ErrorLogger2 errorLogger;
+        CppCheck cppcheck(errorLogger, false, {});
+        cppcheck.settings().libraries.emplace_back("std.cfg");
+        std::vector<std::string> files{ "/some/path/test.cpp" };
+        const std::string expected1 = "  <library lib=\"std.cfg\"/>\n";
+        ASSERT_EQUALS(expected1, cppcheck.getLibraryDumpData());
+        cppcheck.settings().libraries.emplace_back("posix.cfg");
+        const std::string expected2 = "  <library lib=\"std.cfg\"/>\n  <library lib=\"posix.cfg\"/>\n";
+        ASSERT_EQUALS(expected2, cppcheck.getLibraryDumpData());
+    }
+
     // TODO: test suppressions
     // TODO: test all with FS
 };
