@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2023 Cppcheck team.
+ * Copyright (C) 2007-2024 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,24 +17,22 @@
  */
 
 #include "errortypes.h"
+#include "fixture.h"
+#include "helpers.h"
 #include "library.h"
 #include "settings.h"
 #include "standards.h"
-#include "fixture.h"
-#include "helpers.h"
 #include "token.h"
-#include "tokenize.h"
 #include "tokenlist.h"
 
 #include <cstddef>
+#include <cstdint>
 #include <map>
-#include <sstream> // IWYU pragma: keep
+#include <sstream>
 #include <string>
 #include <unordered_map>
 #include <vector>
 
-#include "xml.h"
-
 #define ASSERT_EQ(expected, actual)   ASSERT(expected == actual)
 
 class TestLibrary : public TestFixture {
@@ -70,12 +68,7 @@ private:
         TEST_CASE(container);
         TEST_CASE(version);
         TEST_CASE(loadLibErrors);
-    }
-
-    static bool loadxmldata(Library &lib, const char xmldata[], std::size_t len)
-    {
-        tinyxml2::XMLDocument doc;
-        return (tinyxml2::XML_SUCCESS == doc.Parse(xmldata, len)) && (lib.load(doc).errorcode == Library::ErrorCode::OK);
+        TEST_CASE(loadLibCombinations);
     }
 
     void isCompliantValidationExpression() const {
@@ -104,8 +97,8 @@ private:
         // Reading an empty library file is considered to be OK
         constexpr char xmldata[] = "<?xml version=\"1.0\"?>\n<def/>";
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
-        ASSERT(library.functions.empty());
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(library.functions().empty());
     }
 
     void function() const {
@@ -116,15 +109,14 @@ private:
                                    "  </function>\n"
                                    "</def>";
 
-        TokenList tokenList(nullptr);
-        std::istringstream istr("foo();");
-        tokenList.createTokens(istr);
+        const char code[] = "foo();";
+        SimpleTokenList tokenList(code);
         tokenList.front()->next()->astOperand1(tokenList.front());
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
-        ASSERT_EQUALS(library.functions.size(), 1U);
-        ASSERT(library.functions.at("foo").argumentChecks.empty());
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT_EQUALS(library.functions().size(), 1U);
+        ASSERT(library.functions().at("foo").argumentChecks.empty());
         ASSERT(library.isnotnoreturn(tokenList.front()));
     }
 
@@ -137,18 +129,16 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
         {
-            TokenList tokenList(nullptr);
-            std::istringstream istr("fred.foo(123);"); // <- wrong scope, not library function
-            tokenList.createTokens(istr);
+            const char code[] = "fred.foo(123);"; // <- wrong scope, not library function
+            const SimpleTokenList tokenList(code);
 
             ASSERT(library.isNotLibraryFunction(tokenList.front()->tokAt(2)));
         }
         {
-            TokenList tokenList(nullptr);
-            std::istringstream istr("Fred::foo(123);"); // <- wrong scope, not library function
-            tokenList.createTokens(istr);
+            const char code[] = "Fred::foo(123);"; // <- wrong scope, not library function
+            const SimpleTokenList tokenList(code);
 
             ASSERT(library.isNotLibraryFunction(tokenList.front()->tokAt(2)));
         }
@@ -162,14 +152,14 @@ private:
                                    "  </function>\n"
                                    "</def>";
 
-        TokenList tokenList(nullptr);
+        TokenList tokenList(&settings);
         std::istringstream istr("foo();"); // <- too few arguments, not library function
-        tokenList.createTokens(istr);
+        ASSERT(tokenList.createTokens(istr, Standards::Language::CPP));
         Token::createMutualLinks(tokenList.front()->next(), tokenList.back()->previous());
         tokenList.createAst();
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
         ASSERT(library.isNotLibraryFunction(tokenList.front()));
     }
 
@@ -183,39 +173,39 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
 
         {
-            TokenList tokenList(nullptr);
+            TokenList tokenList(&settings);
             std::istringstream istr("foo();"); // <- too few arguments, not library function
-            tokenList.createTokens(istr);
+            ASSERT(tokenList.createTokens(istr, Standards::Language::CPP));
             Token::createMutualLinks(tokenList.front()->next(), tokenList.back()->previous());
             tokenList.createAst();
 
             ASSERT(library.isNotLibraryFunction(tokenList.front()));
         }
         {
-            TokenList tokenList(nullptr);
+            TokenList tokenList(&settings);
             std::istringstream istr("foo(a);"); // <- library function
-            tokenList.createTokens(istr);
+            ASSERT(tokenList.createTokens(istr, Standards::Language::CPP));
             Token::createMutualLinks(tokenList.front()->next(), tokenList.back()->previous());
             tokenList.createAst();
 
             ASSERT(!library.isNotLibraryFunction(tokenList.front()));
         }
         {
-            TokenList tokenList(nullptr);
+            TokenList tokenList(&settings);
             std::istringstream istr("foo(a, b);"); // <- library function
-            tokenList.createTokens(istr);
+            ASSERT(tokenList.createTokens(istr, Standards::Language::CPP));
             Token::createMutualLinks(tokenList.front()->next(), tokenList.back()->previous());
             tokenList.createAst();
 
             ASSERT(!library.isNotLibraryFunction(tokenList.front()));
         }
         {
-            TokenList tokenList(nullptr);
+            TokenList tokenList(&settings);
             std::istringstream istr("foo(a, b, c);"); // <- too much arguments, not library function
-            tokenList.createTokens(istr);
+            ASSERT(tokenList.createTokens(istr, Standards::Language::CPP));
             Token::createMutualLinks(tokenList.front()->next(), tokenList.back()->previous());
             tokenList.createAst();
 
@@ -231,14 +221,13 @@ private:
                                    "  </function>\n"
                                    "</def>";
 
-        TokenList tokenList(nullptr);
-        std::istringstream istr("Fred foo(123);"); // <- Variable declaration, not library function
-        tokenList.createTokens(istr);
+        const char code[] = "Fred foo(123);"; // <- Variable declaration, not library function
+        SimpleTokenList tokenList(code);
         tokenList.front()->next()->astOperand1(tokenList.front());
         tokenList.front()->next()->varId(1);
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
         ASSERT(library.isNotLibraryFunction(tokenList.front()->next()));
     }
 
@@ -255,14 +244,15 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
-        ASSERT_EQUALS(0, library.functions["foo"].argumentChecks[1].notuninit);
-        ASSERT_EQUALS(true, library.functions["foo"].argumentChecks[2].notnull);
-        ASSERT_EQUALS(true, library.functions["foo"].argumentChecks[3].formatstr);
-        ASSERT_EQUALS(true, library.functions["foo"].argumentChecks[4].strz);
-        ASSERT_EQUALS(false, library.functions["foo"].argumentChecks[4].optional);
-        ASSERT_EQUALS(true, library.functions["foo"].argumentChecks[5].notbool);
-        ASSERT_EQUALS(true, library.functions["foo"].argumentChecks[5].optional);
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
+        const auto& foo_fn_args = library.functions().at("foo").argumentChecks;
+        ASSERT_EQUALS(0, foo_fn_args.at(1).notuninit);
+        ASSERT_EQUALS(true, foo_fn_args.at(2).notnull);
+        ASSERT_EQUALS(true, foo_fn_args.at(3).formatstr);
+        ASSERT_EQUALS(true, foo_fn_args.at(4).strz);
+        ASSERT_EQUALS(false, foo_fn_args.at(4).optional);
+        ASSERT_EQUALS(true, foo_fn_args.at(5).notbool);
+        ASSERT_EQUALS(true, foo_fn_args.at(5).optional);
     }
 
     void function_arg_any() const {
@@ -274,8 +264,8 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
-        ASSERT_EQUALS(0, library.functions["foo"].argumentChecks[-1].notuninit);
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT_EQUALS(0, library.functions().at("foo").argumentChecks.at(-1).notuninit);
     }
 
     void function_arg_variadic() const {
@@ -288,12 +278,11 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
-        ASSERT_EQUALS(0, library.functions["foo"].argumentChecks[-1].notuninit);
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT_EQUALS(0, library.functions().at("foo").argumentChecks.at(-1).notuninit);
 
-        TokenList tokenList(nullptr);
-        std::istringstream istr("foo(a,b,c,d,e);");
-        tokenList.createTokens(istr);
+        const char code[] = "foo(a,b,c,d,e);";
+        SimpleTokenList tokenList(code);
         tokenList.front()->next()->astOperand1(tokenList.front());
 
         ASSERT_EQUALS(false, library.isuninitargbad(tokenList.front(), 1));
@@ -314,11 +303,10 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
 
-        TokenList tokenList(nullptr);
-        std::istringstream istr("foo(a,b,c,d);");
-        tokenList.createTokens(istr);
+        const char code[] = "foo(a,b,c,d);";
+        SimpleTokenList tokenList(code);
         tokenList.front()->next()->astOperand1(tokenList.front());
 
         ASSERT(Library::ArgumentChecks::Direction::DIR_IN == library.getArgDirection(tokenList.front(), 1));
@@ -346,11 +334,10 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
 
-        TokenList tokenList(nullptr);
-        std::istringstream istr("foo(a,b,c,d,e,f,g,h,i,j,k);");
-        tokenList.createTokens(istr);
+        const char code[] = "foo(a,b,c,d,e,f,g,h,i,j,k);";
+        SimpleTokenList tokenList(code);
         tokenList.front()->next()->astOperand1(tokenList.front());
 
         // 1-
@@ -488,11 +475,10 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
 
-        TokenList tokenList(nullptr);
-        std::istringstream istr("foo(a,b,c,d,e);");
-        tokenList.createTokens(istr);
+        const char code[] = "foo(a,b,c,d,e);";
+        SimpleTokenList tokenList(code);
         tokenList.front()->next()->astOperand1(tokenList.front());
 
         // arg1: type=strlen arg2
@@ -547,27 +533,25 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
-        ASSERT_EQUALS(library.functions.size(), 2U);
-        ASSERT(library.functions.at("Foo::foo").argumentChecks.empty());
-        ASSERT(library.functions.at("bar").argumentChecks.empty());
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT_EQUALS(library.functions().size(), 2U);
+        ASSERT(library.functions().at("Foo::foo").argumentChecks.empty());
+        ASSERT(library.functions().at("bar").argumentChecks.empty());
 
         {
-            TokenList tokenList(nullptr);
-            std::istringstream istr("Foo::foo();");
-            tokenList.createTokens(istr);
+            const char code[] = "Foo::foo();";
+            const SimpleTokenList tokenList(code);
             ASSERT(library.isnotnoreturn(tokenList.front()->tokAt(2)));
         }
 
         {
-            TokenList tokenList(nullptr);
-            std::istringstream istr("bar();");
-            tokenList.createTokens(istr);
+            const char code[] = "bar();";
+            const SimpleTokenList tokenList(code);
             ASSERT(library.isnotnoreturn(tokenList.front()));
         }
     }
 
-    void function_method() const {
+    void function_method() {
         constexpr char xmldata[] = "<?xml version=\"1.0\"?>\n"
                                    "<def>\n"
                                    "  <function name=\"CString::Format\">\n"
@@ -576,25 +560,25 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
-        ASSERT_EQUALS(library.functions.size(), 1U);
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT_EQUALS(library.functions().size(), 1U);
 
         {
-            Tokenizer tokenizer(&settings, nullptr);
-            std::istringstream istr("CString str; str.Format();");
-            ASSERT(tokenizer.tokenize(istr, "test.cpp"));
+            SimpleTokenizer tokenizer(settings, *this);
+            const char code[] = "CString str; str.Format();";
+            ASSERT(tokenizer.tokenize(code));
             ASSERT(library.isnotnoreturn(Token::findsimplematch(tokenizer.tokens(), "Format")));
         }
 
         {
-            Tokenizer tokenizer(&settings, nullptr);
-            std::istringstream istr("HardDrive hd; hd.Format();");
-            ASSERT(tokenizer.tokenize(istr, "test.cpp"));
+            SimpleTokenizer tokenizer(settings, *this);
+            const char code[] = "HardDrive hd; hd.Format();";
+            ASSERT(tokenizer.tokenize(code));
             ASSERT(!library.isnotnoreturn(Token::findsimplematch(tokenizer.tokens(), "Format")));
         }
     }
 
-    void function_baseClassMethod() const {
+    void function_baseClassMethod() {
         constexpr char xmldata[] = "<?xml version=\"1.0\"?>\n"
                                    "<def>\n"
                                    "  <function name=\"Base::f\">\n"
@@ -603,19 +587,19 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
 
         {
-            Tokenizer tokenizer(&settings, nullptr);
-            std::istringstream istr("struct X : public Base { void dostuff() { f(0); } };");
-            ASSERT(tokenizer.tokenize(istr, "test.cpp"));
+            SimpleTokenizer tokenizer(settings, *this);
+            const char code[] = "struct X : public Base { void dostuff() { f(0); } };";
+            ASSERT(tokenizer.tokenize(code));
             ASSERT(library.isnullargbad(Token::findsimplematch(tokenizer.tokens(), "f"),1));
         }
 
         {
-            Tokenizer tokenizer(&settings, nullptr);
-            std::istringstream istr("struct X : public Base { void dostuff() { f(1,2); } };");
-            ASSERT(tokenizer.tokenize(istr, "test.cpp"));
+            SimpleTokenizer tokenizer(settings, *this);
+            const char code[] = "struct X : public Base { void dostuff() { f(1,2); } };";
+            ASSERT(tokenizer.tokenize(code));
             ASSERT(!library.isnullargbad(Token::findsimplematch(tokenizer.tokens(), "f"),1));
         }
     }
@@ -632,25 +616,24 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
 
-        TokenList tokenList(nullptr);
-        std::istringstream istr("a(); b();");
-        tokenList.createTokens(istr);
+        const char code[] = "a(); b();";
+        const SimpleTokenList tokenList(code);
 
         const Library::WarnInfo* a = library.getWarnInfo(tokenList.front());
         const Library::WarnInfo* b = library.getWarnInfo(tokenList.front()->tokAt(4));
 
-        ASSERT_EQUALS(2, library.functionwarn.size());
+        ASSERT_EQUALS(2, library.functionwarn().size());
         ASSERT(a && b);
         if (a && b) {
             ASSERT_EQUALS("Message", a->message);
-            ASSERT_EQUALS(static_cast<int>(Severity::style), static_cast<int>(a->severity));
+            ASSERT_EQUALS_ENUM(Severity::style, a->severity);
             ASSERT_EQUALS(Standards::C99, a->standards.c);
             ASSERT_EQUALS(Standards::CPP03, a->standards.cpp);
 
             ASSERT_EQUALS("Obsolescent function 'b' called. It is recommended to use 'c', 'd' or 'e' instead.", b->message);
-            ASSERT_EQUALS(static_cast<int>(Severity::performance), static_cast<int>(b->severity));
+            ASSERT_EQUALS_ENUM(Severity::performance, b->severity);
             ASSERT_EQUALS(Standards::C89, b->standards.c);
             ASSERT_EQUALS(Standards::CPP11, b->standards.cpp);
         }
@@ -666,8 +649,8 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
-        ASSERT(library.functions.empty());
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(library.functions().empty());
 
         const Library::AllocFunc* af = library.getAllocFuncInfo("CreateX");
         ASSERT(af && af->arg == -1);
@@ -693,8 +676,8 @@ private:
                                     "</def>";
 
         Library library;
-        ASSERT_EQUALS(true, loadxmldata(library, xmldata1, sizeof(xmldata1)));
-        ASSERT_EQUALS(true, loadxmldata(library, xmldata2, sizeof(xmldata2)));
+        ASSERT_EQUALS(true, LibraryHelper::loadxmldata(library, xmldata1, sizeof(xmldata1)));
+        ASSERT_EQUALS(true, LibraryHelper::loadxmldata(library, xmldata2, sizeof(xmldata2)));
 
         ASSERT_EQUALS(library.deallocId("free"), library.allocId("malloc"));
         ASSERT_EQUALS(library.deallocId("free"), library.allocId("foo"));
@@ -709,8 +692,8 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
-        ASSERT(library.functions.empty());
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(library.functions().empty());
 
         const Library::AllocFunc* af = library.getAllocFuncInfo("CreateX");
         ASSERT(af && af->arg == 5 && !af->initData);
@@ -728,8 +711,8 @@ private:
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
-        ASSERT(library.functions.empty());
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(library.functions().empty());
 
         ASSERT(Library::isresource(library.allocId("CreateX")));
         ASSERT_EQUALS(library.allocId("CreateX"), library.deallocId("DeleteX"));
@@ -745,7 +728,7 @@ private:
                                        "  <podtype name=\"s16\" sign=\"s\" size=\"2\"/>\n"
                                        "</def>";
             Library library;
-            ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+            ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
             // s8
             {
                 const Library::PodType * const type = library.podtype("s8");
@@ -790,7 +773,7 @@ private:
         }
     }
 
-    void container() const {
+    void container() {
         constexpr char xmldata[] = "<?xml version=\"1.0\"?>\n"
                                    "<def>\n"
                                    "  <container id=\"A\" startPattern=\"std :: A &lt;\" endPattern=\"&gt; !!::\" itEndPattern=\"&gt; :: iterator\">\n"
@@ -821,14 +804,18 @@ private:
                                    "    <type string=\"std-like\"/>\n"
                                    "    <access indexOperator=\"array-like\"/>\n"
                                    "  </container>\n"
+                                   "  <container id=\"E\" startPattern=\"std :: E\"/>\n"
+                                   "  <container id=\"F\" startPattern=\"std :: F\" itEndPattern=\":: iterator\"/>\n"
                                    "</def>";
 
         Library library;
-        ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+        ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
 
-        Library::Container& A = library.containers["A"];
-        Library::Container& B = library.containers["B"];
-        const Library::Container& C = library.containers["C"];
+        const Library::Container& A = library.containers().at("A");
+        const Library::Container& B = library.containers().at("B");
+        const Library::Container& C = library.containers().at("C");
+        const Library::Container& E = library.containers().at("E");
+        const Library::Container& F = library.containers().at("F");
 
         ASSERT_EQUALS(A.type_templateArgNo, 1);
         ASSERT_EQUALS(A.size_templateArgNo, 4);
@@ -869,8 +856,16 @@ private:
         ASSERT_EQUALS(C.stdStringLike, true);
         ASSERT_EQUALS(C.arrayLike_indexOp, true);
 
+        ASSERT_EQUALS(E.startPattern, "std :: E");
+        ASSERT_EQUALS(E.endPattern, "");
+        ASSERT_EQUALS(E.itEndPattern, "");
+
+        ASSERT_EQUALS(F.startPattern, "std :: F");
+        ASSERT_EQUALS(F.endPattern, "");
+        ASSERT_EQUALS(F.itEndPattern, ":: iterator");
+
         {
-            givenACodeSampleToTokenize var("std::A<int> a;");
+            const SimpleTokenizer var(*this, "std::A<int> a;");
             ASSERT_EQUALS(&A, library.detectContainer(var.tokens()));
             ASSERT(!library.detectIterator(var.tokens()));
             bool isIterator;
@@ -879,14 +874,14 @@ private:
         }
 
         {
-            givenACodeSampleToTokenize var("std::A<int>::size_type a_s;");
+            const SimpleTokenizer var(*this, "std::A<int>::size_type a_s;");
             ASSERT(!library.detectContainer(var.tokens()));
             ASSERT(!library.detectIterator(var.tokens()));
             ASSERT(!library.detectContainerOrIterator(var.tokens()));
         }
 
         {
-            givenACodeSampleToTokenize var("std::A<int>::iterator a_it;");
+            const SimpleTokenizer var(*this, "std::A<int>::iterator a_it;");
             ASSERT(!library.detectContainer(var.tokens()));
             ASSERT_EQUALS(&A, library.detectIterator(var.tokens()));
             bool isIterator;
@@ -895,7 +890,7 @@ private:
         }
 
         {
-            givenACodeSampleToTokenize var("std::B<int> b;");
+            const SimpleTokenizer var(*this, "std::B<int> b;");
             ASSERT_EQUALS(&B, library.detectContainer(var.tokens()));
             ASSERT(!library.detectIterator(var.tokens()));
             bool isIterator;
@@ -904,14 +899,14 @@ private:
         }
 
         {
-            givenACodeSampleToTokenize var("std::B<int>::size_type b_s;");
+            const SimpleTokenizer var(*this, "std::B<int>::size_type b_s;");
             ASSERT(!library.detectContainer(var.tokens()));
             ASSERT(!library.detectIterator(var.tokens()));
             ASSERT(!library.detectContainerOrIterator(var.tokens()));
         }
 
         {
-            givenACodeSampleToTokenize var("std::B<int>::iterator b_it;");
+            const SimpleTokenizer var(*this, "std::B<int>::iterator b_it;");
             ASSERT(!library.detectContainer(var.tokens()));
             ASSERT_EQUALS(&B, library.detectIterator(var.tokens()));
             bool isIterator;
@@ -920,57 +915,115 @@ private:
         }
 
         {
-            givenACodeSampleToTokenize var("C c;");
+            const SimpleTokenizer var(*this, "C c;");
             ASSERT(!library.detectContainer(var.tokens()));
             ASSERT(!library.detectIterator(var.tokens()));
             ASSERT(!library.detectContainerOrIterator(var.tokens()));
         }
 
         {
-            givenACodeSampleToTokenize var("D d;");
+            const SimpleTokenizer var(*this, "D d;");
             ASSERT(!library.detectContainer(var.tokens()));
             ASSERT(!library.detectIterator(var.tokens()));
             ASSERT(!library.detectContainerOrIterator(var.tokens()));
         }
+
+        {
+            const SimpleTokenizer var(*this, "std::E e;");
+            ASSERT(library.detectContainer(var.tokens()));
+            ASSERT(!library.detectIterator(var.tokens()));
+            bool isIterator;
+            ASSERT_EQUALS(&E, library.detectContainerOrIterator(var.tokens(), &isIterator));
+            ASSERT(!isIterator);
+            ASSERT(!library.detectContainerOrIterator(var.tokens(), nullptr, true));
+        }
+
+        {
+            const SimpleTokenizer var(*this, "E e;");
+            ASSERT(!library.detectContainer(var.tokens()));
+            ASSERT(!library.detectIterator(var.tokens()));
+            ASSERT(!library.detectContainerOrIterator(var.tokens()));
+            ASSERT_EQUALS(&E, library.detectContainerOrIterator(var.tokens(), nullptr, true));
+        }
+
+        {
+            const SimpleTokenizer var(*this, "std::E::iterator I;");
+            ASSERT(!library.detectContainer(var.tokens()));
+            ASSERT(!library.detectIterator(var.tokens()));
+            ASSERT(!library.detectContainerOrIterator(var.tokens()));
+            ASSERT(!library.detectContainerOrIterator(var.tokens(), nullptr, true));
+        }
+
+        {
+            const SimpleTokenizer var(*this, "std::E::size_type p;");
+            ASSERT(!library.detectContainer(var.tokens()));
+            ASSERT(!library.detectIterator(var.tokens()));
+            ASSERT(!library.detectContainerOrIterator(var.tokens()));
+            ASSERT(!library.detectContainerOrIterator(var.tokens(), nullptr, true));
+        }
+
+        {
+            const SimpleTokenizer var(*this, "std::F f;");
+            ASSERT(library.detectContainer(var.tokens()));
+            ASSERT(!library.detectIterator(var.tokens()));
+            bool isIterator;
+            ASSERT_EQUALS(&F, library.detectContainerOrIterator(var.tokens(), &isIterator));
+            ASSERT(!isIterator);
+        }
+
+        {
+            const SimpleTokenizer var(*this, "std::F::iterator I;");
+            ASSERT(!library.detectContainer(var.tokens()));
+            TODO_ASSERT(library.detectIterator(var.tokens()));
+            bool isIterator = false;
+            TODO_ASSERT_EQUALS((intptr_t)&F, 0, (intptr_t)library.detectContainerOrIterator(var.tokens(), &isIterator));
+            TODO_ASSERT(isIterator);
+        }
+
+        {
+            const SimpleTokenizer var(*this, "F::iterator I;");
+            ASSERT(!library.detectContainer(var.tokens()));
+            ASSERT(!library.detectIterator(var.tokens()));
+            ASSERT(!library.detectContainerOrIterator(var.tokens()));
+            bool isIterator = false;
+            TODO_ASSERT_EQUALS((intptr_t)&F, 0, (intptr_t)library.detectContainerOrIterator(var.tokens(), &isIterator, true));
+            TODO_ASSERT(isIterator);
+        }
     }
 
+#define LOADLIBERROR(xmldata, errorcode) loadLibError(xmldata, errorcode, __FILE__, __LINE__)
+
     void version() const {
         {
             constexpr char xmldata[] = "<?xml version=\"1.0\"?>\n"
                                        "<def>\n"
                                        "</def>";
             Library library;
-            ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+            ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
         }
         {
             constexpr char xmldata[] = "<?xml version=\"1.0\"?>\n"
                                        "<def format=\"1\">\n"
                                        "</def>";
             Library library;
-            ASSERT(loadxmldata(library, xmldata, sizeof(xmldata)));
+            ASSERT(LibraryHelper::loadxmldata(library, xmldata, sizeof(xmldata)));
         }
         {
             constexpr char xmldata[] = "<?xml version=\"1.0\"?>\n"
                                        "<def format=\"42\">\n"
                                        "</def>";
-            Library library;
-            const Library::Error err = readLibrary(library, xmldata);
-            ASSERT_EQUALS(true, err.errorcode == Library::ErrorCode::UNSUPPORTED_FORMAT);
+            LOADLIBERROR(xmldata, Library::ErrorCode::UNSUPPORTED_FORMAT);
         }
     }
 
-    static Library::Error readLibrary(Library& library, const char* xmldata) {
-        tinyxml2::XMLDocument doc;
-        doc.Parse(xmldata); // TODO: check result
-        return library.load(doc);
-    }
-
-    void loadLibError(const char xmldata[], Library::ErrorCode errorcode, const char* file, unsigned line) const {
+    template<std::size_t size>
+    void loadLibError(const char (&xmldata)[size], Library::ErrorCode errorcode, const char* file, unsigned line) const {
         Library library;
-        assertEquals(file, line, true, errorcode == readLibrary(library, xmldata).errorcode);
+        Library::Error liberr;
+        (void)assertEquals(file, line, true, LibraryHelper::loadxmldata(library, liberr, xmldata, size-1));
+        (void)assertEquals(file, line, true, errorcode == liberr.errorcode);
     }
 
-#define LOADLIBERROR(xmldata, errorcode) loadLibError(xmldata, errorcode, __FILE__, __LINE__)
 #define LOADLIB_ERROR_INVALID_RANGE(valid) LOADLIBERROR("<?xml version=\"1.0\"?>\n" \
                                                         "<def>\n" \
                                                         "<function name=\"f\">\n" \
@@ -983,6 +1036,9 @@ private:
 
     void loadLibErrors() const {
 
+        LOADLIBERROR("<?xml version=\"1.0\"?>",
+                     Library::ErrorCode::BAD_XML);
+
         LOADLIBERROR("<?xml version=\"1.0\"?>\n"
                      "<def>\n"
                      "   <X name=\"uint8_t,std::uint8_t\" size=\"1\"/>\n"
@@ -1065,6 +1121,21 @@ private:
         // comma followed by dot
         LOADLIB_ERROR_INVALID_RANGE("-10:0,.5:");
     }
+
+    void loadLibCombinations() const {
+        {
+            const Settings s = settingsBuilder().library("std.cfg").library("gnu.cfg").library("bsd.cfg").build();
+            ASSERT_EQUALS(s.library.defines().empty(), false);
+        }
+        {
+            const Settings s = settingsBuilder().library("std.cfg").library("microsoft_sal.cfg").build();
+            ASSERT_EQUALS(s.library.defines().empty(), false);
+        }
+        {
+            const Settings s = settingsBuilder().library("std.cfg").library("windows.cfg").library("mfc.cfg").build();
+            ASSERT_EQUALS(s.library.defines().empty(), false);
+        }
+    }
 };
 
 REGISTER_TEST(TestLibrary)
