@@ -29,7 +29,6 @@
 #include "threadexecutor.h"
 #include "singleexecutor.h"
 
-#include <cstddef>
 #include <cstring>
 #include <list>
 #include <map>
@@ -229,13 +228,11 @@ private:
     unsigned int _checkSuppression(std::map<std::string, std::string> &f, bool useFS, const std::string &suppression = emptyString) {
         std::list<FileSettings> fileSettings;
 
-        std::list<std::pair<std::string, std::size_t>> filelist;
+        std::list<FileWithDetails> filelist;
         for (std::map<std::string, std::string>::const_iterator i = f.cbegin(); i != f.cend(); ++i) {
             filelist.emplace_back(i->first, i->second.size());
             if (useFS) {
-                FileSettings fs;
-                fs.filename = i->first;
-                fileSettings.emplace_back(std::move(fs));
+                fileSettings.emplace_back(i->first, i->second.size());
             }
         }
 
@@ -279,12 +276,10 @@ private:
     unsigned int _checkSuppressionThreads(const char code[], bool useFS, const std::string &suppression = emptyString) {
         std::list<FileSettings> fileSettings;
 
-        std::list<std::pair<std::string, std::size_t>> filelist;
+        std::list<FileWithDetails> filelist;
         filelist.emplace_back("test.cpp", strlen(code));
         if (useFS) {
-            FileSettings fs;
-            fs.filename = "test.cpp";
-            fileSettings.emplace_back(std::move(fs));
+            fileSettings.emplace_back("test.cpp", strlen(code));
         }
 
         /*const*/ auto settings = dinit(Settings,
@@ -298,8 +293,8 @@ private:
 
         std::vector<std::unique_ptr<ScopedFile>> scopedfiles;
         scopedfiles.reserve(filelist.size());
-        for (std::list<std::pair<std::string, std::size_t>>::const_iterator i = filelist.cbegin(); i != filelist.cend(); ++i)
-            scopedfiles.emplace_back(new ScopedFile(i->first, code));
+        for (std::list<FileWithDetails>::const_iterator i = filelist.cbegin(); i != filelist.cend(); ++i)
+            scopedfiles.emplace_back(new ScopedFile(i->path(), code));
 
         // clear files list so only fileSettings are used
         if (useFS)
@@ -325,12 +320,10 @@ private:
     unsigned int _checkSuppressionProcesses(const char code[], bool useFS, const std::string &suppression = emptyString) {
         std::list<FileSettings> fileSettings;
 
-        std::list<std::pair<std::string, std::size_t>> filelist;
+        std::list<FileWithDetails> filelist;
         filelist.emplace_back("test.cpp", strlen(code));
         if (useFS) {
-            FileSettings fs;
-            fs.filename = "test.cpp";
-            fileSettings.emplace_back(std::move(fs));
+            fileSettings.emplace_back("test.cpp", strlen(code));
         }
 
         /*const*/ auto settings = dinit(Settings,
@@ -344,8 +337,8 @@ private:
 
         std::vector<std::unique_ptr<ScopedFile>> scopedfiles;
         scopedfiles.reserve(filelist.size());
-        for (std::list<std::pair<std::string, std::size_t>>::const_iterator i = filelist.cbegin(); i != filelist.cend(); ++i)
-            scopedfiles.emplace_back(new ScopedFile(i->first, code));
+        for (std::list<FileWithDetails>::const_iterator i = filelist.cbegin(); i != filelist.cend(); ++i)
+            scopedfiles.emplace_back(new ScopedFile(i->path(), code));
 
         // clear files list so only fileSettings are used
         if (useFS)
@@ -974,6 +967,12 @@ private:
         SuppressionList suppressions6;
         ASSERT_EQUALS("", suppressions6.parseFile(file6));
         ASSERT_EQUALS(true, suppressions6.isSuppressed(errorMessage("abc", "test.cpp", 123)));
+
+        std::istringstream file7(" // comment\n" // #11450
+                                 "abc");
+        SuppressionList suppressions7;
+        ASSERT_EQUALS("", suppressions7.parseFile(file7));
+        ASSERT_EQUALS(true, suppressions7.isSuppressed(errorMessage("abc", "test.cpp", 123)));
     }
 
     void inlinesuppress() const {
@@ -1192,7 +1191,7 @@ private:
         settings.exitCode = 1;
 
         const char code[] = "int f() { int a; return a; }";
-        ASSERT_EQUALS(0, cppCheck.check("test.c", code)); // <- no unsuppressed error is seen
+        ASSERT_EQUALS(0, cppCheck.check(FileWithDetails("test.c"), code)); // <- no unsuppressed error is seen
         ASSERT_EQUALS("[test.c:1]: (error) Uninitialized variable: a\n", errout_str()); // <- report error so ThreadExecutor can suppress it and make sure the global suppression is matched.
     }
 
@@ -1201,18 +1200,18 @@ private:
         SuppressionList::Suppression suppression("unusedFunction", "test.c", 3);
         suppression.checked = true; // have to do this because fixes for #5704
         ASSERT_EQUALS("", suppressions.addSuppression(std::move(suppression)));
-        ASSERT_EQUALS(true, !suppressions.getUnmatchedLocalSuppressions("test.c", true).empty());
+        ASSERT_EQUALS(true, !suppressions.getUnmatchedLocalSuppressions(FileWithDetails("test.c"), true).empty());
         ASSERT_EQUALS(false, !suppressions.getUnmatchedGlobalSuppressions(true).empty());
-        ASSERT_EQUALS(false, !suppressions.getUnmatchedLocalSuppressions("test.c", false).empty());
+        ASSERT_EQUALS(false, !suppressions.getUnmatchedLocalSuppressions(FileWithDetails("test.c"), false).empty());
         ASSERT_EQUALS(false, !suppressions.getUnmatchedGlobalSuppressions(false).empty());
     }
 
     void globalsuppress_unusedFunction() const { // #4946 - wrong report of "unmatchedSuppression" for "unusedFunction"
         SuppressionList suppressions;
         ASSERT_EQUALS("", suppressions.addSuppressionLine("unusedFunction:*"));
-        ASSERT_EQUALS(false, !suppressions.getUnmatchedLocalSuppressions("test.c", true).empty());
+        ASSERT_EQUALS(false, !suppressions.getUnmatchedLocalSuppressions(FileWithDetails("test.c"), true).empty());
         ASSERT_EQUALS(true, !suppressions.getUnmatchedGlobalSuppressions(true).empty());
-        ASSERT_EQUALS(false, !suppressions.getUnmatchedLocalSuppressions("test.c", false).empty());
+        ASSERT_EQUALS(false, !suppressions.getUnmatchedLocalSuppressions(FileWithDetails("test.c"), false).empty());
         ASSERT_EQUALS(false, !suppressions.getUnmatchedGlobalSuppressions(false).empty());
     }
 
@@ -1232,7 +1231,7 @@ private:
             "    // cppcheck-suppress unusedStructMember\n"
             "    int y;\n"
             "};";
-        ASSERT_EQUALS(0, cppCheck.check("/somewhere/test.cpp", code));
+        ASSERT_EQUALS(0, cppCheck.check(FileWithDetails("/somewhere/test.cpp"), code));
         ASSERT_EQUALS("",errout_str());
     }
 
